<?php
namespace App\Controllers;
use App\Models\Address;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;

class AddressController
{
    public function searchPostcode(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'postcode' => $inputData['postcode'] ?? '',
            'limit' => $inputData['limit'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}
        
        if($params['limit'] == 0) {
            $responseData = [
				"status" => false, 
				"message" => "Invalid Limit",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
        }

        $address = new Address();
        $postcode = str_replace(' ', '', $params['postcode']);
        $limit = $params['limit'];
        $getPostcodeAddress = $address->getPostcodeAddress($postcode, $limit);

        $addressResponse = array();
        foreach ($getPostcodeAddress as $key => $address) {
            $addressResponse[$key] = [
                'postcode' => $address->postcode,
                'street'   => stripslashes($address->street),
                'town'     => stripslashes($address->town),
            ];
        }

        if(empty($addressResponse)) {
            $responseData = [
                "status" => false, 
                "data" => array(),
                "response_code" => 400,
            ];
        } else {
            $responseData = [
                "status" => true, 
                "data" => $addressResponse,
                "response_code" => 200,
            ];
        }
        
        $response->getBody()->write(json_encode($responseData));
        return $response;

    }

  
}