<?php

namespace APP\Controllers;

use App\Models\BookTable;
use App\Models\ClientHome;
use App\Models\DB;
use App\Models\Location;
use App\Models\ClientPayment;
use App\Helpers\SendEmail;
use App\Helpers\SendSms;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;

class BookTableController
{
    public function CheckTableAvaliablity(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);

        $domain = $inputData['domain'] ?? '';
        $params = [
            'domain' => (strpos($domain, 'www.') === 0 ?  substr($domain, 4) : $domain),
            'date' => $inputData['date'] ?? '',
            'time' => $inputData['time'] ?? '',
            'people' => $inputData['people'] ?? '',
        ];
        $checkAvailable['people'] = $params['people'];
        if (empty($params['domain'])) {
            $responseData = [
                "status" => false,
                "message" => 'Domain is Empty',
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        } else {
            $checkAvailable['domain'] = $params['domain'];
        }

        if (empty($params['time'])) {
            $responseData = [
                "status" => false,
                "message" => 'Booking Time is Empty',
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        } else {
            $time = addslashes($params['time']);
            $checkAvailable['time'] = date("h:i A", strtotime($time));
        }

        if (empty($params['date'])) {
            $responseData = [
                "status" => false,
                "message" => 'Booking Date is Empty',
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        } else {
            $checkAvailable['date'] = $params['date'];
        }
        $booktable = new BookTable($checkAvailable['domain']);
        $getOpenClose = $booktable->getOpenClose();
        $date  = str_replace("/", ".", $checkAvailable['date']);
        $datetime = $date . " " . $checkAvailable['time'];
        $datetime = str_replace("/", ".", $datetime);

        $timeWithoutAMPM = str_replace([' AM', ' PM'], '', $params['time']);
        $checkAvailable['time'] = date("H:i", strtotime($timeWithoutAMPM));
        $dateCheck = date("Y-m-d", strtotime($params['date'])); // Format the date as YYYY-MM-DD

        $dateTimeToCheck = $dateCheck . " " . $checkAvailable['time'];

        $bookingTimestamp = strtotime($params['date'] . ' ' . $params['time']);
        $currentTimestamp = time();

        if ($bookingTimestamp <= $currentTimestamp) {
            $responseData = [
                "status" => false,
                "message" => 'Please Choose Upcoming date/time',
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if (!empty($getOpenClose)) {
            if ($getOpenClose->opentime > strtotime($checkAvailable['date'])) {
                $responseData = [
                    "status" => false,
                    "message" => 'Shop Closed',
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }

        $getHolidays     = $booktable->getHoliday($checkAvailable['domain']);
        $getBankHolidays = $booktable->getBankHoliday($checkAvailable['domain']);
        $bookDateTime    = date("d/m/Y", strtotime($datetime));

        if ($getHolidays) {
            foreach ($getHolidays as $getHoliday) {
                $getDate = is_array($getHoliday) ? $getHoliday['date'] : $getHoliday->date;
                $holidayDate = date("d/m/Y",$getDate);
                if ($holidayDate == $bookDateTime) {
                    $responseData = [
                        "status" => false,
                        "message" => $getHoliday['desc'],
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }
        }

        if ($getHolidays) {
            foreach ($getBankHolidays as $getBankHoliday) {
                $holidayDate = date("d/m/Y", $getBankHoliday['date']);
                if ($holidayDate == $bookDateTime) {
                    $responseData = [
                        "status" => false,
                        "message" => $getBankHoliday['desc'],
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }
        }

        $checkAvailable['status'] =  1;
        $responseData = [
            "status" => true,
            "data" => $checkAvailable,
            "message" => 'Booking Available',
            "response_code" => 200,
        ];
        $connection_close=$booktable->connectionClose();
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function bookTableForNow(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);

        $params = [
            'domain' => $inputData['domain'] ?? '',
            'date' => $inputData['date'] ?? '',
            'time' => $inputData['time'] ?? '',
            'people' => $inputData['people'] ?? '',
            'firstname' => $inputData['firstname'] ?? '',
            'lastname' => $inputData['lastname'] ?? '',
            'phone' => $inputData['phone'] ?? '',
            'email' => $inputData['email'] ?? '',
            'status' => $inputData['status'] ?? '',
            'notes' => $inputData['notes'] ?? '',
            
        ];

        $message = '';

        if ($params['date'] == '' || $params['time'] == '' || $params['people'] == '') {
            $responseData = [
                "status" => false,
                "message" => 'Please Check the Avalibility Above',
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if ($params['domain'] == '' || $params['firstname'] == '' || $params['lastname'] == '' || $params['phone'] == '' ||  $params['email'] == '' || $params['status'] == '') {
            $responseData = [
                "status" => false,
                "message" => 'All Feilds are Requried',
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if (!filter_var($params['email'], FILTER_VALIDATE_EMAIL)) {
            $responseData = [
                "status" => false,
                "message" => 'Email is Not Valid',
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $timeWithoutAMPM = $params['time'];
        $checkAvailable['time'] = date("H:i", strtotime($timeWithoutAMPM));

        $time = addslashes($params['time']);
        $checkAvailable['domain'] = $params['domain'];
        $checkAvailable['date']   = $params['date'];
        $checkAvailable['people'] = $params['people'];
        $checkAvailable['firstname'] = $params['firstname'];
        $checkAvailable['lastname']  = $params['lastname'];
        $checkAvailable['phone'] = $params['phone'];
        $checkAvailable['email'] = $params['email'];
        $checkAvailable['notes'] = $params['notes'];

        $BookTable = new BookTable($checkAvailable['domain']);
        $getOpenClose = $BookTable->getOpenClose();
        $date  = str_replace("/", ".", $checkAvailable['date']);
        $datetime = $date . " " . $checkAvailable['time'];
        $datetime = str_replace("/", ".", $datetime);

        if (strtotime($datetime) < time()) {
            $responseData = [
                "status" => false,
                "message" => 'Please Choose Upcoming date/time',
                "response_code" => 400,
            ];
            $connection_close=$BookTable->connectionClose();
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if (!empty($getOpenClose)) {
            if ($getOpenClose->opentime > strtotime($checkAvailable['date'])) {
                $responseData = [
                    "status" => false,
                    "message" => 'Shop Closed',
                    "response_code" => 400,
                ];
                $connection_close=$BookTable->connectionClose();
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }

        $getHolidays     = $BookTable->getHoliday();
        $getBankHolidays = $BookTable->getBankHoliday();
        $bookDateTime    = date("d/m/Y", strtotime($datetime));

        if ($getHolidays) {
            foreach ($getHolidays as $getHoliday) {
                $holidayDate = date("d/m/Y", $getHoliday->date);
                if ($holidayDate == $bookDateTime) {
                    $responseData = [
                        "status" => false,
                        "message" => $getHoliday->desc,
                        "response_code" => 400,
                    ];
                    $connection_close=$BookTable->connectionClose();
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }
        }

        if ($getBankHolidays) {
            foreach ($getBankHolidays as $getBankHoliday) {
                $holidayDate = date("d/m/Y", $getBankHoliday->date);
                if ($holidayDate == $bookDateTime) {
                    $responseData = [
                        "status" => false,
                        "message" => $getBankHoliday->desc,
                        "response_code" => 400,
                    ];
                    $connection_close=$BookTable->connectionClose();
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }
        }

        $bookingData = $BookTable->checkBookingDeposite();
        if (!empty($bookingData)) {
            if ($bookingData->isActive == 1) {
                if (isset($inputData['payment'])) {
                    if ($inputData['payment'] != "paymentMethodId" && $inputData['payment'] != "paymentIntentId") {
                        $responseData = [
                            "status" => false,
                            "message" => 'Required payment type',
                            "response_code" => 400,
                        ];
                        $connection_close=$BookTable->connectionClose();
                        $response->getBody()->write(json_encode($responseData));
                        return $response;
                    }
                    if (!isset($inputData['total'])) {
                        $responseData = [
                            "status" => false,
                            "message" => 'Required total',
                            "response_code" => 400,
                        ];
                        $connection_close=$BookTable->connectionClose();
                        $response->getBody()->write(json_encode($responseData));
                        return $response;
                    }
                    $createPayment = $this->createPayment($inputData);
                    if ($createPayment['status'] == true) {
                        if (isset($createPayment['bookingId'])) {
                            $responseData = [
                                "status" => true,
                                "bookingId" => $createPayment['bookingId'],
                                "message" => $createPayment['response'],
                                "response_code" => 200,
                            ];
                        } else if (isset($createPayment['type'])) {
                            $responseData = [
                                "status" => true,
                                "type" => $createPayment['type'],
                                "message" => $createPayment['response'],
                                "response_code" => 200,
                            ];
                        } else {
                            $responseData = [
                                "status" => true,
                                "message" => $createPayment['response'],
                                "response_code" => 200,
                            ];
                        }
                    } else {
                        $responseData = [
                            "status" => false,
                            "message" => $createPayment['response'],
                            "response_code" => 400,
                        ];
                    }
                } else {
                    $responseData = [
                        "status" => true,
                        "message" => 'Booking Deposit is Active',
                        "deposit" => $bookingData,
                        "response_code" => 200,
                    ];
                }
                $connection_close=$BookTable->connectionClose();
                $response->getBody()->write(json_encode($responseData));
                return $response;
            } else {
                $placeBooking = $this->placeBooking($checkAvailable, false);
                if ($placeBooking['status'] == true) {
                    if (isset($placeBooking['bookingId'])) {
                        $responseData = [
                            "status" => true,
                            "bookingId" => $placeBooking['bookingId'],
                            "message" => $placeBooking['response'],
                            "BookDetail" => $placeBooking,
                            "response_code" => 200,
                        ];
                    } else {
                        $responseData = [
                            "status" => true,
                            "message" => $placeBooking['response'],
                            "response_code" => 200,
                        ];
                    }
                } else {
                    $responseData = [
                        "status" => false,
                        "message" => $placeBooking['response'],
                        "response_code" => 400,
                    ];
                }
                $connection_close=$BookTable->connectionClose();
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        } else {
            $responseData = [
                "status" => false,
                "message" => "Booking Deposit is Empty",
                "response_code" => 400,
            ];
            $connection_close=$BookTable->connectionClose();
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function createPayment($inputData)
    {
        $home = new ClientHome($inputData['domain']);
        $payment = new ClientPayment();

        $getContacts = $home->getContacts();
        $stripe = $payment->getStripeConnection($inputData['domain']);

        if ($inputData['email'])
            $description = $inputData['firstname'] . " " . $inputData['lastname'] . " " . $inputData['email'];
        else
            $description = $inputData['firstname'] . " " . $inputData['lastname'];

        $securityKey = $stripe['securityKey'];
        $connectId = $stripe['connectId'];

        # create customer account with account
        $transferCalculation = $inputData['total'] * $stripe['stripeCharge'] / 100 + $stripe['stripeAdd'];
        $transferCalculation= number_format($transferCalculation, 2);
        $transferCalculationString = str_replace(".", "", $transferCalculation);
        $totalAmountString = str_replace(".", "", $inputData['total']);
        $clientTransferAmountString = $totalAmountString - $transferCalculationString;

        if (isset($inputData['paymentMethodId'])) {
            $createStripeCustomer = $payment->createStripeCustomerWithCard($securityKey, $description, $inputData['email'], $inputData['paymentMethodId']);

            # create payment intents
            $paymentIntent = $payment->createPaymentIntentWithCard($securityKey, $connectId, $inputData['domain'], $getContacts->name, $totalAmountString, $clientTransferAmountString, $description, $inputData['paymentMethodId'], $createStripeCustomer);
            if ($paymentIntent['status'] == false) {
                $data = [
                    "status" => false,
                    "response" => $paymentIntent['data'],
                ];
                return $data;
            } else {
                $paymentIntent = $paymentIntent['data'];
            }

            if ($paymentIntent['status']) {
                if ($paymentIntent['status'] == 'requires_action' && $paymentIntent['next_action']['type'] == 'use_stripe_sdk') {
                    $secureResponse['id'] = $paymentIntent['id'];
                    $secureResponse['clientSecret'] = $paymentIntent['client_secret'];
                    $data = [
                        "status" => true,
                        "type" => "3dSecure",
                        "response" => $secureResponse
                    ];
                    return $data;
                } else if ($paymentIntent['status'] == 'succeeded' && $paymentIntent['charges']['data'][0]['status'] == 'succeeded') {
                    $createPaymentIntent = $this->createCommonPaymentIntent($payment, $inputData, $securityKey, $paymentIntent['id'], $description, $connectId);
                    return $createPaymentIntent;
                } else {
                    $data = array(
                        "status" => false,
                        "response" => $paymentIntent
                    );
                    return $data;
                }
            } else {
                $data = array(
                    "status" => false,
                    "response" => $paymentIntent
                );
                return $data;
            }
        } else if (isset($inputData['paymentIntentId'])) {
            $createPaymentIntent = $this->createCommonPaymentIntent($payment, $inputData, $securityKey, $inputData['paymentIntentId'], $description, $connectId);
            return $createPaymentIntent;
        }
    }

    public function createCommonPaymentIntent($payment, $inputData, $securityKey, $paymentIntentId, $description, $connectId)
    {
        # retrieve payment intent
        $paymentIntent = $payment->retrievePaymentIntent($securityKey, $paymentIntentId);
        # confirm the payment intent
        if ($paymentIntent['status'] != "succeeded") {
            $paymentIntent = $payment->confirmPaymentIntent($securityKey, $paymentIntentId);
        }

        $chargesRetrieve = $payment->chargesRetrieve($securityKey, $paymentIntent['latest_charge']);

        if ($chargesRetrieve['status'] == false) {
            $data = array(
                "status" => false,
                "response" => $chargesRetrieve['data']
            );
            return $data;
        }

        $chargesRetrieve = $chargesRetrieve['data'];

        if(isset($chargesRetrieve->transfer) && !empty($chargesRetrieve->transfer)) {
            $transfer = $payment->transferRetrieve($securityKey, $chargesRetrieve->transfer);
            
            if(isset($transfer->destination) && isset($transfer->destination_payment)) {
                # create customer account with account
                $createStripeCustomer = $payment->createStripeCustomerWithAccount($securityKey, $description, $inputData['email'], $connectId);
                # update charges
                $charge = $payment->chargesUpdate($securityKey, $transfer->destination_payment, $description, $createStripeCustomer->id, $connectId);

                if ($charge['status'] == false) {
                    $data = array(
                        "status" => false,
                        "response" => $charge['data']
                    );
                    return $data;
                }
            }

            # card refno data
            $refno = json_encode(
                array(
                    'id' => $paymentIntent['id'],
                    'transfer' => $chargesRetrieve->transfer,
                    'transfer_group' => $chargesRetrieve->transfer_group,
                    'transfer_status' => $paymentIntent['status'],
                    '3d_secure' => true
                )
            );

            # placed booking functionalities
            $placeBooking = $this->placeBooking($inputData, $refno);
            return $placeBooking;
        } else {
            $data = array(
                "status" => false,
                "response" => "Transfer key not available"
            );
            return $data;
        }
    }

    public function placeBooking($checkAvailable, $refno = false)
    {
        $BookTable = new BookTable($checkAvailable['domain']);
        $home = new ClientHome($checkAvailable['domain']);
        $customer_id = "";
        if (isset($checkAvailable['domain']) && isset($checkAvailable['email'])) {
            $checkUserWithEmail = $BookTable->checkUserWithEmail($checkAvailable['domain'], $checkAvailable['email']);
            if (isset($checkUserWithEmail->id)) {
                $customer_id = $checkUserWithEmail->id;
                // echo $customer_id;
            } else {
                $typeOfLogin = 0;
                $verficationCode = "";
                if(isset($checkAvailable['firstname']) && isset($checkAvailable['lastname']))
                {
                    $lastInsertId = $BookTable->createCustomerByEmail($checkAvailable['email'], $checkAvailable['firstname'], $checkAvailable['lastname']);
                    if (isset($lastInsertId->insert_id)) {
                        $BookTable->createUserByEmail($checkAvailable['email'], $lastInsertId->insert_id, $typeOfLogin, $verficationCode);
                        // echo "here";
                        $customer_id = $lastInsertId->insert_id;
                    }
                }
                
                // echo "working" . $customer_id;
            }
        }
        $checkAvailable['customer_id'] = $customer_id;
        
        $addReservation = $BookTable->addReservationWithCustomerId($checkAvailable, 0);
        //$addReservation = $BookTable->addReservation($checkAvailable, 0);
        if (!empty($addReservation->inserted_id)) {
            if ($refno != false) {
                $BookTable->createBookingDeposit($checkAvailable['domain'], $addReservation->inserted_id, $refno);
            }

            // currency info
            //$location = new Location($checkAvailable['domain']);
            $currency = $home->getLocation();
            $currency_code = $currency['code'];
            $currency_symbol = $currency['symbol'];

            $BookTable->addBookUser($checkAvailable);
            $getContacts = $home->getContacts($checkAvailable['domain']);
            $checkAvailable['book_id'] = $addReservation->inserted_id;
            $checkAvailable['amount']  = 0;
            $checkAvailable['currency_code']  = $currency_code;
            $checkAvailable['currency_symbol']  = $currency_symbol;
            $checkAvailable['takeaway_name'] = $getContacts->name;
            $checkAvailable['logo'] = $getContacts->image;
            $checkAvailable['client_phone'] = $getContacts->phone;

            $sentMail = new SendEmail();
            $sms = new SendSms();
            $email = $sentMail->sendreservation($checkAvailable);
            $getsmsSetting = $home->getsmsSetting($checkAvailable['domain']);
            $notificationNumb = $home->getnotification($checkAvailable['domain']);

            if (strpos($getsmsSetting->control1, '1') !== false) {

                $clientPhoneNumber = !empty($notificationNumb->reserv_number) ? ($notificationNumb->reserv_number) :  '';

                //sent to owner
                if (!empty($clientPhoneNumber)) {
                    $ownerMsg = "Hi ". $checkAvailable['takeaway_name']." Table Booked Details, Booking ID: " . $checkAvailable['book_id'] . ", Name: " . $checkAvailable['firstname'] . " " . $checkAvailable['lastname'] . ", Email: " . $checkAvailable['email'] . ", Phone: " . $checkAvailable['phone'] . ", Date: " . $checkAvailable['date'] . ", Time: " . $checkAvailable['time'] . ", People: " . $checkAvailable['people'];
                    $sms->sendSmsClient($clientPhoneNumber, $ownerMsg);
                }

                // sent to customer
                if (!empty($checkAvailable['phone'])) {
                    $customerMsg  = 'Hi ' . $checkAvailable['firstname'] . ' ' . $checkAvailable['lastname'] . ', Your table booking with ' . $checkAvailable['takeaway_name'] . ' is pending confirmation: Booking ID: ' . $checkAvailable['book_id'] . ', Date & Time: ' . $checkAvailable['date'] . ', ' . $checkAvailable['time'] . '. Confirmation will be sent soon. For any changes or cancellations, contact us at ' . $checkAvailable['client_phone'] . '. Thanks, ' . $checkAvailable['takeaway_name'];
                    $smsfrom = (array)$getsmsSetting;
                    $from = $smsfrom['from'];
                    $sender = !empty($from) ? $from :  $checkAvailable['takeaway_name'];
                    $sms->sendSmsClient($checkAvailable['phone'], $customerMsg,$sender);
                }

                $sentToBoss = $sentMail->sentReservationToBoss($checkAvailable);
                if ($sentToBoss == false) {
                    $data = array(
                        "status" => false,
                        "response" => "Owner email ID is not Available",
                    );
                    return $data;
                } else {
                    $DB=new DB();
                    $get_client_id=$DB->getClientDetailsByDomain($checkAvailable['domain']); 

                    if(isset($get_client_id->client_id))
                    {
                        $send_notificatiob_url = "https://api-newdev.fmb.eposapi.co.uk/sendPushNotification";
                        if ($_SERVER['SERVER_NAME'] == "localhost") {
                            $send_notificatiob_url = "https://api-newdev.fmb.eposapi.co.uk/sendPushNotification";
                        } else if ($_SERVER['SERVER_NAME'] == "apidev-landing.fusionkitchen.co.uk") {
                            $send_notificatiob_url = "https://api-newdev.fmb.eposapi.co.uk/sendPushNotification";
                        } else {
                            $send_notificatiob_url = "https://api-new.fmb.eposapi.co.uk/sendPushNotification";
                        }

                        $fcm_push_array = array(
                            "type" => "table_booking",
                            "client_id" => $get_client_id->client_id,
                            "customer_name"=>$addReservation->inserted_id,
                        );
                        $curl = curl_init();

                            curl_setopt_array($curl, array(
                                CURLOPT_URL => $send_notificatiob_url,
                                CURLOPT_RETURNTRANSFER => true,
                                CURLOPT_ENCODING => '',
                                CURLOPT_MAXREDIRS => 10,
                                CURLOPT_TIMEOUT => 0,
                                CURLOPT_FOLLOWLOCATION => true,
                                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                                CURLOPT_CUSTOMREQUEST => 'POST',
                                CURLOPT_POSTFIELDS => json_encode($fcm_push_array),
                                CURLOPT_HTTPHEADER => array(
                                    'Content-Type: application/json'
                                ),
                            ));

                            $response = curl_exec($curl);
                            // echo $response;
                            curl_close($curl);
                    }

                    $booking = [
                        "bookingId" => $addReservation->inserted_id,
                        'type' => 'Table Booking',
                        'client_domain' => $checkAvailable['domain']
                    ];
                    $isWebSocketTrigger = '';
                    $jsonData = json_encode($booking);
                    $url = "https://fusionbooking.co.uk/order";
                    $post = curl_init();
                    curl_setopt($post, CURLOPT_URL, $url);
                    curl_setopt($post, CURLOPT_SSL_VERIFYPEER, false);
                    curl_setopt($post, CURLOPT_SSL_VERIFYHOST, false);
                    curl_setopt($post, CURLOPT_POST, true);
                    curl_setopt($post, CURLOPT_POSTFIELDS, $jsonData);
                    curl_setopt($post, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($post, CURLOPT_TIMEOUT, 10); // Set timeout to 10 seconds
                    curl_setopt($post, CURLOPT_HTTPHEADER, [
                        'Content-Type: application/json',
                        'Content-Length: ' . strlen($jsonData)
                    ]);
                    $result = curl_exec($post);
                    if ($result === false) {
                        $error_msg = curl_error($post);
                    } else {
                        $isWebSocketTrigger = "Success!";
                    }
                    curl_close($post);
                   
                    $data = array(
                        "status" => true,
                        "response" => "Table Booked",
                        "bookingId" => $addReservation->inserted_id,
                        "isWebSocketTrigger" => $isWebSocketTrigger
                    );
                    return $data;
                }
            } else {
                $data = array(
                    "status" => false,
                    "response" => "Email Control is Disabled",
                );
                return $data;
            }
        } else {
            $data = array(
                "status" => false,
                "response" => "Reservation Failed",
            );
            return $data;
        }
    }

    public function getTableTiming(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $errorMessage = "";
        $successMessage = "";
        $params = [
            'domain' => $inputData['domain'] ?? '',
            'date' => $inputData['date'] ?? '',
        ];

        # required fields error
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (empty($errors)) {
            $date = strtotime($params['date']);
            $day = date('w', $date);
            if ($day == 0) {
                $times = "times7";
                $openClose = "openclose7";
            } else {
                $times = "times" . $day;
                $openClose = "openclose" . $day;
            }

            $booktable = new BookTable($params['domain']);
            $getRestTiming = $booktable->getRestTiming();
            if ($getRestTiming->$openClose == "open" || $getRestTiming->$openClose == "") {
                $getTableAvailability = $booktable->getTableAvailability($params['domain'], $params['date']);
                if ($getTableAvailability != false) {
                    $successMessage = $getTableAvailability->times;
                } else {
                    $successMessage = str_replace("\r\n", "", $getRestTiming->$times);
                }
            } else {
                $successMessage = "";
            }
        } else {
            $errorMessage = implode('. ', $errors);
        }

        # final error / success message
        if ($errorMessage == "") {
            $responseData = [
                "status" => true,
                "message" => $successMessage,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false,
                "message" => $errorMessage,
                "response_code" => 400,
            ];
        }
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }
    

}
