<?php

namespace APP\Controllers;

use App\Models\ClientHome;
use App\Models\ClientLocation;
use App\Models\ClientSeo;
use App\Models\DB;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;

require '../config/env.php';
class ClientSeoController
{
    public function sitemap(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $errorMessage = "";
        $successMessage = [];
        $params = [
            'domain' => $inputData['domain'] ?? '',
        ];

        # required fields error
        $errors = [];
        if ($params != "") {
            foreach ($params as $key => $value) {
                if (empty($value)) {
                    $errors[] = ucfirst($key) . ' is required';
                }
            }
        } else {
            $errors[] = 'request is required';
        }


        if ($errorMessage == "") {
            $currentDateTime = time(); // Get the current Unix timestamp
            $yesterdayDateTime = $currentDateTime - 86400; // 86400 seconds = 1 day
            $yesterdayFormatted = date('Y-m-d\TH:i:sP', $yesterdayDateTime);

            $home = new ClientHome($params['domain']);
            $getClientDetails = $home->getCidAndPath($params['domain']);
            $businessType = !empty($getClientDetails['bkenable']) ? $getClientDetails['bkenable'] : null;
            $menuPath = !empty($getClientDetails['client_menu_path']) ? $getClientDetails['client_menu_path'] : null;

            $sitemap = ["", "allergy", "privacy", "terms", "review", "rate"];
            // if ($businessType == 1) {
            //     array_push($sitemap, "book");
            // }

            if ($menuPath != "") {
                array_push($sitemap, $menuPath . "/menu");
            }

            foreach ($sitemap as $data) {
                $sitemapArray = [
                    "loc" => "https://" . $params["domain"] . "/" . $data,
                    "lastmod" => $yesterdayFormatted,
                    "priority" => $data == "" ? "1.00" : "0.80",
                ];
                array_push($successMessage, $sitemapArray);
            }
        } else {
            $errorMessage = implode('. ', $errors);
        }

        # final error / success message
        if ($errorMessage == "") {
            $responseData = [
                "status" => true,
                "message" => $successMessage,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false,
                "message" => $errorMessage,
                "response_code" => 400,
            ];
        }
        $connection_close=$home->connectionClose();
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getSeoData(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $errorMessage = "";
        $successMessage = [];
        $params = [
            'domain' => $inputData['domain'] ?? '',
        ];

        # required fields error
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if ($errorMessage == "") {

            $seo = new ClientSeo($params['domain']);
            
            $getSeoData = $seo->getSeoData();
            $getMeta = $seo->getMeta();
            $getPageTitle = $seo->getPageTitle();
            
            $successMessage['og_title'] = $getSeoData->og_title ?? "";
            $successMessage['og_description'] = $getSeoData->og_description ?? "";
           
            $db = new DB();
            $cdetail = $db->getClientDetailsByDomain($params['domain']);
            $url = "https://fusioneposfiles.s3.eu-west-2.amazonaws.com/BannerImages/$cdetail->client_id/";
            $getSEOOGDetails = $seo->getSeoOGData($url);
            $successMessage['seo_og_details'] = $getSEOOGDetails;
           
            if(!empty($getMeta)){
                foreach ($getMeta as $key => $data) {
                    if (isset($data['page_type'])) {
                        if (isset($successMessage['meta'][$data['page_type']])) {
                            $successMessage['meta'][$data['page_type']][] = $data;
                        } else {
                            $successMessage['meta'][$data['page_type']] = [$data];
                        }
                    } else {
                        $successMessage['meta']["Home"][] = $data;
                    }
                }
            }
            
            $title = ["home", "about", "menu", "reservation", "contact", "review", "feedback", "allergy", "privacy", "terms"];
            foreach ($getPageTitle as $key => $data) {
                if (in_array($data['name'], $title)) {
                    $successMessage['title'][$data['name']] = $data['title'];
                }
            }
        } else {
            $errorMessage = implode('. ', $errors);
        }

        # final error / success message
        if ($errorMessage == "") {
            $responseData = [
                "status" => true,
                "message" => $successMessage,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false,
                "message" => $errorMessage,
                "response_code" => 400,
            ];
        }
        $connection_close=$seo->connectionClose();
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }
}