<?php
namespace App\Controllers;
use App\Models\Home;
use App\Models\DB;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;

class HomeController
{
    public function index(Request $request, Response $response, $args)
    {
        $home = new Home();
        $data = $home->getHomePageDetails();
        $responseData = [
            "status" => true, 
            "data" => $data,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;

    }

     public function websocketTrigger(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
      
        if (isset($inputData['ord_id']) && isset($inputData['type'])) {
            $home = new Home();
            if(!empty($inputData['client_domain'])){
                $brId = $home->getBrID($inputData['client_domain'],'domain'); 
            }else{
                $brId = $home->getBrID($inputData['client_id'],'client'); 
            }
             
            $url = "https://fusionbooking.co.uk/order";
            $data = array(
                'ord_id' => $inputData['ord_id'],
                'type' => $inputData['type'],
                'br_id' => $brId
            );
            $jsonData = json_encode($data);
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); 
            curl_setopt($ch, CURLOPT_POST, true);     
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',           
                'Content-Length: ' . strlen($jsonData)
            ));
            curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
            $result = curl_exec($ch);
            if (curl_errno($ch)) {
                echo 'cURL error: ' . curl_error($ch);
            } else {
                // echo 'Server response: ' . $result;
            }
            curl_close($ch);
        } else {
            $responseData = [
                "status" => true,
                "message" => "Required fields are missing!",
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        $responseData = [
            "status" => true,
            "message" => "Websocket Trigger send successfully!",
            "data" => json_decode($result,true),
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getAppDownloadLink(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        if(isset($inputData['phone_number'])&& $inputData['phone_number'] != "")
        {
            $home = new Home();
            $phoneNumber = $inputData['phone_number'];
            $message = "Here's the link to download the Fusion Kitchen app. Order what you love! http://onelink.to/fusion-sms";

            if(substr($phoneNumber,0,1)=='0')
            {
                $phoneNumber = substr($phoneNumber,1);
            }
            $phoneNumber = "\"44".$phoneNumber."\",";
            $phoneNumber = substr($phoneNumber, 0,-1);
            $phoneNumber = preg_replace('/\s+/', '', $phoneNumber);
            $home->sendAppDownloadLink($phoneNumber,$message);

            $responseData = [
                "status" => true, 
                "data" => "Message Sent Successfully",
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        else
        {
            $responseData = [
                "status" => false, 
                "data" => "Phone number field is required",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }


public function emailWebhook(Request $request, Response $response, $args) {
    $inputData = json_decode(file_get_contents('php://input'), true);
    $responseData = [];
    $existsEmails = [];

    $db = new DB();
    $emails = $db->queryAll("SELECT JSON_OBJECTAGG(TRIM(email)  , dnd) AS result FROM tbl_emailwebhook WHERE dnd = 1");
    $invalidEmails = json_decode($emails[0]->result, true);
    if (!is_array($invalidEmails)) {
        $invalidEmails = [];
    }
    if (is_array($inputData)) {
        foreach ($inputData as $event) { 
            if (isset($event['event'], $event['email'], $event['timestamp'])) {
                if(empty($invalidEmails[$event['email']])){
                    if (in_array($event['event'], ['unsubscribe', 'bounce', 'deferred', 'group_unsubscribe'])) {
                        $eventType = $event['event'];
                        $email = $event['email'];
                        $timestamp = date('Y-m-d H:i:s', $event['timestamp']); 
                        $jsonResponse = json_encode($event); 
                        $escapedJsonResponse = addslashes($jsonResponse);
                        $data = [
                            'event_type' => $eventType,
                            'email' => $email,
                            'created_at' => $timestamp,
                            'jsonresponse' => $escapedJsonResponse
                        ];
                        $update = $db->updatewebhook($data);
                        if($update){
                            $responseData[] = [
                                "status" => true,
                                "message" => "Webhook data processed successfully",
                                'data' => $update,
                                "response_code" => 200,
                            ];
                        }else{
                            $responseData[] = [
                                "status" => true,
                                "message" => "Failed to update!...",
                                "response_code" => 200,
                            ];
                        }
                    } 
                }else{
                   $existsEmails[] = $event['email'];
                }
            } 
        }
        if(empty($responseData)){
            $responseData = [
                "status" => false,
                "message" => "All the emails are already present in the existing records",
                "response_code" => 422
            ];
        }
        $responseData['existEmails'] = $existsEmails;  
    } else {
        $responseData[] = [
            "status" => false,
            "message" => "Error! Post data is not in the requested format!",
            "response_code" => 422,
        ];
    }

    $response->getBody()->write(json_encode($responseData));
    return $response->withStatus(200);
}


   public function unsubscribe(Request $request, Response $response, $args)
    {
        
        $queryParams = $request->getQueryParams();
        $email = !empty($queryParams['email']) ? $queryParams['email'] : '';
        if (!empty($email) && filter_var($email, FILTER_VALIDATE_EMAIL)) {
            $dbobj = new DB();
            if ($email) {
                $existEmail = $dbobj->getEmail($email);
                if (!empty($existEmail)) {
                    if($existEmail->dnd == 1){
                            echo "<div style='
                            background-color: #dde2ee;
                            color: #721c24;
                            padding: 15px;
                            border-radius: 5px;
                            border: 1px solid #f5c6cb;
                            text-align: center;
                            font-family: Arial, sans-serif;
                            margin: 20px auto;
                            max-width: 600px;
                        '>
                            <strong>Notice!</strong> Your email is already unsubscribed. If you received a message recently, it might have been sent before your unsubscription request was processed.
                        </div>";
                    }else{
                       
                       $dbobj->updateDND($email);
                       echo "<div style='
                            background-color: #f8d7da;
                            color: #721c24;
                            padding: 15px;
                            border-radius: 5px;
                            border: 1px solid #f5c6cb;
                            text-align: center;
                            font-family: Arial, sans-serif;
                            margin: 20px auto;
                            max-width: 600px;
                        '>
                            <strong>Success!</strong> We've successfully unsubscribed your email. You won't receive any more messages from us.
                        </div>";
                    }
                } else {
                    $dbobj->updateDND($email);
                    echo "<div style='
                            background-color: #f8d7da;
                            color: #721c24;
                            padding: 15px;
                            border-radius: 5px;
                            border: 1px solid #f5c6cb;
                            text-align: center;
                            font-family: Arial, sans-serif;
                            margin: 20px auto;
                            max-width: 600px;
                        '>
                            <strong>Success!</strong> We've successfully unsubscribed your email. You won't receive any more messages from us.
                        </div>";
                }
            }
        } else {
            echo "<div style='
                background-color: #ffeeba;
                color: #856404;
                padding: 15px;
                border-radius: 5px;
                border: 1px solid #ffeeba;
                text-align: center;
                font-family: Arial, sans-serif;
                margin: 20px auto;
                max-width: 600px;
            '>
                <strong>Warning!</strong> Invalid request!
            </div>";
        }
        die();
    }
}

