<?php
namespace App\Controllers;
use App\Models\Location;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use App\Models\Menu;
use App\Models\RestaurantPage;
use App\Controllers\PostcodeController;
use App\Models\MenuPage;

require '../config/env.php';

class MenuController
{
    public function getOrderTime(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'client_id' => $inputData['client_id'] ?? '',
		];
		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!='0') {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if(!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $clientId = $params['client_id'];

        // $getBusinessTime = $this->getPreOrderTiming("0", $clientId);
        $getBusinessTime['Delivery'] = $this->getPreOrderTiming("0", $clientId);
        $getBusinessTime['Collection'] = $this->getPreOrderTiming("1", $clientId);

        $responseData = [
            "status" => true, 
            "message" => $getBusinessTime,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getPreOrderTiming($orderMode, $clientId)
    {
        $menu = new Menu();
        date_default_timezone_set('Europe/London');
        $currentTime = time();
        // $currentTime = strtotime("2024-10-22 11:55");
        $getBusinessTime = array();

        # get mode of order, tat time enable status and cooking time
        $getMode = $menu->getMode($clientId);
        $getTatEnableStatus = $menu->getTatEnableStatus($clientId);
        $tatStatus = $getTatEnableStatus->tat_enable;
        $getCookingTime = $menu->getCookingTime($clientId);

        if($orderMode == 0) {
            $type = "delivery";
        } elseif($orderMode == 1) {
            $type = "collection";
        }
        $holidayDates = $menu->getholidayDates($clientId);
        $todayDate    = date("Y-m-d");
        # check the preorder enable or not & working on 8 to 8 concept
        if ($getMode->preorder_sts == 1) {
            $pre = "1"; 

            # Get the current hour and minute
            $currentHour = date('H', $currentTime);
            $currentMinute = date('i', $currentTime);

            # Check if the current time is between 00:00 and 07:55
            if (($currentHour == '00' && $currentMinute >= '00') || ($currentHour < '08' && $currentMinute <= '55')) 
            {
                # Call getBusinessTime
                 if(in_array($todayDate,$holidayDates))
                 {
                    $getBusinessTime['today'] = "";
                 }else{
                    $getBusinessTime['today'] = $menu->getBusinessTime($currentTime, $getMode->mode, $clientId, $orderMode, $pre, $tatStatus, $getCookingTime);
                 }
            } else {
                # Call getBusinessTimeLater
                if(in_array($todayDate,$holidayDates))
                {
                    $getBusinessTime['today'] = "";
                }else{
                    $getBusinessTime['today'] = $menu->getBusinessTimeLater($currentTime, $getMode->mode, $clientId, $orderMode, $pre, $tatStatus, $getCookingTime);
                }
            }
        } else {
            $getBusinessTime['today'] = "No Pre-order";
        }


        # get asap order time and tat time
        $restaurantPage = new RestaurantPage();
        $openCloseStatus = $restaurantPage->restaurantOpenCloseStatus($clientId, $currentTime, $orderMode);
        if($openCloseStatus['status'] == "0") {
            $getCookingTime[$type] = ($getCookingTime[$type] == "") ? "0" : $getCookingTime[$type];
            $endTatTime = intval($getCookingTime[$type])+intval(10);
            $getBusinessTime['asap'] = array(
                'tatTime' => $getCookingTime[$type] .' - '. $endTatTime,
                'time' => date('Y-m-d H:i', $currentTime),
            );
        } else {
            $getBusinessTime['asap'] = "";
        }
        
        # check the preorder later enable or not & working on 12 to 12 concept
        $emptyPreOrder = "";
        if($getMode->preorder_sts != 0 && $getMode->preorderenable != 0) {
            $preorderLaterStatus = "+".$getMode->preorder_later_status."day";
            $tomorrow = strtotime(date('d-m-Y',(strtotime ( '+1 day' , $currentTime))));
            $endDate = strtotime(date('d-m-Y',(strtotime ($preorderLaterStatus , $tomorrow))));
            $shiftDate = strtotime(date('d-m-Y', $tomorrow)." "."08:00");
            
            while($shiftDate < $endDate) {
                $pre = "2"; 
                $selectedDate = date('Y-m-d', $shiftDate);
                
                if(in_array($selectedDate, $holidayDates))
                {
                    $getBusinessTime['later'][date('Y-m-d', $shiftDate)] = "";
                }else{
                    $getBusinessTime['later'][date('Y-m-d', $shiftDate)] = $menu->getBusinessTimeLater($shiftDate, $getMode->mode, $clientId, $orderMode, $pre, $tatStatus, $getCookingTime);
                }
                
                if($getBusinessTime['later'][date('Y-m-d', $shiftDate)] != "") {
                    $emptyPreOrder = "1";
                }
                $shiftDate = strtotime('+1 day', $shiftDate);
            }
            $preorderClosedData = 1;
            foreach($getBusinessTime['later'] as $key => $data) {
                if($data == "" || $data == "Shift closed") {
                    $preorderClosedData += 1;
                }
            }
            if($getMode->preorder_later_status < $preorderClosedData) {
                $getBusinessTime['later'] = "";
            }
            if($emptyPreOrder == "") {
                $getBusinessTime['later'] = ""; 
            }
        } else {
            $getBusinessTime['later'] = "";
        }

        return $getBusinessTime;
    }

    public function getRestaurantMenuDetail(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'order_mode' => $inputData['order_mode'] ?? '',
            'client_path' => $inputData['client_path'] ?? '',
            'order_time' => $inputData['order_time'] ?? '',
        ];
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value) && $value != '0') {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false,
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $clientPath = $params['client_path'];
        $orderMode = (int) $params['order_mode'];
        $orderTime = strtotime($params['order_time']);
        $customerId = (isset($inputData['customer_id']) && $inputData['customer_id'] != '0') ? $inputData['customer_id'] : "";
        $postCode   = (isset($inputData['post_code']) && $inputData['post_code'] != '0') ? $inputData['post_code'] : "";

        $menu = new Menu();
        $restaurantDetail = $menu->getRestaurantDetails($postCode, $clientPath, $orderMode, $customerId, $orderTime);

        if (isset($restaurantDetail['clientId'])) {
            $responseData = [
                "status" => true,
                "message" => $restaurantDetail,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false,
                "message" => $restaurantDetail,
                "response_code" => 400,
            ];
        }

        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getMenuPage(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'order_mode' => $inputData['order_mode'] ?? '',
            'client_path' => $inputData['client_path'] ?? '',
            'order_time' => $inputData['order_time'] ?? '',
        ];
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value) && $value!='0'  ) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if(!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $clientPath = $params['client_path'];
        $orderMode = $params['order_mode'];
        $orderTime = strtotime($params['order_time']);

        $menu = new Menu();
        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        $getMenuCollapse = $menu->getMenuCollapse($getClientId->id);
        $getlayoutthree  = $menu->getpromotionbannerimg($getClientId->id);

        $menuDetail = array();
        $menuDetail['clientPath'] = $clientPath;
        $menuDetail['clientId'] = $getClientId->id;
        $menuDetail['menuLayoutType'] = $getClientId->menuLayoutType;
        $menuDetail['LayoutPagethree'] = isset($getlayoutthree->status) ? "https://fusionbucket.co.uk/promotion_Image/".$getlayoutthree->promotion_Image : "";
        $menuDetail['menuCollapse'] = isset($getMenuCollapse->collapse) ? $getMenuCollapse->collapse : 0;
        $menuDetail['menuData'] = $menu->getMenus($getClientId->id, $orderMode, $orderTime);

        $location = new Location($getClientId->id);
        $menuDetail['currencyData'] = $location->getCurrency();
        
        $responseData = [
            "status" => true, 
            "message" => $menuDetail,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

	public function getReviewPage(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
		$clientPath = $inputData['client_path'];
		if(isset($clientPath) && $clientPath != "")
		{
			$menu = new Menu();
            # get client id using path
            $getClientId = $menu->getClientUsingB2cPath($clientPath);
            if($getClientId == false) {
                $responseData = [
                    "status" => false,
                    "message" => "Incorrect Path",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response; 
            }

            # get review datas from DB
            $getReview = array();
            $getReview['clientPath'] = $clientPath;
            $getReview['clientId'] = $getClientId->id;
			$getReview['review'] = $menu->getReviewData($getClientId->id);
            $responseData = [
                "status" => true, 
                "message" => "Review List get successfully",
                "data" => $getReview,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
		}
		else
		{
            $responseData = [
                "status" => false, 
                "message" => "Client Path is Required",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
		}
	}

	public function getAboutPage(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
		$clientPath = $inputData['client_path'];
		if(isset($clientPath) && $clientPath != "")
		{
			$menu = new Menu();

            # get client id using path
            $getClientId = $menu->getClientUsingB2cPath($clientPath);
            if($getClientId == false) {
                $responseData = [
                    "status" => false,
                    "message" => "Incorrect Path",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response; 
            }

            # get review datas from DB
            $getAbout = array();
            $getAbout['clientPath'] = $clientPath;
            $getAbout['clientId'] = $getClientId->id;
            # get about datas from DB
			$getAbout['about'] = $menu->getAboutData($getClientId->id);
            $responseData = [
                "status" => true, 
                "message" => "About Page get successfully",
                "data" => $getAbout,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
		}
		else
		{
            $responseData = [
                "status" => false, 
                "message" => "Client Path is Required",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
		}
	}

    public function getItemImage(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'client_path' => $inputData['client_path'] ?? '',
			'item_id' => $inputData['item_id'] ?? '',
			'order_mode' => $inputData['order_mode'] ?? '',
			'order_time' => $inputData['order_time'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!="0") {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $menu = new Menu();
        $itemImage = array();
        $itemStatus = array();
        $itemImageResponse = array();
        $workingDays = array();
        $itemId = $params['item_id'];
        $clientPath = $params['client_path'];
        $orderMode = $params['order_mode'];
        $orderTime = strtotime($params['order_time']);
        $zeroBasedDay = date('w', $orderTime);
        $oneBasedDay = date('N', $orderTime);

        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        # get item details
        $getItemDetail = $menu->getItemDetail($getClientId->id, $itemId, $orderMode, $zeroBasedDay);

        if($getItemDetail == false) {
            $responseData = [
                "status" => false,
                "message" => "Invalid Item Id",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }
        $itemImage['itemId'] = $getItemDetail['itemId'];
        $itemImage['description'] = $getItemDetail['itemDesc'];
        $itemImage['allowoffer'] = $getItemDetail['itemAllowOffer'];
        $itemImage['allowgift'] = $getItemDetail['itemAllowGift'];
        $itemImage['addon'] = $getItemDetail['itemAddon'];
        $itemImage['anotherName'] = str_replace("Â", "", $getItemDetail['itemAnotherName']);
        $itemImage['itemName'] =  str_replace(["<br>", "Â"], "", $getItemDetail['itemName']);
        
        if($getItemDetail['itemImage'] != "") {
            $itemImage['image'] = $_ENV["FUSION_BUCKET"]."menu/".$getItemDetail['itemImage'];
        } else {
            $itemImage['image'] = "";
        }

        # New menu price functionalities
        $itemPrice = $getItemDetail['itemPrice'];
        $increaseType = $getItemDetail['fkPriceType'];
        $percentageType = $getItemDetail['fkPercentageType'];
        $extraPrice = $getItemDetail['fkPrice'];
        $priceEnable = $getItemDetail['fkPriceStatus'];
        $priceMode = $getItemDetail['fkPriceMode'];
        $convertedPrice = $menu->getItemPriceValue($itemPrice, $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, $orderMode);
        $itemImage['price'] = $convertedPrice;

        # item available function based on category,sub category and item working times 
        $categoryTime = $getItemDetail["categoryTime".$oneBasedDay];
        $subCategoryTime = $getItemDetail["subCategoryTime".$oneBasedDay];
        $itemTime = $getItemDetail["itemTime".$oneBasedDay];
        $available = $menu->checkAvailableItem($categoryTime, $subCategoryTime, $itemTime, $orderTime);
        $itemImage['available'] = $available;

        # Check mode of order, open close, holidays functionalities
        $getMode = $menu->getMode($getClientId->id);
        $checkOpenStatus = $menu->checkOpenStatus($getClientId->id, $orderTime);

        $getOrderMode = $getMode->mode;
        $checkOrderMode = strpos($getOrderMode, $orderMode);
        if($checkOrderMode === false)
        {
            if($orderMode == 0) {
                $itemStatus = "change the mode to collection";
            } else if($orderMode == 1) {
                $itemStatus = "change the mode to delivery";
            }
        }

        if($checkOpenStatus != "") {
            $itemStatus = $checkOpenStatus;
        }

        $itemImageResponse['item'] = $itemImage;
        $itemImageResponse['message'] = $itemStatus;

        $responseData = [
            "status" => true, 
            "data" => $itemImageResponse,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getItemAddonList(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'client_path' => $inputData['client_path'] ?? '',
			'order_mode' => $inputData['order_mode'] ?? '',
			'order_time' => $inputData['order_time'] ?? '',
            'item_id' => $inputData['item_id'] ?? '',
            'addon_id' => $inputData['addon_id'] ?? '',
            'prev_step' => $inputData['prev_step'] ?? 0,
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!="0") {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}
        
        $menu = new Menu();
        $itemId = $params['item_id'];
        $addonId = $params['addon_id'];
        $clientPath = $params['client_path'];
        $orderMode = $params['order_mode'];
        $orderTime = strtotime($params['order_time']);
        if(isset($inputData['addon_item'])) {
            $addonItem = $inputData['addon_item'];
        } else {
            $addonItem = '';
        } 
        if(isset($inputData['extra'])) {
            $extra = $inputData['extra'];
        } else {
            $extra = '';
        }
        $zeroBasedDay = date('w', $orderTime);
        $oneBasedDay = date('N', $orderTime);

        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        # get item details
        $getItemDetail = $menu->getItemDetail($getClientId->id, $itemId, $orderMode, $zeroBasedDay); 

        if($getItemDetail == false) {
            $responseData = [
                "status" => true, 
                "message" => "Invalid Item Id",
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        # New menu price functionalities
        $itemPrice = $getItemDetail['itemPrice'];
        $increaseType = $getItemDetail['fkPriceType'];
        $percentageType = $getItemDetail['fkPercentageType'];
        $extraPrice = $getItemDetail['fkPrice'];
        $priceEnable = $getItemDetail['fkPriceStatus'];
        $priceMode = $getItemDetail['fkPriceMode'];
        $convertedPrice = $menu->getItemPriceValue($itemPrice, $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, $orderMode);
        $getItemDetail['itemPrice'] = $convertedPrice;
        $getItemDetail['deliveryPrice'] = $menu->getItemPriceValue($getItemDetail['deliveryPrice'], $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, 0);
        $getItemDetail['collectionPrice'] = $menu->getItemPriceValue($getItemDetail['collectionPrice'], $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, 1);

        if($getItemDetail['itemAddon'] == "") {
            $responseData = [
                "status" => true, 
                "message" => "No addons, Item Added",
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        } else {
            $selected = array();
            $getAddonDetail = $menu->getAddonDetail($getClientId->id, $addonId);
            $getAddonItemDetailWithAddon = $menu->getAddonItemDetailWithAddon($getClientId->id, $addonId);
            
            if($addonItem != ""){
                $items = explode(",",$addonItem); 
                $extra = explode(",",$extra);
                $notify = array("No ","less ","Half ","On ","With ","On Burger ","On Chips ");
                foreach($items as $key => $id){
                    $item = $menu->getAddonItemDetailWithId($getClientId->id, $id);
                    $item['extra'] = "";
                    if(strlen($extra[$key])){
                        $item['name'] = $notify[$extra[$key]].$item['name'];
                        $item['extra'] = $extra[$key];
                    }
                    if ($item) {
                        $selected[] = ['items' => $item['name'], 'extra' => $item['extra'], 'id' => $item['id']];
                        $getItemDetail['itemPrice'] += $item['price'];
                        $getItemDetail['deliveryPrice'] += $item['price'];
                        $getItemDetail['collectionPrice'] += $item['price'];
                    }
                }
            }

            // get currency info
            $location = new Location($getClientId->id);
            $currency = $location->getCurrency();

            $data = array(
                "item"=>$addonItem,
                "addon"=>$getAddonDetail,
                "addonItems"=>$getAddonItemDetailWithAddon,
                "selected"=>$selected,
                "orderItem"=>$getItemDetail,
                "currency"=>$currency,
            );
            $ResultAry = $menu->getAddonList($data);
            $ResultAry['count'] = $inputData['count'] ?? 0;
            $ResultAry['addon'] = $getItemDetail['itemAddon'] ?? 0;

            $ResultAry['current_step'] = $params['prev_step'] + 1;
            $ResultAry['total_step'] = $params['prev_step'] + $menu->countUpcomingAddonSteps($getClientId->id, $addonId);
            
            $responseData = [
                "status" => true, 
                "message" => $ResultAry,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;   
        }
    }

    public function getChargesDetail(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'client_path' => $inputData['client_path'] ?? '',
		];
		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!="0") {
				$errors[] = ucfirst($key) . ' is required';
			}
		}
		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $menu = new Menu();
        $data = array();
        $clientPath = $params['client_path'];

        $postCode   = $inputData['postCode'] ?? '';
        $postCode   = str_replace(' ','',$postCode);
        $orderMode  = $inputData['orderMode'] ?? '';
        $appKey     = $inputData['appKey'] ?? '';
        $customerId = $inputData['customerId'] ?? '';
        $orderTime  = $inputData['orderTime'] ?? '';

        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        $postcodeController = new PostcodeController();

        $clientId =$getClientId->id;
       if(!empty($appKey) && $appKey == "1")
       {
            # min order functions
            $getMin = $menu->getMin($clientId);

            # get client order config radius/postcode
            $getConfigOrder = $menu->getConfigOrder($clientId);

            if($getConfigOrder->deliveryType === 0) 
            {
                # postcode method
                $getPostcodeResponse = $postcodeController->checkPostcode($clientId, $postCode);
                $charge['DeliveryCharge'] =  $getPostcodeResponse;
                if(!empty($getPostcodeResponse)) 
                {
                    $charge['DeliveryCharge']['status'] = true;
                }
                else 
                {
                    $charge['DeliveryCharge']['postcode'] = '';
                    $charge['DeliveryCharge']['deliveryCharge'] = '';
                    $charge['DeliveryCharge']['minAmount'] = 0;
                    $charge['DeliveryCharge']['freeAbove'] = '';
                    $charge['DeliveryCharge']['status'] = false;
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['status'] = false;
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['distance'] = '';
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['minAmount'] = 0;
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['deliveryCharge'] = "";
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['freeAbove'] = "";
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['address1'] = '';
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['address2'] = '';
                    $charge['DeliveryCharge']['stuartDeliveryCharge']['country'] =  '';
                }
            } 
            else 
            {
                # radius method
                $getContacts =  $menu->getContacts($clientId);
                $restaurantPostcode = $getContacts->pincode;
                
                # get distance option and maximum radius
                $distanceOption = $getConfigOrder->distanceOption;
                $maximumRadius = $getConfigOrder->maximumRadius;
                $getPostcodeResponse = $postcodeController->checkRadius($clientId, $restaurantPostcode, $postCode, $distanceOption, $maximumRadius);
                $charge['DeliveryCharge'] = $getPostcodeResponse;
                if($getPostcodeResponse['status'] == true)
                {
                    $getPostCodeDetail = $postcodeController->getPostCodeDetail($postCode);
                    $charge['DeliveryCharge'] = json_decode($getPostCodeDetail);	
                    $charge['DeliveryCharge']['status'] = true;	
                }
                else
                {
                    $charge['DeliveryCharge']['status'] = false;
                    $charge['DeliveryCharge']['minAmount'] = 0;
                }
            }
            $minOrder = ($orderMode == 0) ? $getMin->delivery_min : $getMin->collection_min;

            $charge['DeliveryCharge']['minAmount'] = (floatval($charge['DeliveryCharge']['minAmount']) >= floatval($minOrder))? $getPostcodeResponse['minAmount'] : $minOrder;

            $getStuartStatus = $menu->getStuartStatus($clientId);
            $charge['DeliveryCharge']['stuartDeliveryCharge']['status'] = false; 
            $charge['DeliveryCharge']['stuartDeliveryCharge']['distance'] = '';            
            $charge['DeliveryCharge']['stuartDeliveryCharge']['minAmount'] = 0;
            $charge['DeliveryCharge']['stuartDeliveryCharge']['deliveryCharge'] = "";
            $charge['DeliveryCharge']['stuartDeliveryCharge']['freeAbove'] = "";
            $charge['DeliveryCharge']['stuartDeliveryCharge']['address1'] = '';
            $charge['DeliveryCharge']['stuartDeliveryCharge']['address2'] = '';
            $charge['DeliveryCharge']['stuartDeliveryCharge']['country'] =  '';            
            if($getStuartStatus) {
                if($getStuartStatus->status == 1) {
                    $charge['DeliveryCharge']['stuartDeliveryCharge'] = $postcodeController->getStuartDeliveryCharge($clientId, $postCode,$orderMode);
                }
            }
       }

        # get service and bag charge
        $getChargesDetail = $menu->getChargesDetail($getClientId->id);
        
        $charge['serviceCharge']  = $getChargesDetail['serviceCharge'];
        $charge['baggageCharge']  = $getChargesDetail['baggageCharge'];
        $charge['clientFkCharge'] = $getChargesDetail['clientFkCharge'];
        $charge['is_FkCharge']    = $getChargesDetail['is_FkCharge'];
        $charge['FkCharge']       = $getChargesDetail['FkCharge'];
        $charge['ClientCharge']   = $getChargesDetail['ClientCharge'];

        $getDriverTip = $menu->getDriverTip($getClientId->id);
        if(isset($getDriverTip->is_restauranttip))
        {
            $charge['driverTipStatus'] = ($getDriverTip->is_restauranttip == 0 || $getDriverTip->is_restauranttip == null || $getDriverTip->is_restauranttip == 2)
                                         ? true : false;
        }
        else
        {
            $charge['driverTipStatus'] = "";
        }
        $responseData = [
            "status" => true,
            "message" => $charge,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;   
    }

    public function createEmailFeedback(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'clientPath' => $inputData['clientPath'] ?? '',
			'reason' => $inputData['reason'] ?? '',
			'rating' => $inputData['rating'] ?? '',
			'orderId' => $inputData['orderId'] ?? '',
			'comment' => $inputData['comment'] ?? '',
			'agent' => $inputData['agent'] ?? '',
		];
		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!="0") {
				$errors[] = ucfirst($key) . ' is required';
			}
		}
		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $menu = new Menu();

        $clientPath = $params['clientPath'];

        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        # update email feedback mail
        $params['clientId'] = $getClientId->id;
        $updateEmailFeedback = $menu->createEmailFeedback($params);

        if($updateEmailFeedback['status'] == false) {
            $responseData = [
                "status" => false,
                "message" => $updateEmailFeedback['data'],
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;   
        } else {
            $responseData = [
                "status" => true,
                "message" => $updateEmailFeedback,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;   
        }
    }

    public function checkVoucherCode(Request $request, Response $response, $args) 
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'customerId' => $inputData['customerId'] ?? '',
			'clientPath' => $inputData['clientPath'] ?? '',
			'orderTime' => $inputData['orderTime'] ?? '',
			'voucherCode' => $inputData['voucherCode'] ?? '',
		];
		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!="0") {
				$errors[] = ucfirst($key) . ' is required';
			}
		}
		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $menu = new Menu();
        $clientPath = $params['clientPath'];

        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        $clientId = $getClientId->id;
        $customerId = $params['customerId'];
        $orderTime = strtotime($params['orderTime']);
        $voucherCode = $params['voucherCode'];

        $zeroBasedDay = date('w', $orderTime);

        $voucher = $menu->checkVoucher($clientId, $customerId);
        if($voucher['status'] == false) {
            $responseData = [
                "status" => false, 
                "message" => "voucher not available",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        $getVoucherDetail = $menu->getVoucherDetail($clientId, $voucherCode, $zeroBasedDay);

        if($getVoucherDetail != false) {
            if($getVoucherDetail->from <= $orderTime && $orderTime <= $getVoucherDetail->to) {
                $location = new Location($clientId);
                $currency = $location->getCurrency();

                $result['code'] = $getVoucherDetail->code;
                $result['discount']    = $getVoucherDetail->discount;
                $result['description'] = $getVoucherDetail->name;
                $result['minAmount'] = $getVoucherDetail->minimum;
                $result['type'] = ($getVoucherDetail->type == "%") ? "%" : $currency['symbol'];
                if(!empty($voucher['voucherCode'])) {
                    $code = explode(",",$voucher['voucherCode']); 
                    if(!in_array(strtoupper($voucherCode), array_map('strtoupper', $code))) {
                        $responseData = [
                            "status" => true, 
                            "message" => $result,
                            "response_code" => 200,
                        ];
                        $response->getBody()->write(json_encode($responseData));
                        return $response;
                    } else {
                        $responseData = [
                            "status" => false, 
                            "message" => "voucher code was already Used",
                            "response_code" => 400,
                        ];
                        $response->getBody()->write(json_encode($responseData));
                        return $response;
                    }
                } else {
                    $responseData = [
                        "status" => true, 
                        "message" => $result,
                        "response_code" => 200,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => "Validity period expired",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        } else {
            $responseData = [
                "status" => false, 
                "message" => "Invalid code",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function getItemsNotAddOns(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);

        $params = [
            'clientPath' => $inputData['clientPath'] ?? '',
        ];

        $clientPath = $params['clientPath'];

        $menu = new Menu();
        $getClientId = $menu->getClientUsingB2cPath($clientPath);

        if($getClientId == false) 
        {
            $responseData = [ "status" => false, "message" => "Incorrect Path", "response_code" => 400, ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        $clientId = $getClientId->id;

        $ItemList = array();
        $getItemsNotAddOns = $menu->getItemsNotAddOns($clientId);
        
        if(!empty($getItemsNotAddOns))
        {
            $getItemsNotAddOns = json_decode(json_encode($getItemsNotAddOns), true);
            $oneBasedDay  = date('N');
            foreach($getItemsNotAddOns as $key => $getItems)
            {
            
                $currentTime = strtotime(date('h:i A'));
                      # item available function based on category,sub category and item working times 
                      $categoryTime    = $getItems["categoryTime".$oneBasedDay];
                      $subCategoryTime = $getItems["subCategoryTime".$oneBasedDay];
                      $itemTime  = $getItems["itemTime".$oneBasedDay];
                      $available = $menu->checkAvailableItem($categoryTime, $subCategoryTime, $itemTime, $currentTime);

                      # New menu price functionalities
                      $itemPrice      = $getItems['itemPrice'];
                      $increaseType   = $getItems['fkPriceType'];
                      $percentageType = $getItems['fkPercentageType'];
                      $extraPrice     = $getItems['fkPrice'];
                      $priceEnable    = $getItems['fkPriceStatus'];
                      $priceMode      = $getItems['fkPriceMode'];
                      $price = $menu->getItemPriceValue($itemPrice, $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, $currentTime);

                          
                          $Item['categoryId']             = $getItems['categoryId'];
                          $Item['categoryName']           = $getItems['categoryName'];
                          $Item['categoryAnotherName']    = $getItems['categoryAnotherName'];
                          $Item['categoryAllowOffer']     = $getItems['categoryAllowOffer'];
                          $Item['categoryAllowGift']      = $getItems['categoryAllowGift'];
                          $Item['subCategoryId']          = $getItems['subCategoryId'];
                          $Item['subCategoryName']        = $getItems['subCategoryName'];
                          $Item['subCategoryAnotherName'] = $getItems['subCategoryAnotherName'];
                          $Item['subCategoryDesc']        = $getItems['subCategoryDesc'];
                          $Item['subCategoryAllowOffer']  = $getItems['subCategoryAllowOffer'];
                          $Item['subCategoryAllowGift']   = $getItems['subCategoryAllowGift'];
                          $Item['itemId']                 = $getItems['itemId'];
                          $Item['itemImage']              = $getItems['itemImage'] ? $_ENV["FUSION_BUCKET_MENU"].$getItems['itemImage'] : $getItems['itemImage'];
                          $Item['itemName']               = $getItems['itemName'];
                          $Item['itemAnotherName']        = $getItems['itemAnotherName'];
                          $Item['itemDesc']               = $getItems['itemDesc'];
                          $Item['available']              = $available;
                          $Item['price']                  = $price;
                          $Item['itemAddon']              = $getItems['itemAddon'];
                          $Item['itemFreeAddon']          = $getItems['itemFreeAddon'];
                          $Item['itemAllowOffer']         = $getItems['itemAllowOffer'];
                          $Item['itemAllowGift']          = $getItems['itemAllowGift'];

                          array_push($ItemList,$Item);
            }

            $responseData = [ "status" => true, "data"  =>  $ItemList, "message" => "Item List Provided","response_code" => 200, ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        } else 
        {
            $responseData = [ "status" => false, "message" => "Item List is Empty", "response_code" => 400, ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function getOfferDetailsList(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);

        $params = [
            'clientPath' => $inputData['clientPath'] ?? '',
            'orderMode'  => $inputData['orderMode'] ?? '',
            'orderTime'  => $inputData['orderTime'] ?? '',
            'customerId' => $inputData['customerId'] ?? '',
        ];


		if (empty($params['clientPath'])) 
        {
			$responseData = [ "status" => false, "message" => "client Path is Empty","response_code" => 400, ];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        if (empty($params['orderTime'])) 
        {
			$responseData = [ "status" => false, "message" => "Order Time is Empty", "response_code" => 400, ];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}


        $clientPath = $params['clientPath'];

        $menu = new Menu();
        $getClientId = $menu->getClientUsingB2cPath($clientPath);

        if($getClientId == false) 
        {
            $responseData = [ "status" => false, "message" => "Incorrect Path", "response_code" => 400, ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        $clientId   = $getClientId->id;
        $customerId = $params['customerId'];
        $orderTime  = strtotime($params['orderTime']);
        $orderMode  = $params['orderMode'];
        $getoffers = $menu->getOfferDetails($clientId,$orderMode, $orderTime,$customerId,$clientPath);

        $response->getBody()->write(json_encode($getoffers));
		return $response;
    }

    public function searchOfferCode(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);

        $params = [
            'clientPath' => $inputData['clientPath'] ?? '',
            'orderMode'  => $inputData['orderMode'] ?? '',
            'orderTime'  => $inputData['orderTime'] ?? '',
            'customerId' => $inputData['customerId'] ?? '',
            'offerCode'  => $inputData['offerCode'] ?? '',
        ];

        if (empty($params['clientPath'])) 
        {
			$responseData = [ "status" => false, "message" => "client Path is Empty","response_code" => 400, ];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        if (empty($params['orderTime'])) 
        {
			$responseData = [ "status" => false, "message" => "Order Time is Empty", "response_code" => 400, ];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        if(empty($params['offerCode']))
        {
            $responseData = [ "status" => false, "message" => "Offer Code is Empty", "response_code" => 400, ];
			$response->getBody()->write(json_encode($responseData));
			return $response;
        }


        $clientPath = $params['clientPath'];

        $menu = new Menu();
        $getClientId = $menu->getClientUsingB2cPath($clientPath);

        $clientId   = $getClientId->id;
        $customerId = $params['customerId'];
        $orderTime  = strtotime($params['orderTime']);
        $orderMode  = $params['orderMode'];
        $offerCode  = $params['offerCode'];
        $orderModeCode =  ($orderMode == 0)? "Delivery" : "Collection";
        $checkOfferCode = $menu->getOfferDetails($clientId, $orderMode,$orderTime, $customerId,$clientPath);

        $getkey = '';
        $getOfferCode = [];
        $zeroBasedDay = date('w', $orderTime);

        if(!empty($customerId))
        {
            $checkVoucherCode = $menu->checkVoucher($clientId, $customerId);
            if($checkVoucherCode['status'] == false) 
            {
                $responseData = [ "status" => false, "message" => "voucher not available", "response_code" => 400, ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }

            $getVoucherDetail = $menu->getVoucherDetail($clientId, $offerCode, $zeroBasedDay);
            if($getVoucherDetail != false) 
            {
                if($getVoucherDetail->from <= $orderTime && $orderTime <= $getVoucherDetail->to) 
                {
                    $location = new Location($clientId);
                    $currency = $location->getCurrency();

                    $getOfferCode['code'] = $getVoucherDetail->code;
                    $getOfferCode['description'] = $getVoucherDetail->name;
                    $getOfferCode['terms1'] = "Coupon code applicable only on orders above " . $currency['symbol'] . ' ' . $getVoucherDetail->minimum . ".";
                    $getOfferCode['terms2'] = "Offer is valid only for this particular takeaway/restaurant.";
                    $getOfferCode['terms3'] = "No maximum limit to apply the coupon code.";
                    $getOfferCode['terms4'] = "Offer valid only till " . date('d/m/Y h:i A', $getVoucherDetail->to) . ".";
                    $getOfferCode['terms5'] = "This coupon code is applicable only for " . $orderModeCode . " orders.";
                    $getOfferCode['minAmount'] = $getVoucherDetail->minimum;
                    $getOfferCode['type'] = ($getVoucherDetail->type == "%") ? "%" : $currency['symbol'];

                    if(!empty($checkVoucherCode['voucherCode'])) 
                    {
                        $code = explode(",",$checkVoucherCode['voucherCode']); 
                        if(!in_array($offerCode, $code)) 
                        {
                            $responseData = [ "status" => true, "message" => $getOfferCode, "response_code" => 200,];
                            $response->getBody()->write(json_encode($responseData));
                            return $response;
                        } else 
                        {
                            $responseData = [ "status" => false, "message" => "voucher code was already Used", "response_code" => 400, ];
                            $response->getBody()->write(json_encode($responseData));
                            return $response;
                        }
                    } else 
                    {
                        $responseData = [ "status" => true, "message" => $getOfferCode, "response_code" => 200, ];
                        $response->getBody()->write(json_encode($responseData));
                        return $response;
                    }
                } else 
                {
                    $responseData = [ "status" => false, "message" => "Validity period expired", "response_code" => 400, ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            } else 
            {
                $responseData = [ "status" => false, "message" => "Invalid code", "response_code" => 400, ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }

            $getOfferCode = $getOfferCode;
        }
          


          if(empty($getOfferCode))
          {
                   #check offercode in PROMO 
                    if($checkOfferCode['promo'] === array()){}
                    else 
                    {
                        foreach($checkOfferCode['promo'] as $key => $promo )
                        {
                            if($promo['promo_code'] == $offerCode){ $getkey = $key; }
                        }
                        if(is_numeric($getkey))
                        {
                            if($checkOfferCode['promo'][$getkey])
                            {
                                $getOfferCode = $checkOfferCode['promo'][$getkey];
                                $responseData = [ "status" => true, "data" => $getOfferCode, "message" => "Offer Code Provided", "response_code" => 200, ];
                                $response->getBody()->write(json_encode($responseData));
                                return $response;
                            } 
                        }
                    }

                    #check offercode in seasonal_online
                    if($checkOfferCode['seasonal_online'] === array()) {}
                    else 
                    {
                            foreach($checkOfferCode['seasonal_online'] as $key => $seasonal)
                            {
                                if($seasonal['promo_code'] == $offerCode){ $getkey = $key; }
                            }

                            if(is_numeric($getkey))
                            {
                                if($checkOfferCode['seasonal_online'][$getkey])
                                {
                                    $getOfferCode = $checkOfferCode['seasonal_online'][$getkey];
                                    $responseData = [ "status" => true, "data" => $getOfferCode, "message" => "Offer Code Provided", "response_code" => 200, ];
                                    $response->getBody()->write(json_encode($responseData));
                                    return $response;
                                }
                            }
                    }

                    #check offercode in online
                    if($checkOfferCode['online'] === array()){}
                    else{

                            foreach($checkOfferCode['online'] as $key => $online)
                            {
                                if($online['code'] == $offerCode)
                                {
                                    $getkey = $key;
                                }
                            }
                            
                            if(is_numeric($getkey))
                            {
                                if($checkOfferCode['online'][$getkey])
                                {
                                    $getOfferCode = $checkOfferCode['online'][$getkey];
                                    $responseData = [ "status" => true, "data" => $getOfferCode, "message" => "Offer Code Provided", "response_code" => 200, ];
                                    $response->getBody()->write(json_encode($responseData));
                                    return $response;
                                }
                            }
                            
                    }

                    #check offercode in common
                    if($checkOfferCode['common'] === array()){}
                    else{
                            foreach($checkOfferCode['common'] as $key => $common)
                            {
                                if($common['discount_code'] == $offerCode)
                                {
                                    $getkey = $key;
                                }
                            }

                            if(is_numeric($getkey))
                            {
                                if($checkOfferCode['common'][$getkey])
                                {
                                    $getOfferCode = $checkOfferCode['common'][$getkey];
                                    $responseData = [ "status" => true, "data" => $getOfferCode, "message" => "Offer Code Provided", "response_code" => 200, ];
                                    $response->getBody()->write(json_encode($responseData));
                                    return $response;
                                }
                            }   
                    }

                    #check offercode in autoDiscount
                    if($checkOfferCode['autoDiscount'] === array()){}
                    else{

                            foreach($checkOfferCode['autoDiscount'] as $key => $autoDiscount)
                            {
                                if($autoDiscount['discount_code'] == $offerCode)
                                {
                                    $getkey = $key;
                                }
                            }

                            if(is_numeric($getkey))
                            {
                                if($checkOfferCode['autoDiscount'][$getkey])
                                {
                                    $getOfferCode = $checkOfferCode['autoDiscount'][$getkey];
                
                                    $response->getBody()->write(json_encode($getOfferCode));
                                    return $response;
                                }
                            }      
                    }
          }
        
          
        

        if($getOfferCode)
        {
            $responseData = [ "status" => true, "data" => $getOfferCode, "message" => "Offer Code Provided", "response_code" => 200, ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        else {
            $responseData = [ "status" => false, "message" => "Offer Code Not Found", "response_code" => 400, ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function getItemAvailable(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'client_path' => $inputData['client_path'] ?? '',
            'order_time' => $inputData['order_time'] ?? '',
        ];
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value) && $value != '0') {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false,
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $clientPath = $params['client_path'];
        $orderTime = strtotime($params['order_time']);

        $menu = new Menu();

        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if ($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $zeroBasedDay = date('w', $orderTime);
        $currentTime = strtotime(date('h:i A', $orderTime));
        $oneBasedDay = date('N', $orderTime);

        $itemsList = $menu->getItemsList((int) $getClientId->id, (int) $zeroBasedDay);

        $itemDetail = array();
        $itemDetail['clientId'] = $getClientId->id;
        $itemDetail['items'] = array_map(function ($item) use ($currentTime, $oneBasedDay) {
            $item['itemImage'] = empty($item['itemImage']) ? "" : $_ENV['FUSION_BUCKET'] . 'menu/' . $item['itemImage'];

            $item['deliveryAvailable'] = "";
            $item['collectionAvailable'] = "";

            if (isset($item['orderMode'])) {
                $itemTime = $item['itemTime' . $oneBasedDay];
                if ($itemTime != "") {
                    $itemTimes = explode('-', $itemTime);
                    $itemFrom = strtotime($itemTimes[0]);
                    $itemTo = strtotime($itemTimes[1]);
                    if ($itemFrom <= $currentTime && $itemTo >= $currentTime) {
                        $available = "";
                    } else {
                        $available = "Available at " . $itemTime;
                    }
                } else {
                    $available = "";
                }

                switch ($item['orderMode']) {
                    case 0:
                        $item['deliveryAvailable'] = isset($available) ? $available : "Delivery Not Available";
                        $item['collectionAvailable'] = "Collection Not Available";
                        break;
                    case 1:
                        $item['deliveryAvailable'] = "Delivery Not Available";
                        $item['collectionAvailable'] = isset($available) ? $available : "Collection Not Available";
                        break;
                    case 2:
                        $item['deliveryAvailable'] = isset($available) ? $available : "Delivery Not Available";
                        $item['collectionAvailable'] = isset($available) ? $available : "Collection Not Available";
                        break;
                }
            }

            return $item;
        }, $itemsList);

        $responseData = [
            "status" => true,
            "data" => $itemDetail,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getRestaurantMenuDetailOptimize(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'order_mode' => $inputData['order_mode'] ?? '',
            'client_path' => $inputData['client_path'] ?? '',
            'order_time' => $inputData['order_time'] ?? '',
        ];
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value) && $value != '0') {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false,
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $clientPath = $params['client_path'];
        $orderMode = (int) $params['order_mode'];
        $orderTime = strtotime($params['order_time']);

        if (isset($inputData['customer_id']) && $inputData['customer_id'] != '0') {
            $customerId = $inputData['customer_id'];
        } else {
            $customerId = "";
        }

        if (isset($inputData['post_code']) && $inputData['post_code'] != '0') {
            $postCode = $inputData['post_code'];
        } else {
            $postCode = "";
        } 
       

        $Menu = new Menu();

        $getClientId = $Menu->getClientUsingB2cPath($clientPath);

        if ($getClientId == false) {
            return "Incorrect Path";
        }
        $MenuPage = new MenuPage($getClientId->id);
        $restaurantDetail = $MenuPage->getRestaurantDetails($postCode, $orderMode, $customerId, $orderTime,$clientPath,$getClientId);

        if (isset($restaurantDetail['clientId'])) {
            $responseData = [
                "status" => true,
                "message" => $restaurantDetail,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false,
                "message" => $restaurantDetail,
                "response_code" => 400,
            ];
        }

        $response->getBody()->write(json_encode($responseData));
        return $response;
    }
}

?>