<?php
namespace App\Controllers;
use App\Models\Location;
use App\Models\Subscriber;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use App\Models\Myaccount;
use App\Models\User;
use App\Models\Client;
use App\Models\DB;
use App\Models\RestaurantPage;
use App\Helpers\SendEmail;
use App\Helpers\SendSms;
use App\Controllers\PagesController;
use App\Helpers\SendEposEmail;
require '../config/env.php';

class MyAccountController
{
    public function getProfile(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        if(!isset($inputData['customer_id']) || $inputData['customer_id'] == "") 
        {
            $responseData = [
				"status" => false, 
				"message" => "customer id is required",
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
        }
        else 
        {
            $myAccount = new Myaccount();
            $customerId = $inputData['customer_id'];

            # get profile
            $getProfile['profile'] = $myAccount->getProfile($customerId);
            $getOrderHistory = $myAccount->getOrderHistory($customerId);
            if($getOrderHistory != false) {
                if($getOrderHistory['order'] != false) {
                    $getProfile['lastOrder']['orderId'] = $getOrderHistory['order'][0]['orderId'];
                    $getProfile['lastOrder']['clientId'] = $getOrderHistory['order'][0]['clientId'];
                } else {
                    $getProfile['lastOrder']['orderId'] = "";
                    $getProfile['lastOrder']['clientId'] = "";
                }
                $getProfile['totalOrderCount'] = count($getOrderHistory['order']);
                $getProfile['totalAppliedDiscount'] = $getOrderHistory['totalAppliedDiscount'];
            } else {
                $getProfile['lastOrder']['orderId'] = "";
                $getProfile['lastOrder']['clientId'] = "";
                $getProfile['totalOrderCount'] = 0;
                $getProfile['totalAppliedDiscount'] = 0;
            }
            if (isset($inputData['domain']) && $inputData['domain']) {
                // Get client id using domain
                $client = (new Myaccount())->getPathUsingDomain($inputData['domain']);
                $subscriber = new Subscriber($client->id, $customerId, 1);
                $getProfile["subscription_status"] = $subscriber->is_subscribed();
                $getProfile["unsubscription_status"] = $subscriber->is_unsubscribe();
            } else {
                $getProfile["subscription_status"] = false;
                $getProfile["unsubscription_status"] = false;
            }

            $responseData = [
				"status" => true, 
				"message" => "customer detail ready",
                "data" => $getProfile,
                "response_code" => 200,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
            
        }
    }

    public function sendEposemail(Request $request, Response $response, $args)
    {
       $inputData = json_decode(file_get_contents('php://input'), true);
        

             if($inputData)
            {
                # send otp via mail id
               
                $sendEmail = new SendEposEmail();
                $otpHtml = $inputData['html'];
                $otpTo = $inputData['to'];
                $otpCc = array();
                $otpSub = $inputData['subject'];
                $reply = $inputData['replyto'];
                $replyName = '';
                $from = $inputData['from'];
                $fromName = '';

                $otpMail = $sendEmail->sendMail($otpTo,$otpCc,$otpSub,$otpHtml,$reply,$replyName, $from, $fromName);

                $responseData = [
                    "status" => true, 
                    "message" => "otp",
                    "data" => $otpMail,
                    "response_code" => 200,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            } 
            else
            {
                $responseData = [
                    "status" => false, 
                    "message" => "Not valid data",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        
    }



    public function sendProfileOtp(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        if(!isset($inputData['otp_data']) || $inputData['otp_data'] == "") 
        {
            $responseData = [
				"status" => false, 
				"message" => "data is required",
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
        }
        else 
        {
            $otpData = $inputData['otp_data'];
            if(is_numeric($otpData))
            {
                # send via mobile number
                $otp = mt_rand(1000,9999);
                $sms = new SendSms();
                $mobileNo = "44".$otpData;
                $message = "Your Fusion Kitchen verification code is: ".$otp;
                $otpSms = $sms->sendSms($mobileNo,$message);

                $responseData = [
                    "status" => true, 
                    "message" => "otp",
                    "data" => $otp,
                    "response_code" => 200,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
            else if(filter_var($otpData, FILTER_VALIDATE_EMAIL))
            {
                # send otp via mail id
                $otp = mt_rand(1000,9999);
                $sendEmail = new SendEmail();
                $otpHtml = "Your Fusion Kitchen verification code is: ".$otp;
                $otpTo = $otpData;
                $otpCc = array();
                $otpSub = "Fusion Kitchen";
                $otpMail = $sendEmail->sendMail($otpTo,$otpCc,$otpSub,$otpHtml);

                $responseData = [
                    "status" => true, 
                    "message" => "otp",
                    "data" => $otp,
                    "response_code" => 200,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            } 
            else
            {
                $responseData = [
                    "status" => false, 
                    "message" => "Not valid data",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }
    }

    public function updateProfile(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'customer_id' => $inputData['customer_id'] ?? '',
			'first_name' => $inputData['first_name'] ?? '',
			'last_name' => $inputData['last_name'] ?? '',
			'email' => $inputData['email'] ?? '',
			'phone' => $inputData['phone'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $customerId = $params['customer_id'];
        $firstName = $params['first_name'];
        $lastName = $params['last_name'];
        $email = $params['email'];
        $phone = $params['phone'];

        # update profile
        $updateProfile = $myAccount->updateProfile($customerId, $firstName, $lastName, $email, $phone);

        $responseData = [
            "status" => true, 
            "message" => "profile updated",
            "data" => $updateProfile,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
		return $response;
    }

    public function getAddress(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        if(isset($inputData['customer_id']) && $inputData['customer_id'] != "") 
        {
            # getting address by using customer id
            $myAccount = new Myaccount();
            $customerId = $inputData['customer_id'];

            # getting address
            $getAddress = $myAccount->getAddress($customerId);

            $responseData = [
                "status" => true, 
                "message" => "Address list",
                "data" => $getAddress,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        } 
        else if(isset($inputData['email']) && $inputData['email'] != "") 
        {
            # getting address by using customer email
            $myAccount = new Myaccount();
            $user = new User();
            $email = $inputData['email'];
            $checkCustomerByEmail = $user->checkCustomerByEmail($email);
            if($checkCustomerByEmail != false) {
                # getting address
                $getAddress = $myAccount->getAddress($checkCustomerByEmail->id);

                $responseData = [
                    "status" => true, 
                    "message" => "Address list",
                    "data" => $getAddress,
                    "response_code" => 200,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => "Invalid Email",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }            
        } else {
            $responseData = [
                "status" => false, 
                "message" => "customer id is required",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function updatePrimaryAddress(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'customer_id' => $inputData['customer_id'] ?? '',
			'address_type' => $inputData['address_type'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        if($params['address_type'] > 3)
        {
            $responseData = [
				"status" => false, 
				"message" => "Invalid address type is required",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
        }
        else 
        {
            $myAccount = new Myaccount();
            $customerId = $params['customer_id'];
            $addressType = $params['address_type'];

            #updating primary address
            $primaryAddress = $myAccount->updatePrimaryAddress($customerId, $addressType);

            $responseData = [
                "status" => true, 
                "message" => "primary address updated",
                "data" => $primaryAddress,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function addAddress(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'customer_id' => $inputData['customer_id'] ?? '',
			'address_type' => $inputData['address_type'] ?? '',
			'first_name' => $inputData['first_name'] ?? '',
			'last_name' => $inputData['last_name'] ?? '',
			'phone_number' => $inputData['phone_number'] ?? '',
			'postcode' => $inputData['postcode'] ?? '',
			'door' => $inputData['door'] ?? '',
			'street' => $inputData['street'] ?? '',
			'city' => $inputData['city'] ?? '',
			'primary' => $inputData['primary'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value) && $value!='0') {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}
    
        $myAccount = new Myaccount();
        $request = array();
        $request['customerId'] = $params['customer_id'];
        $request['addressType'] = $params['address_type'];
        $request['firstName'] = $params['first_name'];
        $request['lastName'] = $params['last_name'];
        $request['phoneNumber'] = $params['phone_number'];
        $request['postcode'] = $params['postcode'];
        $request['door'] = $params['door'];
        $request['street'] = $params['street'];
        $request['city'] = $params['city'];
        $request['primary'] = $params['primary'];

        # add new address detail
        $addAddress = $myAccount->addAddress($request);

        $responseData = [
            "status" => true, 
            "message" => "successfully added",
            "data" => $addAddress->insert_id,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;    
    }

    public function updateAddress(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'address_id' => $inputData['address_id'] ?? '',
			'address_type' => $inputData['address_type'] ?? '',
			'first_name' => $inputData['first_name'] ?? '',
			'last_name' => $inputData['last_name'] ?? '',
			'phone_number' => $inputData['phone_number'] ?? '',
			'postcode' => $inputData['postcode'] ?? '',
			'door' => $inputData['door'] ?? '',
			'street' => $inputData['street'] ?? '',
			'city' => $inputData['city'] ?? '',
			'primary' => $inputData['primary'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value) && $value!='0') {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}
    
        $myAccount = new Myaccount();
        $request = array();
        $request['addressId'] = $params['address_id'];
        $request['addressType'] = $params['address_type'];
        $request['firstName'] = $params['first_name'];
        $request['lastName'] = $params['last_name'];
        $request['phoneNumber'] = $params['phone_number'];
        $request['postcode'] = $params['postcode'];
        $request['door'] = $params['door'];
        $request['street'] = $params['street'];
        $request['city'] = $params['city'];
        $request['primary'] = $params['primary'];

        # update existing address detail
        $updateAddress = $myAccount->updateAddress($request);

        $responseData = [
            "status" => true, 
            "message" => "successfully updated",
            "data" => $updateAddress,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response; 
    }

    public function deleteAddress(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'address_id' => $inputData['address_id'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $addressId = $params['address_id'];

        # delete address detail
        $deleteAddress = $myAccount->deleteAddress($addressId);

        $responseData = [
            "status" => true, 
            "message" => "successfully deleted",
            "data" => $deleteAddress,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response; 

    }

    public function getBillDetail(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'order_id' => $inputData['order_id'] ?? '',
			'client_id' => $inputData['client_id'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $clientId = $params['client_id'];
        $orderId = $params['order_id'];

        # get order details
        $getOrderDetail = $myAccount->getOrderDetail($clientId, $orderId);

        $responseData = [
            "status" => true, 
            "message" => $getOrderDetail,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getOrderHistory(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'customer_id' => $inputData['customer_id'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $customerId = $params['customer_id'];
        if(isset($inputData['path'])) {
            $clientPath = $inputData['path'];
        } else {
            $clientPath = ""; 
        }

        # get customer order history
        $getOrderHistory = $myAccount->getOrderHistory($customerId, $clientPath);

        $responseData = [
            "status" => true, 
            "message" => $getOrderHistory['order'],
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function orderTracking(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'client_id' => $inputData['client_id'] ?? '',
			'order_id' => $inputData['order_id'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $clientId = $params['client_id'];
        $orderId = $params['order_id'];
        $co_ordinates=array(
            'latitude'=>$inputData['latitude'] ?? '',
            'longitude'=>$inputData['longitude'] ?? '',
        );

        # get customer order history
        $getOrderStatus = $myAccount->getOrderStatus($clientId, $orderId,$co_ordinates);

        $responseData = [
            "status" => true, 
            "message" => $getOrderStatus,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function orderAgain(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'client_id' => $inputData['client_id'] ?? '',
            'order_id' => $inputData['order_id'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

        if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $clientId = $params['client_id'];
        $orderId = $params['order_id'];

        $getOrderedItemsList = $myAccount->getOrderedItemsList($clientId, $orderId);

        $responseData = [
            "status" => true, 
            "message" => $getOrderedItemsList,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;

    }

    public function updateCustomerAddressPopup(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $fieldsToSanitize = ['firstName', 'lastName', 'phone', 'city', 'state', 'town'];
        $sanitizeApostrophes = function ($value) {
            return str_replace("'", "", $value);
        };
        foreach ($fieldsToSanitize as $field) {
            if (isset($inputData[$field])) {
                $inputData[$field] = $sanitizeApostrophes($inputData[$field]);
            }
        }
        if($inputData['orderMode'] == 0) {
            $params = [
                'firstName' => $inputData['firstName'] ?? '',
                'lastName' => $inputData['lastName'] ?? '',
                'phone' => $inputData['phone'] ?? '',
                'email' => $inputData['email'] ?? '',
                'door' => $inputData['door'] ?? '',
                'street' => $inputData['street'] ?? '',
                'city' => $inputData['city'] ?? '',
                'postcode' => $inputData['postcode'] ?? '',
            ];
        } else if($inputData['orderMode'] == 1) {
            $params = [
                'firstName' => $inputData['firstName'] ?? '',
                'lastName' => $inputData['lastName'] ?? '',
                'phone' => $inputData['phone'] ?? '',
                'email' => $inputData['email'] ?? '',
            ];
        } else {
            $responseData = [
                "status" => false, 
                "message" => "order mode missing",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $myAccount = new Myaccount();
        $updateAddress = $myAccount->updateAddressPopup($inputData);

        $responseData = [
            "status" => true, 
            "message" => $updateAddress,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;

    }

    public function sendConfirmMail(Request $request, Response $response, $args)
	{
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'domain' => $inputData['domain'] ?? '',
			'orderId' => $inputData['orderId'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $domain = $params['domain'];
        $orderId = $params['orderId'];
        
        # get path using domain name
        $getPath = $myAccount->getPathUsingDomain($domain);
        if(!$getPath->path)
        {
            $responseData = [
                "status" => false, 
                "message" => "Data not found",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        $status = 1;
        $myAccount->updateOrderStatus($getPath->id,$orderId,$status);

        $getOrderDetail = $myAccount->getMailOrderDetail($getPath->id, $orderId);
        $getOrderDetail['review'] = "/".$getPath->path."/rateus/".$orderId;

        if($getOrderDetail['user']['email'] == "") {
            $confirmMail = "User don't have a mail id, Order confirmed";
        } else {
            $pagesController = new PagesController();
            $confirmMail = $pagesController->confirmMail($getOrderDetail);
        }
        $responseData = [
            "status" => true, 
            "message" => $confirmMail,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function orderAcceptMail(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'domain' => $inputData['domain'] ?? '',
			'order_id' => $inputData['order_id'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $domain = $params['domain'];
        $orderId = $params['order_id'];

        # get path using domain name
        $getPath = $myAccount->getPathUsingDomain($domain);

        if(!$getPath->path)
        {
            $responseData = [
                "status" => false, 
                "message" => "Data not found",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $getOrderDetail = $myAccount->OrderacceptDetail($getPath->id, $orderId);
        $getOrderDetail['url'] = $domain;
        $getOrderDetail['orderDetailTime'] = $myAccount->getMailOrderDetail($getPath->id, $orderId);
        if($getOrderDetail['userEmail'] == "") {
            $rejectMail = "User don't have a mail id, Order Accept";
        } else {
            $pagesController = new PagesController();
            $client_id = $getPath->id;
            $client = New Client( $client_id);

            // Get currency info
            $location = new Location($client_id);
            $currency = $location->getCurrency();

            $getOrderDetail['orderDetails'] = $client->query("SELECT * FROM tbl_order WHERE id= $orderId");
           
            if($getOrderDetail['orderDetails']->ptype == "0") {
                $getOrderDetail['paymentType'] = 'cash';
            }if($getOrderDetail['orderDetails']->ptype == "1") {
                $getOrderDetail['paymentType'] = 'Card';

            }if($getOrderDetail['orderDetails']->ptype == "2") {
                $getOrderDetail['paymentType'] = 'Card at shop';

            }if($getOrderDetail['orderDetails']->ptype == "3") {
                $getOrderDetail['paymentType'] = 'Qr';

            }if($getOrderDetail['orderDetails']->ptype == "4") {
                $getOrderDetail['paymentType'] = 'Unpaid';

            }if($getOrderDetail['orderDetails']->ptype == "5") {
                $getOrderDetail['paymentType'] = 'Motto payment';

            }if($getOrderDetail['orderDetails']->ptype == "6") {
                $getOrderDetail['paymentType'] = 'Pay link';

            }if($getOrderDetail['orderDetails']->ptype == "7") {
                $getOrderDetail['paymentType'] = 'Google pay';

            }if($getOrderDetail['orderDetails']->ptype == "14") {
                $getOrderDetail['paymentType'] = 'Paypal';

            }if($getOrderDetail['orderDetails']->ptype == "9") {
                $getOrderDetail['paymentType'] = 'wallet';

            }if($getOrderDetail['orderDetails']->ptype == "11") {
                $getOrderDetail['paymentType'] = 'PDQ';

            }
            $time= $getOrderDetail['orderDetails']->datetime;
            $getOrderDetail['datetime'] =  date('m/d/Y H:i:s', $time);
            $getOrderDetail['userdetails'] = $client->query("SELECT * FROM tbl_order_user WHERE oid=$orderId");
            $getOrderDetail['itemDetails']['currency'] = $currency['symbol'];
            $getOrderDetail['orderDetails']['currency'] = $currency['symbol'];
            $getOrderDetail['itemDetails']['items'] = $client->queryall("SELECT * FROM tbl_order_items WHERE order_id=$orderId");
            $AcceptMail = $pagesController->AcceptMail($getOrderDetail);
        }

        $responseData = [
            "status" => true, 
            "message" => $AcceptMail,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }
    public function orderRejectMail(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'domain' => $inputData['domain'] ?? '',
			'orderId' => $inputData['orderId'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $domain = $params['domain'];
        $orderId = $params['orderId'];

        # get path using domain name
        $getPath = $myAccount->getPathUsingDomain($domain);

        if(!$getPath->path)
        {
            $responseData = [
                "status" => false, 
                "message" => "Data not found",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $getOrderDetail = $myAccount->getOrderRejectDetail($getPath->id, $orderId);
        $getOrderDetail['url'] = $domain;
        if($getOrderDetail['userEmail'] == "") {
            $rejectMail = "User don't have a mail id, Order Rejected";
        } else {
            $pagesController = new PagesController();
            $client_id = $getPath->id;
            $client = New Client( $client_id);
            $getOrderDetail['orderDetails'] = $client->query("SELECT * FROM tbl_order WHERE id= $orderId");
            if($getOrderDetail['orderDetails']->ptype == "0") {
                $getOrderDetail['paymentType'] = 'cash';
            }if($getOrderDetail['orderDetails']->ptype == "1") {
                $getOrderDetail['paymentType'] = 'Card';

            }if($getOrderDetail['orderDetails']->ptype == "2") {
                $getOrderDetail['paymentType'] = 'Card at shop';

            }if($getOrderDetail['orderDetails']->ptype == "3") {
                $getOrderDetail['paymentType'] = 'Qr';

            }if($getOrderDetail['orderDetails']->ptype == "4") {
                $getOrderDetail['paymentType'] = 'Unpaid';

            }if($getOrderDetail['orderDetails']->ptype == "5") {
                $getOrderDetail['paymentType'] = 'Motto payment';

            }if($getOrderDetail['orderDetails']->ptype == "6") {
                $getOrderDetail['paymentType'] = 'Pay link';

            }if($getOrderDetail['orderDetails']->ptype == "7") {
                $getOrderDetail['paymentType'] = 'Google pay';

            }if($getOrderDetail['orderDetails']->ptype == "8") {
                $getOrderDetail['paymentType'] = 'Apple pay';

            }if($getOrderDetail['orderDetails']->ptype == "14") {
                $getOrderDetail['paymentType'] = 'Paypal';
            }if($getOrderDetail['orderDetails']->ptype == "9") {
                $getOrderDetail['paymentType'] = 'wallet';
            }if($getOrderDetail['orderDetails']->ptype == "10") {
                $getOrderDetail['paymentType'] = 'Klarna';
            }if($getOrderDetail['orderDetails']->ptype == "11") {
                $getOrderDetail['paymentType'] = 'PDQ';
            }
            $time= $getOrderDetail['orderDetails']->rejectOrCancelTime;
            $Ordertime= $getOrderDetail['orderDetails']->datetime;
            $getOrderDetail['RejectTime'] =  date('m/d/Y H:i:s', $time);
            $getOrderDetail['OrderTime'] =  date('m/d/Y H:i:s', $Ordertime);
            $getOrderDetail['userdetails'] = $client->query("SELECT * FROM tbl_order_user WHERE oid=$orderId");
            $rejectMail = $pagesController->rejectMail($getOrderDetail);
        }

        $responseData = [
            "status" => true, 
            "message" => $rejectMail,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function orderdelivery(Request $request, Response $response, $args){

        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'domain' => $inputData['domain'] ?? '',
			'orderId' => $inputData['orderId'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $domain = $params['domain'];
        $orderId = $params['orderId'];

        # get path using domain name
        $getPath = $myAccount->getPathUsingDomain($domain);

        if(!$getPath->path)
        {
            $responseData = [
                "status" => false, 
                "message" => "Data not found",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        $getOrderDetail = $myAccount->getMailOrderDetail($getPath->id, $orderId);
        $getOrderDetail['url'] = $domain;
        if($getOrderDetail['user']['email'] == "") {
            $OrderDelivery = "User don't have a mail id, Order Delivery";
            $responseData = [
                "status" => true, 
                "message" => $OrderDelivery,
                "response_code" => 200,
            ];
        } else{
            $pagesController = new PagesController();
            $OrderDelivery = $pagesController->DeliveryMail($getOrderDetail);

            $responseData = [
                "status" => true, 
                "message" => $OrderDelivery,
                "response_code" => 200,
            ];

        }
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function createMyaccountFeedback(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'clientId' => $inputData['clientId'] ?? '',
			'orderId' => $inputData['orderId'] ?? '',
			'customerId' => $inputData['customerId'] ?? '',
			'rating' => $inputData['rating'] ?? '',
			'foodQuality' => $inputData['foodQuality'] ?? '',
			'comments' => $inputData['comments'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $customer = $myAccount->getProfile($params['customerId']);
        $params['name'] = $customer->fname.' '.$customer->lname;
        $params['email'] = $customer->email;
        $params['phone'] = $customer->phone;
        $createFkFeedback = $myAccount->createFkFeedback($params);

        
        $push_notification_type="feedback_received";
        $MyAccountController=new MyAccountController();
        $push_notification=$MyAccountController->pushNotificationCurl($push_notification_type,$inputData['clientId'], $params['name'],$params['orderId'] );

        $responseData = [
            "status" => true, 
            "message" => $createFkFeedback,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }


    public function  validateAndModifyMobileNumber($mobileNumber) {
        // Remove any non-numeric characters
        $mobileNumber = preg_replace('/\D/', '', $mobileNumber);

        // Check if the mobile number starts with '0'
        if (substr($mobileNumber, 0, 1) === '0') {
            // Replace the '0' with '44'
            $mobileNumber = '44' . substr($mobileNumber, 1);
        }

        if (substr($mobileNumber, 0, 2) === '44' && strlen($mobileNumber) === 12) {
            return $mobileNumber;
        }

        if (substr($mobileNumber, 0, 2) != '44' && strlen($mobileNumber) === 10) {

            $mobileNumber = '44' . substr($mobileNumber, 0);
            return $mobileNumber;
        }

        if (substr($mobileNumber, 0, 1) != '0' && strlen($mobileNumber) === 10) {
            
            $mobileNumber = '44' . substr($mobileNumber, 0);
            return $mobileNumber;
        }

        return false; // Invalid mobile number
    }
    

    public function sendOrderSms(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'domain' => $inputData['domain'] ?? '',
			'orderId' => $inputData['orderId'] ?? '',
			'status' => $inputData['status'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $domain = $params['domain'];
        $orderId = $params['orderId'];
        $status = $params['status'];

        # get path using domain name
        $getPath = $myAccount->getPathUsingDomain($domain);
        if(!$getPath->path)
        {
            $responseData = [
                "status" => false, 
                "message" => "Data not found",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $getOrderDetail = $myAccount->getOrderSmsDetail($getPath->id, $orderId);

        if($getOrderDetail['userPhone'] == "") {
            $responseData = [
                "status" => false, 
                "message" => "User don't have a phone number, SMS not sended",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        } else {


            $mobilenumber = $this->validateAndModifyMobileNumber($getOrderDetail['userPhone']);

            if(!$mobilenumber){

            $responseData = [
                "status" => false, 
                "message" => "Invalid Number",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;

            }

            // $orderMode = array("Delivery","Collection");
            if($status == "1"){

                $message = "Hi ".$getOrderDetail['userFirstName']." ".$getOrderDetail['userLastName'].", Good news! ".$getOrderDetail['restaurantName']." has successfully accepted your order. For any queries or special requests, please contact us at ".$getOrderDetail['restaurantPhone']."";
            
            }

            if($status == "2" || $status == "4"){

                 $message = "Hi".$getOrderDetail['userFirstName']." ".$getOrderDetail['userLastName']." We regret to inform you that order ".$getOrderDetail['orderId']."  has been cancelled by ".$getOrderDetail['restaurantName'].". If paid, your refund will be processed within 3-5 working days. For assistance, contact us at ".$getOrderDetail['restaurantPhone'].".";
            }

            if(($status == "3" && $getOrderDetail['isStuartEnable'] == 0) || ($status == "3" && $getOrderDetail['isStuartEnable'] == 1 && $getOrderDetail['orderType'] == 1) || ($status == "3" && $getOrderDetail['isCollected'] == 'delivered' && $getOrderDetail['isStuartEnable'] == 1 && $getOrderDetail['orderType'] == 0)){
                $url = "https://".$domain."/rate";

                $message =  "Hi ".$getOrderDetail['userFirstName']." ".$getOrderDetail['userLastName'].",  Exciting news! Your delicious order from ".$getOrderDetail['restaurantName']." has just arrived at your doorstep! Got any questions or need assistance? Reach out to us at ".$getOrderDetail['restaurantName'].". We're here to make your dining experience extraordinary.Don't forget to share your thoughts with us! Your feedback helps us serve you better. Click here: ".$url." Thank you for choosing ".$getOrderDetail['restaurantName'].".";

            }

            if(isset($message)) {
                $user = new User();
                $sendSms = $user->sendOtpSms($mobilenumber, $message);    
                $responseData = [
                    "status" => true, 
                    "message" => $sendSms,
                    "response_code" => 200,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            } else {
                $responseData = [
                    "status" => false, 
                    "message" => "no number",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }
    }

    public function getClientFeedbackStatus(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'path' => $inputData['path'] ?? '',
			'orderId' => $inputData['orderId'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new Myaccount();
        $path = $params['path'];
        $orderId = $params['orderId'];

        # get client id using path
        $getPath = $myAccount->getClientUsingB2cPath($path);

        if(!$getPath->id)
        {
            $responseData = [
                "status" => false, 
                "message" => "Data not found",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        
        $getClientFeedback = $myAccount->getClientFeedback($orderId, $getPath->id);
       
        if($getClientFeedback != false) {
            $responseData = [
                "status" => true, 
                "message" => $getClientFeedback,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false, 
                "message" => "No Data",
                "response_code" => 400,
            ];
        }
        
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function testing(Request $request, Response $response, $args)
	{

        $value = "CONNECT2FUSIONRDS211990";
        $key = "(^%(^@@&25061991";
        // Encrypt the value
        $encryptedValue = openssl_encrypt($value, 'AES-128-ECB', $key);

        // Decrypt the value
        $decryptedValue = openssl_decrypt($encryptedValue, 'AES-128-ECB', $key);

        $result = [
            'encrypted' => $encryptedValue,
            'decrypted' => $decryptedValue
        ];

        print_r($result);
        
        phpinfo();

    }


    public function send_otp(Request $request, Response $response, $args){
        $inputData = json_decode($request->getBody(), true);
        $otp = isset($inputData['otp']) ? $inputData['otp'] : null;
        $to = isset($inputData['to']) ? $inputData['to'] : null;
        
        if(empty($otp) || empty($to)){
            $response->getBody()->write("Please provide both OTP and recipient phone number.");
            return $response->withStatus(400);
        }


        $otp = htmlspecialchars($otp);
        $to = htmlspecialchars($to);

        $orig = ""; 
        $pwd = $_ENV["SMS_PASSWORD"];
        $usr = $_ENV["SMS_FROM"];
        $txt = !empty($inputData['request_from']) && $inputData['request_from'] == 'partneradmin'  ? html_entity_decode($otp) : "Your OTP is: $otp";
        
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => "http://api.infobip.com/sms/1/text/single",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => json_encode(array("from" => $orig, "to" => [$to], "text" => $txt)),
            CURLOPT_HTTPHEADER => array(
                "accept: application/json",
                "authorization: Basic " . base64_encode($usr . ":" . $pwd),
                "content-type: application/json"
            ),
        ));

        $responseBody = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);

        if ($err) {
            $response->getBody()->write("cURL Error: " . $err);
            return $response->withStatus(500);
        } else {
            $response->getBody()->write($responseBody);
            return $response->withStatus(200);
        }
    }
    public function pushNotificationCurl($type,$client_id, $order_id=0,$feedback_order_id=0)
    {
        $fcm_push_array=array();
        switch ($type) {
            case 'order_placed':
                $fcm_push_array = array(
                    "type" => "order_placed",
                    "client_id" => $client_id,
                    "customer_name"=>$order_id,
                );
                // $order_id=0;
                break;
            
            case 'order_rejected':
                $fcm_push_array = array(
                    "type" => "order_rejected",
                    "client_id" => $client_id,
                    "customer_name"=>$order_id,
                );
                break;
            case 'feedback_received':
                $fcm_push_array = array(
                    "type" => "feedback_received",
                    "client_id" => $client_id,
                    "customer_name"=>$order_id,
                    'order_date_time'=>null,
                    'feedback_order_id'=>$feedback_order_id
                );
                $order_id=$feedback_order_id;
                break;    
        }
        
        
        $send_notification_url = ($_SERVER['SERVER_NAME'] == "localhost") ? "http://localhost:8080/sendPushNotification" 
            : (($_SERVER['SERVER_NAME'] == "newfkapi.fusiontesting.co.uk") ? "https://api-newdev.fmb.eposapi.co.uk/sendPushNotification" : "https://api-new.fmb.eposapi.co.uk/sendPushNotification");
              
        $client = new Client($client_id);
        $getFcmHistoryByOrderId = $client->query("CALL getFcmHistoryByOrderId($order_id,'$type')");
        $order_id_history="";
        
       
        if(isset($getFcmHistoryByOrderId->order_id) )
        {
            $order_id_history=$getFcmHistoryByOrderId->order_id;
        }
      
        if($order_id_history==""  || $order_id_history=="0" || $order_id_history==0)
        {
            $curl = curl_init();
            curl_setopt_array($curl, array(
                CURLOPT_URL => $send_notification_url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => json_encode($fcm_push_array),
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: application/json'
                ),
            ));
            $response = curl_exec($curl);
            // echo $response;
            curl_close($curl);
            $getFcmHistoryByOrderId = $client->query("CALL getFcmHistoryByOrderId($order_id,'$type')");
            $order_id_history="";
            if(isset($getFcmHistoryByOrderId->order_id))
            {
                $order_id_history=$getFcmHistoryByOrderId->order_id;
            }
            else
            {
                
                $result_array=array(
                    'client_id'=>$client_id,
                    'success_response'=>"0",
                    'failure_response'=>"0",
                    "type"=>"order_rejected",
                    "order_id"=>$order_id,
                    "from"=>"from_fk",
                );
                if($type=="order_rejected")
                {
                    $insertFcmHistory=$this->insertFcmHistory($result_array);
                }
                
            }
        }
        
            
        
       
        
      
        return true;
    }
    public function insertFcmHistory($result_array)
    {
        date_default_timezone_set('Europe/London');
        $date=Date("Y-m-d H:i:s");
        $clientId=$result_array['client_id'];
        $client_connection = new Client($clientId);
        $success_response=$result_array['success_response'];
        $failure_response=$result_array['failure_response'];
        $type=$result_array['type'];
        $order_id_send=$result_array['order_id'];
        $client = new Client($clientId);
        $DB=new DB();
        $get_type_id="";
        $get_type_id_query=$DB->query("CALL getFcmTypeId('$type')");
        if(isset($get_type_id_query->id))
        {
            $get_type_id=$get_type_id_query->id;
        }
        $insertFcmHistory_log = $client->query("CALL insertFcmHistory('$get_type_id','$type','$order_id_send','$success_response','$failure_response','A','$date','$date',@new_id)");
        return $insertFcmHistory_log;
    }
    public function getTableHistory(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'customer_id' => $inputData['customer_id'] ?? '',
            'email' => $inputData['email'] ?? '',
		];
		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}
		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}
        $myAccount = new Myaccount();
        
        $getTableookingHistory=$myAccount->getTablebookingHistory($params);
        $responseData = [
            "status" => true, 
            "message" => $getTableookingHistory,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }
    public function updateTblFKDomain(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'order_id' => $inputData['order_id'] ?? '',
            'type' =>$inputData['type'] ?? '',
            
		];
		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}
		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}
        $myAccount = new Myaccount();
        $updateTableStatusFk='';
        $client_id=$inputData['client_id'] ?? '';
        if($client_id=="" || $client_id=='')
        {
            $client = (new Myaccount())->getPathUsingDomain($inputData['domain']);
            $client_id=$client->id;
       
        }
        $responseData = [
            "status" => true, 
            "message" => '',
            "response_code" => 200,
        ];
        if(isset($client_id))
        {
            if($params['type']=="hide")
            {
                $updateTableStatusFk=$myAccount->hideTblStatusFK($client_id,$params['order_id']);
            }
            else
            {
                $updateTableStatusFk=$myAccount->updateTblOrderStatusFk($client_id,$params['order_id'],$inputData['status']);
            }
            
            $responseData = [
                "status" => true, 
                "message" => $updateTableStatusFk->inserted_id,
                "response_code" => 200,
            ];
        }
       
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }
    function getFavouriteList(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'customer_id' => $inputData['customer_id'] ?? '',
            // 'orderTime'=>$inputData['orderTime'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $myAccount = new myAccount();
        $order_time_get=date('Y-m-d H:i');
        $restaurant_details=$myAccount->getFavouriteList($params['customer_id'],$order_time_get);
        
      
        $responseData = [
            "status" => true, 
            "message" => $restaurant_details,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;

        

    }
}


?>