<?php
namespace App\Controllers;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use App\Helpers\SendEmail;
use App\Helpers\SmsValidation;
use App\Helpers\MailValidation;
use Slim\Views\Twig;
use Slim\Views\TwigMiddleware;
use Twig\Environment;
use Twig\Loader\FilesystemLoader;
use App\Helpers\s3;
use App\Models\Page;


class PagesController
{

	protected $twig;

	public function __construct()
	{
		$loader = new FilesystemLoader(__DIR__ .'/../view/Templates');
		$this->twig = new Environment($loader, [
			'cache' => false, 
		]);
	}

	public function getHelpPageList(Request $request, Response $response, $args)
	{
		$inputData = json_decode(file_get_contents('php://input'), true);
		$page = new Page();
		$helpPageDatas = array();
		if(isset($inputData["pageId"]) && $inputData["pageId"] != "") {
			$pageId = $inputData["pageId"];
		} else {
			$pageId = "0";
		}

		# get all help content come from DB
		$helpPageDatas = $page->getHelpPageListById($pageId);

		$responseData = [
			"status" => true, 
			"message" => "Help List Page",
			"data" => $helpPageDatas,
			"response_code" => 200,
		];
		$response->getBody()->write(json_encode($responseData));
		return $response;
	}

	public function resellerSignup(Request $request, Response $response, $args)
	{
		$inputData = json_decode(file_get_contents('php://input'), true);
		$params = [
			'name' => $inputData['name'] ?? '',
			'email' => $inputData['email'] ?? '',
			'phone' => $inputData['phone'] ?? '',
			'business_name' => $inputData['business_name'] ?? '',
			'business_vertical' => $inputData['business_vertical'] ?? '',
			'postcode' => $inputData['postcode'] ?? '',
			'ip' => $inputData['ip'] ?? '',
			'agent' => $inputData['agent'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		$page = new Page();
		$sendEmail = new SendEmail();
		$validMail = new MailValidation();
		$validSms = new SmsValidation();

		# email id validation
		$mail = $validMail->checkEmail($params['email']);
		if($mail!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid Email",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# phone number validation
		$sms = $validSms->checkUkNumber($params['phone']);
		if($sms!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid contact Number",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# insert function for reseller page
		$record = $page->createReseller($params);

		# lead mail sended to company person
		$resellerTo = $_ENV['HARI'];
		$resellerCc = array($_ENV['AJAY'],$_ENV['MARKETING'],$_ENV['SALES']);
		$resellerSub = "Fusion Kitchen Reseller";
		$resellerHtml = $this->twig->render('reseller.twig', $params);
		$resellerMail = $sendEmail->sendMail($resellerTo,$resellerCc,$resellerSub,$resellerHtml);

		# acknowledgement mail content
		$params['social'] = $_ENV['SOCIAL'];
		$resellerAckTo = $params['email'];
		$resellerAckCc = array();
		$resellerAckSub = "Thank you for reselling Fusion Kitchen";
		$resellerAckHtml = $this->twig->render('acknowledgement.twig', $params);
		$resellerAckMail = $sendEmail->sendMail($resellerAckTo,$resellerAckCc,$resellerAckSub,$resellerAckHtml);
			
		$responseData = [
			"status" => true, 
			"data" => "Form submitted",
			"response_code" => 200,
		];
		$response->getBody()->write(json_encode($responseData));
		return $response;
	}

	public function contactlessDinning(Request $request, Response $response, $args)
	{
		$inputData = json_decode(file_get_contents('php://input'), true);
		$params = [
			'name' => $inputData['name'] ?? '',
			'restaurant_name' => $inputData['restaurant_name'] ?? '',
			'postcode' => $inputData['postcode'] ?? '',
			'email' => $inputData['email'] ?? '',
			'phone' => $inputData['phone'] ?? '',
			'ip' => $inputData['ip'] ?? '',
			'agent' => $inputData['agent'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		$page = new Page();
		$sendEmail = new SendEmail();
		$validMail = new MailValidation();
		$validSms = new SmsValidation();

		# email id validation
		$mail = $validMail->checkEmail($params['email']);
		if($mail!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid Email",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# phone number validation
		$sms = $validSms->checkUkNumber($params['phone']);
		if($sms!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid contact Number",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# insert function for contactless page
		$record = $page->createContactless($params);

		# lead mail sended to company person
		$contactlessTo = $_ENV['HARI'];
		$contactlessCc = array($_ENV['MARKETING'],$_ENV['SALES']);
		$contactlessSub = "Contactless Dinning - Lead";
		$contactlessHtml = $this->twig->render('contactless.twig', $params);
		$contactlessMail = $sendEmail->sendMail($contactlessTo,$contactlessCc,$contactlessSub,$contactlessHtml);

		# acknowledgement mail content
		$params['social'] = $_ENV['SOCIAL'];
		$contactlessAckTo = $params['email'];
		$contactlessAckCc = array();
		$contactlessAckSub = "Thank you for contacting Fusion Kitchen";
		$contactlessAckHtml = $this->twig->render('acknowledgement.twig', $params);
		$contactlessAckMail = $sendEmail->sendMail($contactlessAckTo,$contactlessAckCc,$contactlessAckSub,$contactlessAckHtml);
			
		$responseData = [
			"status" => true, 
			"data" => "Form submitted",
			"response_code" => 200,
		];
		$response->getBody()->write(json_encode($responseData));
		return $response;
	}

	public function careerForm(Request $request, Response $response, $args)
	{
		// $inputData = json_decode(file_get_contents('php://input'), true);
		$inputData = $_POST;
		$params = [
			'first_name' => $inputData['first_name'] ?? '',
			'last_name' => $inputData['last_name'] ?? '',
			'email' => $inputData['email'] ?? '',
			'position' => $inputData['position'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		$sendEmail = new SendEmail();
		$validMail = new MailValidation();

		# email id validation
		$mail = $validMail->checkEmail($params['email']);
		if($mail!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid Email",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# file upload to innovative server
		if (isset($_FILES['resume'])) {
			$file = $_FILES['resume'];
			$file_name = $file['name'];
			$file_type = $file['type'];
			$file_size = $file['size'];
			$file_tmp_name = $file['tmp_name'];

			$exp = explode('.',$file_name);
			$file_ext=strtolower($exp[1]);

			$expansions= array("pdf","doc","docx");
			if(in_array($file_ext,$expansions) === false){
				$responseData = [
					"status" => false, 
					"data" => "Extension not allowed, please choose a .pdf, .doc, .docx file.",
					"response_code" => 400,
				];
				$response->getBody()->write(json_encode($responseData));
				return $response;
			}
			if($file_size > 3 * 1024 * 1024){
				$responseData = [
					"status" => false, 
					"data" => "File size must be max 3 MB",
					"response_code" => 400,
				];
				$response->getBody()->write(json_encode($responseData));
				return $response;
			}
			$unique_id = rand();
			$new_file_name = $unique_id.'.'.$file_ext;
			$moved = move_uploaded_file($file_tmp_name, '/home/fusioninnovative/public_html/mail/upload_files/' . $new_file_name);

			if($moved === false){
				$responseData = [
					"status" => false, 
					"data" => "Your resume was not uploaded",
					"response_code" => 400,
				];
				$response->getBody()->write(json_encode($responseData));
				return $response;
			}

			$params['resume'] = "https://www.fusioninnovative.com/mail/upload_files/".$new_file_name;
		}

		# lead mail sended to company person
		$contactlessTo = $_ENV['HR'];
		$contactlessCc = array($_ENV['HARI']);
		$contactlessSub = "Job application from Fusion Kitchen";
		$contactlessHtml = $this->twig->render('career.twig', $params);
		$contactlessMail = $sendEmail->sendMail($contactlessTo,$contactlessCc,$contactlessSub,$contactlessHtml);

		# acknowledgement mail content
		$contactlessAckTo = $params['email'];
		$contactlessAckCc = array();
		$contactlessAckSub = "Thank you";
		$contactlessAckHtml = $this->twig->render('careerAcknowledgement.twig', $params);
		$contactlessAckMail = $sendEmail->sendMail($contactlessAckTo,$contactlessAckCc,$contactlessAckSub,$contactlessAckHtml);
		
		$responseData = [
			"status" => true, 
			"data" => "Form submitted",
			"response_code" => 200,
		];
		$response->getBody()->write(json_encode($responseData));
		return $response;
	}

	public function partnerSignup(Request $request, Response $response, $args)
	{
		// $inputData = json_decode(file_get_contents('php://input'), true);
		$inputData = $_POST;
		$params = [
			'type' => $inputData['type'] ?? '',
			'takeaway_name' => $inputData['takeaway_name'] ?? '',
			'phone' => $inputData['phone'] ?? '',
			'email' => $inputData['email'] ?? '',
			'delivery_charge' => $inputData['delivery_charge'] ?? '',
			'opening_time' => $inputData['opening_time'] ?? '',
			'ip' => $inputData['ip'] ?? '',
			'agent' => $inputData['agent'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}
			
		$page = new Page();
		$sendEmail = new SendEmail();
		$validMail = new MailValidation();
		$validSms = new SmsValidation();

		# email id validation
		$mail = $validMail->checkEmail($params['email']);
		if($mail!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid Email",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# phone number validation
		$sms = $validSms->checkUkNumber($params['phone']);
		if($sms!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid contact Number",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# file uploaded to s3 bucket
		if (isset($_FILES['menu'])) {

			$s3 = new s3();

			$file = $_FILES['menu'];
			$file_name = $file['name'];
			$file_type = $file['type'];
			$file_size = $file['size'];
			$file_tmp_name = $file['tmp_name'];

			$exp = explode('.',$file_name);
			$file_ext=strtolower($exp[1]);

			$expansions= array("pdf", "git", "jpg", "tif", "jpeg", "svg", "png", "doc", "docx", "docm", "csv", "xls", "xlsx", "xlsm");
			if(in_array($file_ext,$expansions) === false){
				$responseData = [
					"status" => false, 
					"data" => "Extension not allowed, please choose another file formate.",
					"response_code" => 400,
				];
				$response->getBody()->write(json_encode($responseData));
				return $response;
			}
			if($file_size > 3 * 1024 * 1024){
				$responseData = [
					"status" => false, 
					"data" => "File size must be max 3 MB.",
					"response_code" => 400,
				];
				$response->getBody()->write(json_encode($responseData));
				return $response;
			}
			$unique_id = rand();
			$new_file_name = $unique_id.'.'.$file_ext;
			$status = $s3->putFilesToObject('partner/'.$new_file_name,$file_tmp_name);

			if($status['@metadata']['statusCode']==200) {
				$params['menu'] = $status['ObjectURL'];
				$params['menuName'] = $new_file_name;
			} else {
				$responseData = [
					"status" => false, 
					"data" => "Your resume was not uploaded.",
					"response_code" => 400,
				];
				$response->getBody()->write(json_encode($responseData));
				return $response;
			}
		}
		
		# insert function for partner page
		$record = $page->createPartner($params);

		# lead mail sended to company person
		$contactlessTo = $_ENV['HARI'];
		$contactlessCc = array($_ENV['AJAY'],$_ENV['MARKETING'],$_ENV['SALES']);
		$contactlessSub = "Partner With us";
		$customer=[];
		$customer['takeaway_name']=$params['takeaway_name'];
		$contactlessHtml = $this->twig->render('becomePartner.twig', $customer);
		$contactlessMail = $sendEmail->sendMail($contactlessTo,$contactlessCc,$contactlessSub,$contactlessHtml);

		# acknowledgement mail content
		$params['social'] = $_ENV['SOCIAL'];
		$contactlessAckTo = $params['email'];
		$contactlessAckCc = array();
		$contactlessAckSub = "Thank you for contacting Fusion Kitchen";
		$contactlessAckHtml = $this->twig->render('acknowledgement.twig', $params);
		$contactlessAckMail = $sendEmail->sendMail($contactlessAckTo,$contactlessAckCc,$contactlessAckSub,$contactlessAckHtml);
			

		$responseData = [
			"status" => true, 
			"data" => "Form submitted",
			"response_code" => 200,
		];
		$response->getBody()->write(json_encode($responseData));
		return $response;
	}

	public function leadSignup(Request $request, Response $response, $args)
	{
		$inputData = json_decode(file_get_contents('php://input'), true);
		$params = [
			'name' => $inputData['name'] ?? '',
			'email' => $inputData['email'] ?? '',
			'phone' => $inputData['phone'] ?? '',
			'banner_type' => $inputData['banner_type'] ?? '',
			'ip' => $inputData['ip'] ?? '',
			'agent' => $inputData['agent'] ?? '',
		];

		$errors = [];

		foreach ($params as $key => $value) {
			if (empty($value)) {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		$page = new Page();
		$sendEmail = new SendEmail();
		$validMail = new MailValidation();

		if($params['banner_type'] == 1) {
			$params['type'] = "Google";
		} else if($params['banner_type'] == 2) {
			$params['type'] = "Facebook";
		} else if($params['banner_type'] == 3) {
			$params['type'] = "Others";
		}

		# email id validation
		$mail = $validMail->checkEmail($params['email']);
		if($mail!='true') {
			$responseData = [
				"status" => false, 
				"data" => "Invalid Email",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		# insert function for partner page
		$record = $page->createLeadSignup($params);

		# lead mail sended to company person
		$leadTo = $_ENV['HARI'];
		$leadCc = array($_ENV['AJAY'],$_ENV['MARKETING'],$_ENV['SALES']);
		$leadSub = "Lead Signup";
		$leadHtml = $this->twig->render('leadSignup.twig', $params);
		$leadMail = $sendEmail->sendMail($leadTo,$leadCc,$leadSub,$leadHtml);

		# acknowledgement mail content
		$params['social'] = $_ENV['SOCIAL'];
		$leadAckTo = $params['email'];
		$leadAckCc = array();
		$leadAckSub = "Thank you for contacting Fusion Kitchen";
		$leadAckHtml = $this->twig->render('acknowledgement.twig', $params);
		$leadAckMail = $sendEmail->sendMail($leadAckTo,$leadAckCc,$leadAckSub,$leadAckHtml);
			
		$responseData = [
			"status" => true, 
			"data" => "Form submitted",
			"response_code" => 200,
		];
		$response->getBody()->write(json_encode($responseData));
		return $response;
	}

	public function confirmMail($orderDetail)
	{
		$sendEmail = new SendEmail();
		
		# confirm mail 
		$orderDetail['social'] = $_ENV['SOCIAL'];
		$toMail = $orderDetail['user']['email'];
		$ccMail = array();
		$mailSub = "Your order's in the kitchen";
		$reply = $_ENV["SUPPORT"];
		$replyName = "Support";
		$mailHtml = $this->twig->render('confirmMail.twig', $orderDetail);
		$mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);

		return $mail;
	}

	public function AcceptMail($orderDetail)
	{
		$sendEmail = new SendEmail();
		
		# reject mail 
		$data = $orderDetail;
		$data['social'] = $_ENV['SOCIAL'];
		$data['acceptorder'] = $_ENV["ORDER_ACCEPT"];
		$toMail =$orderDetail['userEmail'];
		$ccMail = array();
		$mailSub = "Your order has been Accepted";
		$reply = $orderDetail['restaurantEmail'];
		$replyName = " ";
		$from = $orderDetail['restaurantEmail'];
		$fromName = " ";
		$data['content']="Accept";
		$clientLogo=$orderDetail['restaurantLogo'];
		$mailHtml = $this->twig->render('AcceptOrder.twig', $data);
		// print_r($mailHtml);die;
		$mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName,$from,$fromName);
		return $mail;
	}

	public function rejectMail($orderDetail)
	{
		$sendEmail = new SendEmail();
		
		# reject mail 
		$data = $orderDetail;
		$data['social'] = $_ENV['SOCIAL'];
		$data['rejectIcon'] = $_ENV["ORDER_REJECT"];
		$toMail =$orderDetail['userEmail'];
		$ccMail = array();
		$mailSub = "Sorry your order has been Rejected";
		$reply = $orderDetail['restaurantEmail'];
		$replyName = " ";
		$from = $orderDetail['restaurantEmail'];
		$fromName = " ";
		$data['content']="rejected";
		$clientLogo=$orderDetail['restaurantLogo'];
		$mailHtml = $this->twig->render('rejectMail.twig', $data);
		$mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName,$from,$fromName);
        
		
		return $mail;
	}

	public function DeliveryMail($orderDetail)
	{
		$sendEmail = new SendEmail();
		
		# reject mail 
		$data = $orderDetail;
		$data['social'] = $_ENV['SOCIAL'];
		$toMail =$orderDetail['user']['email'];
		$ccMail = array();
		$mailSub = "Your Order Has Landed! Time to Dig In";
		$reply = $orderDetail['restaurant']['restaurantEmail'];
		$replyName = "";
		$from = $orderDetail['restaurant']['restaurantEmail'];
		$fromName = "";
		$data['content']="Delivery";
		$clientLogo=$orderDetail['restaurantLogo'];
		$mailHtml = $this->twig->render('orderdelivery.twig', $data);
		$mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName,$from,$fromName);
		return $mail;
	}

	public function noTakeawayMail($data)
	{
		$sendEmail = new SendEmail();
		
		# no takeaway mail
		// $data['social'] = $_ENV['SOCIAL'];
		$toMail = $_ENV["SUPPORT"];
		// $toMail = $data['email'];
		$ccMail = array();
		$mailSub = "NO TAKEAWAYS";
		$reply = $_ENV["SUPPORT"];
		$replyName = "Support";
		$mailHtml = $this->twig->render('noTakeawayMail.twig',$data);
		$mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);
        if($mail){
		$email=$data['email'];
		$mailTo = $email;
		// $toMail = $data['email'];
		$mailCc = array();
		$subMail = "TAKEAWAYS";
		$htmlMail = $this->twig->render('takeAway.twig',$data);
		$mailTake = $sendEmail->sendMail($mailTo,$mailCc,$subMail,$htmlMail);
		}
		return $mailTake;
	}
    public function welcomeMail($data)
	{
		$sendEmail = new SendEmail();
		
		# no takeaway mail
		// $data['social'] = $_ENV['SOCIAL'];
		$toMail = $data['email'];
		$ccMail = array();
		$mailSub = "NO TAKEAWAYS";
		$reply = $_ENV["SUPPORT"];
		$replyName = "Support";
		$mailHtml = $this->twig->render('welcome.twig', $data);
		$mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);

		return $mail;
	}
	public function placeOrderMail($data)
	{
		$sendEmail = new SendEmail();
		
		# confirm mail 
		$data['social'] = $_ENV['SOCIAL'];
		$data['bgImage'] = $_ENV["ORDER_PLACED"];
		$protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
		$toMail = $data['userDetail']['email'];
		
		$queryString = 'email=' . $toMail;
		$encodedData = base64_encode($queryString);
		$data['unsubscribe'] = $protocol . $_SERVER['HTTP_HOST'] .'/unsubscribe?unsubdata=' . urlencode($encodedData);
		
		$ccMail = array();
		$mailSub = "Order Placed";
		$reply = $_ENV["SUPPORT"];
		$replyName = "Support";
		$mailHtml = $this->twig->render('orderPlaced.twig', $data);
		
		$mail = $sendEmail->sendMail($toMail,$ccMail,$mailSub,$mailHtml,$reply,$replyName);

		$responseData = [
			"status" => true, 
			"data" => $mail,
			"response_code" => 200,
		];
		return $responseData;
	}


	public function __destruct() 
	{
		$this->twig = null;
	}

}

?>