<?php
namespace App\Controllers;
use App\Models\Location;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Stripe\Stripe;
use Stripe\Exception\ApiErrorException;
use App\Models\Payment;
use App\Models\Menu;
use App\Models\Myaccount;
use Exception;

require '../config/env.php';

class PaymentController
{
    public function walletDashboard(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'customer_id' => $inputData['customer_id'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $payment = new Payment();
        $customerId = $params['customer_id'];
        $getWallet = $payment->getWallet($customerId);

        $responseData = [
            "status" => true, 
            "message" => "success",
            "data" => $getWallet,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getWalletHistory(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'customer_id' => $inputData['customer_id'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        
        $payment = new Payment();
        $totalTransaction = array();
        $transactionValue = array();
        $customerId = $params['customer_id'];
        $getTransaction = $payment->getTransaction($customerId);

        $clients = [];
        
        $menu = new Menu();

        # wallet history
        foreach ($getTransaction as $transaction) {
            $clientId = $transaction->clientId;

            if(isset($clients[$clientId]) && isset($clients[$clientId]['clientDetail'])) {
                $clientDetail = $clients[$clientId]['clientDetail'];
            } else {
                $clientDetail = $menu->getContacts($clientId);
                $clients[$clientId]['clientDetail'] = $clientDetail;
            }

            if(isset($clients[$clientId]) && isset($clients[$clientId]['currency'])) {
                $currency = $clients[$clientId]['currency'];
            } else {
                $location = new Location($clientId);
                $currency = $location->getCurrency();
                $clients[$clientId]['currency'] = $currency;
            }

            $transactionValue['type'] = $transaction->type;
            $transactionValue['date'] = date("M d, Y", $transaction->datetime);
            $transactionValue['orderId'] = $transaction->orderId;
            if ($transaction->type == 1) {
                $transactionValue['History'] = "Refund from " . $clientDetail->name;
                $transactionValue['amount'] = $currency['symbol'] . $payment->decrypt($transaction->amount);
            } else {
                $transactionValue['History'] = "Paid for " . $clientDetail->name;
                $transactionValue['amount'] = "-" . $currency['symbol'] . $payment->decrypt($transaction->amount);
            }
            $totalTransaction[] = $transactionValue;
        }

        $responseData = [
            "status" => true, 
            "message" => "success",
            "data" => $totalTransaction,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getSaveCard(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'customer_id' => $inputData['customer_id'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $payment = new Payment();
        $saveCardResponse = array();
        $cardDetail = array();
        $customerId = $params['customer_id'];

        $getCustomerStripeConnection = $payment->getCustomerStripeConnection($customerId);
        $getSaveCard = $payment->getSaveCard($customerId);

        if(!empty($getSaveCard))
        {
            $customerPaymentId = $getCustomerStripeConnection['paymentMode'] == 0 ? $getSaveCard->test_pay_customer_id : $getSaveCard->pay_customer_id;
            $customerPaymentValue = $payment->decrypt($customerPaymentId);

            # Call the Stripe API to get a list of payment methods
            $customerCard = $payment->stripePaymentMethod($getCustomerStripeConnection['skey'], $customerPaymentValue);

            if(!isset($customerCard['data']))
            {
                $responseData = [
                    "status" => false, 
                    "message" => $customerCard,
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }

            $saveCardCount = count($customerCard['data']);

            $saveCardResponse['saveCardCount'] = $saveCardCount;

            if($getSaveCard->finger_print == "") 
            {
                if($saveCardCount != "0") 
                {
                    $payment->updatePrimarySaveCard($customerId, $customerCard['data'][0]['card']->fingerprint);
                }
            }

            foreach($customerCard['data'] as $card){
                $getCardDetail = $this->getCardDetail($card, $getSaveCard,  $saveCardCount);
                $saveCardResponse['cardDetail'][] = $getCardDetail;
            }
        }
        else
        {
            $saveCardResponse['saveCardCount'] = "1"; 
        }
        
        $responseData = [
            "status" => true, 
            "message" => "your save card datas",
            "data" => $saveCardResponse,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getCardDetail($card, $getSaveCard, $saveCardCount)
    {
        # payment method id
        $cardDetail['paymentMethodId'] = $card->id;
        # card branch
        if ($card['card']['brand'] == 'visa') {
            $cardDetail['brand'] = $_ENV['FUSION_CRM_BUCKET']."FusionKitchen/cards/visa.svg";
        } elseif ($card['card']['brand'] == 'mastercard') {
            $cardDetail['brand'] = $_ENV['FUSION_CRM_BUCKET']."FusionKitchen/cards/master.svg";
        } elseif ($card['card']['brand'] == 'amex') {
            $cardDetail['brand'] = $_ENV['FUSION_CRM_BUCKET']."FusionKitchen/cards/American_Express.svg";
        } elseif ($card['card']['brand'] == 'discover') {
            $cardDetail['brand'] = $_ENV['FUSION_CRM_BUCKET']."FusionKitchen/cards/Discover.svg";
        } else {
            $cardDetail['brand'] = $_ENV['FUSION_CRM_BUCKET']."FusionKitchen/cards/else_payment.svg";
        }

        # expire month
        if(strlen($card['card']['exp_month']) == "1") { 
            $cardDetail['expMonth'] = "0".$card['card']['exp_month'];
        } else {
            $cardDetail['expMonth'] = $card['card']['exp_month'];
        }
        $cardDetail['expYear'] =  $card['card']['exp_year']; # expire year
        $cardDetail['card'] =  'xxxx xxxx xxxx '. $card['card']['last4']; # last 4 digit of card

        $cardDetail['fingerPrint'] = $card['card']['fingerprint'];

        if(empty($getSaveCard->finger_print) || $getSaveCard->finger_print == "") {
            if($saveCardCount == "0") {
                $cardDetail['primary'] = "0";
            } else {
                $cardDetail['primary'] = "1";
            }
        } else {
            if($card['card']['fingerprint'] == $getSaveCard->finger_print) {
                $cardDetail['primary'] = "1";
            } else{
                $cardDetail['primary'] = "0";
            }
        }
        return $cardDetail;
    }

    public function updatePrimarySaveCard(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'customer_id' => $inputData['customer_id'] ?? '',
            'finger_print' => $inputData['finger_print'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $payment = new Payment();
        $customerId = $params['customer_id'];
        $fingerPrint = $params['finger_print'];

        $getCustomerStripeConnection = $payment->getCustomerStripeConnection($customerId);
        $getSaveCard = $payment->getSaveCard($customerId);

        if(!empty($getSaveCard))
        {
            if($getCustomerStripeConnection['paymentMode'] == 0)
            {
                # test customer payment mode
                $customerPaymentId = $getSaveCard->test_pay_customer_id;
            }
            else
            {
                # live customer payment mode
                $customerPaymentId = $getSaveCard->pay_customer_id;
            }
            $customerPaymentValue = $payment->decrypt($customerPaymentId);

            # Call the Stripe API to get a list of payment methods
            $customerCard = $payment->stripePaymentMethod($getCustomerStripeConnection['skey'], $customerPaymentValue);

            if(!isset($customerCard['data']))
            {
                $responseData = [
                    "status" => false, 
                    "message" => $customerCard,
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }

            $saveCardCount = count($customerCard['data']);

            if($saveCardCount != "0") 
            {
                $updatePrimary = $payment->updatePrimarySaveCard($customerId, $fingerPrint);
            }

            $responseData = [
                "status" => true, 
                "message" => "primary card updated",
                "data" => $updatePrimary,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        else
        {
            $responseData = [
                "status" => false, 
                "message" => "no savecard",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function deleteSaveCard(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'customer_id' => $inputData['customer_id'] ?? '',
            'payment_method_id' => $inputData['payment_method_id'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $payment = new Payment();
        $customerId = $params['customer_id'];
        $paymentMethodId = $params['payment_method_id'];

        $getCustomerStripeConnection = $payment->getCustomerStripeConnection($customerId);
        $getSaveCard = $payment->getSaveCard($customerId);

        if(!empty($getSaveCard))
        {
            # Retrieve with detach a payment method by ID
            $retrieveCard = $payment->detachPaymentMethod($getCustomerStripeConnection['skey'], $paymentMethodId);

            if(!isset($retrieveCard['id']))
            {
                $responseData = [
                    "status" => false, 
                    "message" => $retrieveCard,
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }

            $responseData = [
                "status" => true, 
                "message" => "successfully deleted",
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        else
        {
            $responseData = [
                "status" => false, 
                "message" => "no savecard",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
    }

    public function addSaveCard(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'customer_id' => $inputData['customer_id'] ?? '',
            'payment_method_id' => $inputData['payment_method_id'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        
        $payment = new Payment();
        $customerId = $inputData['customer_id'];
        $paymentMethodId = $inputData['payment_method_id'];

        $getCustomerStripeConnection = $payment->getCustomerStripeConnection($customerId);
        $getSaveCard = $payment->getSaveCard($customerId);

        # Retrieve a payment method by ID
        $retrieveCard = $payment->retrievePaymentMethod($getCustomerStripeConnection['skey'], $paymentMethodId);

        if(!isset($retrieveCard['id']))
        {
            $responseData = [
                "status" => false, 
                "message" => $retrieveCard,
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $saveCardFingerPrint = $retrieveCard['card']['fingerprint'];

        if(!empty($getSaveCard))
        {
            if($getCustomerStripeConnection['paymentMode'] == 0)
            {
                # test customer payment mode
                $customerPaymentId = $getSaveCard->test_pay_customer_id;
            }
            else
            {
                # live customer payment mode
                $customerPaymentId = $getSaveCard->pay_customer_id;
            }
            $customerPaymentValue = $payment->decrypt($customerPaymentId);

            # Call the Stripe API to get a list of payment methods
            $customerCard = $payment->stripePaymentMethod($getCustomerStripeConnection['skey'], $customerPaymentValue);

            if(!isset($customerCard['data']))
            {
                $responseData = [
                    "status" => false, 
                    "message" => $customerCard,
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }

            $saveAllCardDetail = [];
            foreach($customerCard as $cardDetail){
                $saveAllCardDetail[] = $cardDetail['card']['fingerprint']; # array of unique card fingerprint id
                if($cardDetail['card']['fingerprint'] === $saveCardFingerPrint) {
                    $responseData = [
                        "status" => false, 
                        "message" => "Duplicate card",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }

            # check duplicate card entry
            if(!in_array($saveCardFingerPrint, $saveAllCardDetail))
            { 
                $attachedCardDetail = $payment->attachStripeCustomer($retrieveCard, $customerPaymentValue);

                if(!isset($attachedCardDetail['id']))
                {
                    $responseData = [
                        "status" => false, 
                        "message" => $attachedCardDetail,
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }
            $responseData = [
                "status" => true, 
                "message" => "successfully added",
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        else
        {
            $getCustomer = $payment->getCustomer($customerId);
            $description = $getCustomer->fname ." ". $getCustomer->lname ."  ".$getCustomer->email;
            $email = $getCustomer->email;

            $createCustomer = $payment->createStripeCustomer($getCustomerStripeConnection['skey'], $description, $email);

            if(!isset($createCustomer->id))
            {
                $responseData = [
                    "status" => false, 
                    "message" => $createCustomer,
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
            else
            {
                $attachedCardDetail = $payment->attachStripeCustomer($retrieveCard, $createCustomer->id);

                if(!isset($attachedCardDetail['id']))
                {
                    $responseData = [
                        "status" => false, 
                        "message" => $attachedCardDetail,
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }

            $fingerPrint = $attachedCardDetail['card']['fingerprint'];
            $paymentCustomerId = $payment->encrypt($createCustomer->id);

            if($getCustomerStripeConnection['paymentMode'] == 0)
            {
                # test customer payment mode
                $createSavecard = $payment->createSavecard($customerId, $paymentCustomerId, $fingerPrint, 0);
            }
            else
            {
                # live customer payment mode
                $createSavecard = $payment->createSavecard($customerId, $paymentCustomerId, $fingerPrint, 1);
            }
        
            if(isset($createSavecard->insert_id)) 
            {
                $responseData = [
                    "status" => true, 
                    "message" => "successfully added",
                    "response_code" => 200,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
            else
            {
                $responseData = [
                    "status" => false, 
                    "message" => "can't added",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }
    }

    public function tableBookingRefund(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'amount' => $inputData['amount'] ?? '',
            'transfer_key' => $inputData['transfer_key'] ?? '',
            'skey' => $inputData['skey'] ?? '',
            'pay_iid' => $inputData['pay_iid'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        
        $payment = new Payment();
        $amount = $inputData['amount'];
        $transferKey = $inputData['transfer_key'];
        $skey = $inputData['skey'];
        $transferId = $inputData['pay_iid'];

        $refundAmount = $amount * 100;

        # Retrieve a transfer ID
        $transferRetrieve = $payment->transferRetrieve($skey, $transferKey);

        $stripeAmount = $transferRetrieve->amount;

        $totalAmount = $refundAmount > $stripeAmount ? $stripeAmount : $refundAmount;

        # Create transfer reversal
        $createReversal = $payment->createReversal($skey, $transferKey, $totalAmount);

        $responseData = [
            "status" => true, 
            "message" => "Amount refunded",
            "data" => $createReversal,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getStripePaymentDetail(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'client_path' => $inputData['client_path'] ?? '',
        ];
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value) && $value!="0") {
                $errors[] = ucfirst($key) . ' is required';
            }
        }
        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $clientPath = $params['client_path'];
        if(isset($inputData['customer_id']) && $inputData['customer_id']!='0') {
            $customerId = $inputData['customer_id'];
        } else {
            $customerId = "";
        }
        
        $menu = new Menu();
        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        # Type of payment enable disable options
        $cashCardStatus = $menu->cashCardStatus();
        $getMode = $menu->getMode($getClientId->id);
        $getPaymentDetail = $menu->getPaymentDetail($getClientId->id);
        $securityKey = $getPaymentDetail->securitykey;
        $getContacts = $menu->getContacts($getClientId->id);
        $getStuartStatus = $menu->getStuartStatus($getClientId->id);

        if($getStuartStatus && $getStuartStatus->status == "1") {
            $data['stuart'] = true;
        } else {
            $data['stuart'] = false;
        }
        
        $data['clientDetail']['name'] = $getContacts->name;
        $data['clientDetail']['address1'] = $getContacts->address1;
        $data['clientDetail']['address2'] = $getContacts->address2;
        $data['clientDetail']['city'] = $getContacts->city;
        $data['clientDetail']['country'] = $getContacts->country;
        $data['clientDetail']['pincode'] = $getContacts->pincode;
        $data['clientDetail']['email'] = $getContacts->email;
        
        if(isset($inputData['platform'])){
            $data['paymentStatus'] = [
                'cash' => strpos($getMode->payment_del, '0') !== false,
                'card' => strpos($getMode->payment_del, '1') !== false,
                'applePay' => strpos($getMode->payment_del, '7') !== false,
                'googlePay' => strpos($getMode->payment_del, '8') !== false,
                'coll_cash' => strpos($getMode->payment, '0') !== false,
                'coll_card' => strpos($getMode->payment, '1') !== false,
                'coll_applePay' => strpos($getMode->payment, '7') !== false,
                'coll_googlePay' => strpos($getMode->payment, '8') !== false,
                'klarna' => $getMode->klarna !== 0,
                'paypalStatus' => $getMode->paypal_status !== 0,
                'guest_cash' => $getMode->guest_cash_enable !==0
            ];
        }
        else{
            $data['paymentStatus'] = [
                'cash' => $cashCardStatus->cash_enable == 0 && strpos($getMode->payment_del, '0') !== false,
                'card' => $cashCardStatus->card_enable == 0 && 1,
                'applePay' => true,
                'googlePay' => true,
                'coll_cash' => $cashCardStatus->cash_enable == 0 && 1,
                'coll_card' => $cashCardStatus->card_enable == 0 && 1,
                'coll_applePay' => true,
                'coll_googlePay' => true,
                'klarna' => $getMode->klarna !== 0,
                'paypalStatus' => $getMode->paypal_status !== 0,
                'guest_cash' => $cashCardStatus->cash_enable == 0 && $getMode->guest_cash_enable == 1
            ];
        }

        if($customerId != "") 
        {
            if($getPaymentDetail->provider == "Stripe")
            {
                # stripe payment only
                $payment = new Payment();
                # get customer detail
                $getCustomer = $payment->getCustomer($customerId);
                $description = $getCustomer->fname ." ". $getCustomer->lname ." ".$getCustomer->email;
                $email = $getCustomer->email;
                $getSaveCard = $payment->getSaveCard($customerId);

                # create customer from stripe
                $createCustomer = $payment->createStripeCustomer($securityKey, $description, $email);
                $paymentCustomerId = $payment->encrypt($createCustomer->id);
    
                if(empty($getSaveCard)) 
                {
                    # new save card
                    $fingerPrint = "";
                    if($getPaymentDetail->live == 0) {
                        # test customer payment mode
                        $createSavecard = $payment->createSavecard($customerId, $paymentCustomerId, $fingerPrint, 0);
                    } else {
                        # test customer payment mode
                        $createSavecard = $payment->createSavecard($customerId, $paymentCustomerId, $fingerPrint, 1);
                    }
                    $data['stripe']['defaultPaymentMethod'] = 0;
                    $data['stripe']['apiId'] = "";
                    $data['stripe']['url'] = "";
                    $data['card']['cardDetail'] = "";
                    $data['wallet'] = 0;
                } 
                else 
                {
                    # already have a save card
                    if($getPaymentDetail->live == 0) {
                        if(empty($getSaveCard->test_pay_customer_id)) {    
                            # test customer payment mode
                            $updateSavecardCustomerId = $payment->updateSavecardCustomerId($customerId, $paymentCustomerId, 0); 
                        }
                    } else {
                        if(empty($getSaveCard->pay_customer_id)) {
                            # test customer payment mode
                            $updateSavecardCustomerId = $payment->updateSavecardCustomerId($customerId, $paymentCustomerId, 1); 
                        }
                    }
    
                    # again get updated save card details
                    $getSaveCard = $payment->getSaveCard($customerId);
                    $customerPaymentId = $getPaymentDetail->live == 0 ? $getSaveCard->test_pay_customer_id : $getSaveCard->pay_customer_id;
                    $customerPaymentValue = $payment->decrypt($customerPaymentId);

                    # Retrieve a payment method by ID and another stripe payment details
                    $retrieveStripeCustomer = $payment->retrieveStripeCustomer($securityKey, $customerPaymentValue);
                    $data['stripe']['defaultPaymentMethod'] = $retrieveStripeCustomer['invoice_settings']['default_payment_method'] == NULL ? 0 : $retrieveStripeCustomer['invoice_settings']['default_payment_method'];
                    $data['stripe']['apiId'] = $getPaymentDetail->api_id;
                    $clientData = $menu->clientData($getClientId->id);
                    $data['stripe']['url'] = $clientData->url;

                    # Call the Stripe API to get a list of payment methods
                    $customerCard = $payment->stripePaymentMethod($securityKey, $customerPaymentValue);
                    $saveCardCount = count($customerCard['data']);
                    
                    # update primary save card list
                    if($saveCardCount != 0) {
                        if($getSaveCard->finger_print == "") {
                            $payment->updatePrimarySaveCard($customerId, $customerCard['data'][0]['card']->fingerprint);
                            $getSaveCard = $payment->getSaveCard($customerId);
                        }
                    }

                    foreach($customerCard['data'] as $card){
                        $getCardDetail = $this->getCardDetail($card, $getSaveCard, $saveCardCount);
                        $data['card']['cardDetail'][] = $getCardDetail;
                    }
                }
            }
            # get wallet balance
            $data['wallet'] = $payment->getWallet($customerId);
        } else {
            $data['stripe']['defaultPaymentMethod'] = 0;
            $data['stripe']['apiId'] = "";
            $data['stripe']['url'] = "";
            $data['card']['cardDetail'] = "";
            $data['wallet'] = 0;
        }

        $responseData = [
            "status" => true,
            "message" => $data,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;        
    }

    public function placeOrderWithPayment(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'clientPath' => $inputData['clientPath'] ?? '',
            'itemsDetail' => $inputData['itemsDetail'] ?? '',
            'orderDetail' => $inputData['orderDetail'] ?? '',
            'chargesDetail' => $inputData['chargesDetail'] ?? '',
            'userDetail' => $inputData['userDetail'] ?? '',
        ];
        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value) && $value!="0") {
                $errors[] = ucfirst($key) . ' is required';
            }
        }
        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $menu = new Menu();
        $clientPath = $params['clientPath'];
        $itemsDetail = $params['itemsDetail'];
        $orderDetail = $params['orderDetail'];
        $chargesDetail = $params['chargesDetail'];
        $userDetail = $params['userDetail'];
        # check opening time
        // $checkOrderTime = $menu->checkOrderTime($params);
        // if($checkOrderTime['status'] == false) {
        //     $responseData = [
        //         "status" => false,
        //         "message" => "Closed",
        //         "response_code" => 500,
        //     ];
        //     $response->getBody()->write(json_encode($responseData));
        //     return $response;
        // }

        $paymentDetail = ['loginType', 'orderType', 'paymentType', 'instruction', 'preOrder', 'cutlery', 'gift', 'giftQty'];
        foreach($paymentDetail as $data) {
            if(!isset($orderDetail[$data])) {
                $responseData = [
                    "status" => false,
                    "message" => $data." is required",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }

        $orderChargesDetail = ['subTotal', 'deliveryCharge', 'driverTip', 'roundAmount', 'extraDonate', 'clientStuartCharge', 'discount', 'discountDescription', 'promo', 'promoDescription', 'voucher', 'voucherDescription', 'sessionalOffer', 'sessionalOfferTotal'];
        foreach($orderChargesDetail as $data) {
            if(!isset($chargesDetail[$data])) {
                $responseData = [
                    "status" => false,
                    "message" => $data." is required",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }

        $validPaymentTypes = [0, 1, 7, 8, 9, 10, 14];
        if (!in_array($orderDetail['paymentType'], $validPaymentTypes)) {
            $responseData = [
                "status" => false,
                "message" => "Invalid Payment Type",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if($orderDetail['paymentType'] != 9) {
            if(empty($inputData['paymentDetail'])) {
                $responseData = [
                    "status" => false,
                    "message" => "PaymentDetail is required",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
            $params['paymentDetail'] = $inputData['paymentDetail'];
        }

        if (empty($userDetail['customerId'])) {
            if ($orderDetail['orderType'] == "0") {
                if (empty($userDetail['firstName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "First name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['lastName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Last name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['phone'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Phone number missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['doorNo'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Door number missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['streetName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Street name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['town'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Town missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['email'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['postcode'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Postcode missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            } else if ($orderDetail['orderType'] == "1") {
                if (empty($userDetail['firstName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "First name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['lastName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Last name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['phone'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Phone missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['email'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }
        } else {
            $Myaccount = new Myaccount();
            if($orderDetail['orderType'] == "0") {
                if (empty($userDetail['postcode'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "postcode missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                $getProfile = $Myaccount->getProfile($userDetail['customerId']);
                if (empty($userDetail['addressId'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Address Id missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                $getAddress = $Myaccount->getAddressWithId($userDetail['addressId']);
                if(empty($getAddress)) {
                    $responseData = [
                        "status" => false,
                        "message" => "Correct the address id",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                } 
                $userDetail['firstName'] = $getAddress->fname;
                $userDetail['lastName'] = $getAddress->lname;
                $userDetail['phone'] = $getAddress->phone;
                $userDetail['doorNo'] = $getAddress->no;
                $userDetail['streetName'] = $getAddress->address1;
                $userDetail['town'] = $getAddress->address2;

                if($getProfile->email == false) {
                    $responseData = [
                        "status" => false,
                        "message" => "email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                } 
                $userDetail['email'] = $getProfile->email;
                
            } else {
                $getProfile = $Myaccount->getProfile($userDetail['customerId']);
                if (empty($userDetail['addressId'])) {
                    $userDetail['firstName'] = $getProfile->fname;
                    $userDetail['lastName'] = $getProfile->lname;
                    $userDetail['phone'] = $getProfile->phone;
                    $userDetail['email'] = $getProfile->email;
                } else {
                    $getAddress = $Myaccount->getAddressWithId($userDetail['addressId']);
                    if(empty($getAddress)) {
                        $responseData = [
                            "status" => false,
                            "message" => "Correct the address id",
                            "response_code" => 400,
                        ];
                        $response->getBody()->write(json_encode($responseData));
                        return $response;
                    } 
                    $userDetail['firstName'] = $getAddress->fname;
                    $userDetail['lastName'] = $getAddress->lname;
                    $userDetail['phone'] = $getAddress->phone;
                    $userDetail['email'] = $getProfile->email;
                }
                if(empty($userDetail['email'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                } 
            }
        }

        if (empty($userDetail['userAgent'])) {
            $responseData = [
                "status" => false,
                "message" => "userAgent missing",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if (empty($userDetail['ip'])) {
            $responseData = [
                "status" => false,
                "message" => "ip missing",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        
        $params['userDetail'] = $userDetail;
        $data = $menu->placeOrderResponse($clientPath, $params);

        
        $isWebSocketTrigger = "failed"; 
        if (isset($data['response']['clientId'])) {
            $clientId = $data['response']['clientId'];
            $client = $menu->query("SELECT c.id, c.takeaway_name, td.id AS domain_id, td.url AS domain 
                            FROM tbl_clients AS c 
                            INNER JOIN tbl_domain AS td 
                            ON c.id = td.client_id 
                            WHERE c.id = $clientId LIMIT 1");
            $br_id = $menu->query("SELECT id from tbl_bridge where domain = '$client->domain'");
            if (!empty($client)) {
                $order = $data['response']['order'];
                if ($order->preorder != 0) {
                    $preorderDate = $order->preorder;
                    $s = date("M d, Y", $preorderDate) . "_" . date("h:i A", $preorderDate);
                    $order->newpreorder = $s;
                } else {
                    $now_time = date('h:i A', strtotime('now'));
                    $b_target = 0;
                    if ($order->otype == 0) {
                        $b_target = !empty($order->collection->delivery_time) ? $order->collection->delivery_time : 0;
                    } elseif ($order->otype == 1) {
                        $b_target = !empty($order->collection->collection_time) ? $order->collection->collection_time : 0;
                    }
                    $data_time = $order->datetime;
                    $asap_timestamp = $data_time + ($b_target * 60);
                    $asap_date = date("M d, Y", $asap_timestamp);
                    $asap_time = date("h:i A", $asap_timestamp);
                    $s = $asap_date . "_" . $asap_time;
                    $order->newdatetime = $s;
                }  
                $order->br_id = $br_id->id;
                $order->domain = $client->domain;
                $order->client_name = $client->takeaway_name;
                $order->client_id = $clientId;
                $order->type = 'neworder';
                $jsonData = json_encode($order);
                $url = "https://fusionbooking.co.uk/order";
                $post = curl_init();
                curl_setopt($post, CURLOPT_URL, $url);
                curl_setopt($post, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($post, CURLOPT_SSL_VERIFYHOST, false);
                curl_setopt($post, CURLOPT_POST, true);
                curl_setopt($post, CURLOPT_POSTFIELDS, $jsonData);
                curl_setopt($post, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($post, CURLOPT_TIMEOUT, 10); // Set timeout to 10 seconds
                curl_setopt($post, CURLOPT_HTTPHEADER, [
                    'Content-Type: application/json',
                    'Content-Length: ' . strlen($jsonData)
                ]);
                $result = curl_exec($post);
                if ($result === false) {
                    $error_msg = curl_error($post);
                    // echo "Curl error: " . $error_msg;
                } else {
                    $isWebSocketTrigger = "Success!";
                }
                curl_close($post);
            }
        }

        if($data['status'] == true) {
            $responseData = [
                "status" => true,
                "type" => $data['type'],
                "message" => $data['response'],
                "websocket_status" => $isWebSocketTrigger,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;  
        } else {
            $responseData = [
                "status" => false,
                "message" => $data['response'],
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;  
        }        
    }

    public function getLoyaltyPointsWithOrderAmount(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'ordered_amount' => $inputData['ordered_amount'] ?? '',
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $menu = new Menu();
        $orderedAmount = $params['ordered_amount'];
        $getLoyaltyAmount = $menu->getLoyaltyAmount($orderedAmount);
        
        if($getLoyaltyAmount['status'] == 1) {
            $responseData = [
                "status" => true,
                "data" => $getLoyaltyAmount,
                "response_code" => 200,
            ];
        }else {
            $responseData = [
                "status" => false,
                "data" => $getLoyaltyAmount,
                "response_code" => 400,
            ];
        }

        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function inCompleteOrder(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
            'itemsDetail' => $inputData['itemsDetail'] ?? '',
            'clientPath'=>$inputData['clientPath'] ?? '',
            'totalPrice'=>$inputData['totalPrice'] ?? '',
            'orderTime'=>$inputData['orderTime'] ?? '',
            'orderDetail'=>$inputData['orderDetail'] ?? '',
            'paymentDetail'=>$inputData['paymentDetail'] ?? '',
            'userDetail'=>$inputData['userDetail'] ?? '',
            'chargesDetail'=>$inputData['chargesDetail'] ?? '',
            // 'incompletedOrder'=>$inputData['incompletedOrder'] ?? '',
            
        ];

        $errors = [];
        foreach ($params as $key => $value) {
            if (empty($value)) {
                $errors[] = ucfirst($key) . ' is required';
            }
        }

        if (!empty($errors)) {
            $responseData = [
                "status" => false, 
                "message" => implode('. ', $errors),
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
       
       
      

        $clientPath = $params['clientPath'];
        $menu = new Menu();
        # get client id using path
       


        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if(isset($getClientId->id))
        {
           
            
            $data=[
                'itemsDetail'=>json_encode($params['itemsDetail']),
                'orderTime'=> strtotime($params['orderTime']),
                'incompletedOrder'=>(isset($inputData['incompletedOrder'])?$inputData['incompletedOrder']:0),
                'customerId'=>(isset($inputData['customerId'])?$inputData['customerId']:''),
                'logintype'=>(isset($inputData['logintype'])?$inputData['logintype']:0),
                'totalPrice'=>$params['totalPrice'],
                'client_id'=>$getClientId->id,
                'orderDetail'=>json_encode($params['orderDetail']),
                'paymentDetail'=>json_encode($params['paymentDetail']),
                'userDetail'=>json_encode($params['userDetail']),
                'chargesDetail'=>json_encode($params['chargesDetail']),
            ];
            $payment = new Payment();
            $insertInCompletedOrder=$payment->insertInCompletedOrder($data);
            $responseData=array();
            if(isset($insertInCompletedOrder->inserted_id))
            {
                $responseData=[
                    "status" => true, 
                    "data" => $insertInCompletedOrder->inserted_id,
                    "response_code" => 200,
                ];
            }
            else
            {
                $responseData=[
                    "status" => false, 
                    "data" => "Insertion failed",
                    "response_code" => 500,
                ];
            }
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        else
        {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }
       

    }


    // place order api for InCompletePayment
    
    public function placeOrderIncompletePayment(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        if(empty($inputData['id']))
        {
            $responseData = [
                "status" => false, 
                "message" => "InComplete Id is required",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $menu = new Menu();

        $params = $menu->getInCompleteOrder($inputData['id']);
        if($params->place_status!=0)
        {
            if((int)$params->place_status==1)
            {
                $responseData = [
                    "status" => false, 
                    "message" => "Order Already Placed",
                    "response_code" => 200,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
            $responseData = [
                "status" => false, 
                "message" => "In Correct incomplete id",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        $params=json_decode(json_encode($params),true);
        $client_id = $params['client_id'];
        $params['itemsDetail']=json_decode($params['itemsDetail'],true);
        $params['orderDetail']=json_decode($params['orderDetail'],true);
        $params['chargesDetail']=json_decode($params['chargesDetail'],true);
        $params['userDetail']=json_decode($params['userDetail'],true);
        $params['paymentDetail']=json_decode($params['paymentDetail'],true);
        $itemsDetail = $params['itemsDetail'];
        $orderDetail = $params['orderDetail'];
        $chargesDetail = $params['chargesDetail'];
        $userDetail = $params['userDetail'];
        $paymentDetail=$params['paymentDetail'];
        # check opening time
        // $checkOrderTime = $menu->checkOrderTime($params);
        // if($checkOrderTime['status'] == false) {
        //     $responseData = [
        //         "status" => false,
        //         "message" => "Closed",
        //         "response_code" => 500,
        //     ];
        //     $response->getBody()->write(json_encode($responseData));
        //     return $response;
        // }
        
        $getClientMenuPath=$menu->getClientMenuBasedId($client_id);
        $clientPath=isset($getClientMenuPath->client_menu_path)?$getClientMenuPath->client_menu_path:'';
        $params['clientPath']=$clientPath;
        $paymentDetail = ['loginType', 'orderType', 'paymentType', 'instruction', 'preOrder', 'cutlery', 'gift', 'giftQty'];
        foreach($paymentDetail as $data) {
            if(!isset($orderDetail[$data])) {
                $responseData = [
                    "status" => false,
                    "message" => $data." is required",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }

        $orderChargesDetail = ['subTotal', 'deliveryCharge', 'driverTip', 'roundAmount', 'extraDonate', 'clientStuartCharge', 'discount', 'discountDescription', 'promo', 'promoDescription', 'voucher', 'voucherDescription', 'sessionalOffer', 'sessionalOfferTotal'];
        foreach($orderChargesDetail as $data) {
            if(!isset($chargesDetail[$data])) {
                $responseData = [
                    "status" => false,
                    "message" => $data." is required",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        }

        $validPaymentTypes = [0, 1, 7, 8, 9, 10, 14];
        if (!in_array($orderDetail['paymentType'], $validPaymentTypes)) {
            $responseData = [
                "status" => false,
                "message" => "Invalid Payment Type",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if($orderDetail['paymentType'] != 9) {
            
            if(empty($paymentDetail)) {
                $responseData = [
                    "status" => false,
                    "message" => "PaymentDetail is required",
                    "response_code" => 400,
                ];
                $response->getBody()->write(json_encode($responseData));
                return $response;
            }
        
        }

        if (empty($userDetail['customerId'])) {
            if ($orderDetail['orderType'] == "0") {
                if (empty($userDetail['firstName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "First name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['lastName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Last name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['phone'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Phone number missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['doorNo'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Door number missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['streetName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Street name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['town'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Town missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['email'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['postcode'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Postcode missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            } else if ($orderDetail['orderType'] == "1") {
                if (empty($userDetail['firstName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "First name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['lastName'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Last name missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['phone'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Phone missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                if (empty($userDetail['email'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
            }
        } else {
            $Myaccount = new Myaccount();
            if($orderDetail['orderType'] == "0") {
                if (empty($userDetail['postcode'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "postcode missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                $getProfile = $Myaccount->getProfile($userDetail['customerId']);
                if (empty($userDetail['addressId'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "Address Id missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                }
                $getAddress = $Myaccount->getAddressWithId($userDetail['addressId']);
                if(empty($getAddress)) {
                    $responseData = [
                        "status" => false,
                        "message" => "Correct the address id",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                } 
                $userDetail['firstName'] = $getAddress->fname;
                $userDetail['lastName'] = $getAddress->lname;
                $userDetail['phone'] = $getAddress->phone;
                $userDetail['doorNo'] = $getAddress->no;
                $userDetail['streetName'] = $getAddress->address1;
                $userDetail['town'] = $getAddress->address2;

                if($getProfile->email == false) {
                    $responseData = [
                        "status" => false,
                        "message" => "email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                } 
                $userDetail['email'] = $getProfile->email;
                
            } else {
                $getProfile = $Myaccount->getProfile($userDetail['customerId']);
                if (empty($userDetail['addressId'])) {
                    $userDetail['firstName'] = $getProfile->fname;
                    $userDetail['lastName'] = $getProfile->lname;
                    $userDetail['phone'] = $getProfile->phone;
                    $userDetail['email'] = $getProfile->email;
                } else {
                    $getAddress = $Myaccount->getAddressWithId($userDetail['addressId']);
                    if(empty($getAddress)) {
                        $responseData = [
                            "status" => false,
                            "message" => "Correct the address id",
                            "response_code" => 400,
                        ];
                        $response->getBody()->write(json_encode($responseData));
                        return $response;
                    } 
                    $userDetail['firstName'] = $getAddress->fname;
                    $userDetail['lastName'] = $getAddress->lname;
                    $userDetail['phone'] = $getAddress->phone;
                    $userDetail['email'] = $getProfile->email;
                }
                if(empty($userDetail['email'])) {
                    $responseData = [
                        "status" => false,
                        "message" => "email missing",
                        "response_code" => 400,
                    ];
                    $response->getBody()->write(json_encode($responseData));
                    return $response;
                } 
            }
        }

        if (empty($userDetail['userAgent'])) {
            $responseData = [
                "status" => false,
                "message" => "userAgent missing",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        if (empty($userDetail['ip'])) {
            $responseData = [
                "status" => false,
                "message" => "ip missing",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }
        
        $params['userDetail'] = $userDetail;
        
        $data = $menu->placeOrderResponseInComplete($clientPath, $params,$inputData['id']);

        
        $isWebSocketTrigger = "failed"; 
        if (isset($data['response']['clientId'])) {
            $clientId = $data['response']['clientId'];
            $client = $menu->query("SELECT c.id, c.takeaway_name, td.id AS domain_id, td.url AS domain 
                            FROM tbl_clients AS c 
                            INNER JOIN tbl_domain AS td 
                            ON c.id = td.client_id 
                            WHERE c.id = $clientId LIMIT 1");
            $br_id = $menu->query("SELECT id from tbl_bridge where domain = '$client->domain'");
            if (!empty($client)) {
                $order = $data['response']['order'];
                if ($order->preorder != 0) {
                    $preorderDate = $order->preorder;
                    $s = date("M d, Y", $preorderDate) . "_" . date("h:i A", $preorderDate);
                    $order->newpreorder = $s;
                } else {
                    $now_time = date('h:i A', strtotime('now'));
                    $b_target = 0;
                    if ($order->otype == 0) {
                        $b_target = !empty($order->collection->delivery_time) ? $order->collection->delivery_time : 0;
                    } elseif ($order->otype == 1) {
                        $b_target = !empty($order->collection->collection_time) ? $order->collection->collection_time : 0;
                    }
                    $data_time = $order->datetime;
                    $asap_timestamp = $data_time + ($b_target * 60);
                    $asap_date = date("M d, Y", $asap_timestamp);
                    $asap_time = date("h:i A", $asap_timestamp);
                    $s = $asap_date . "_" . $asap_time;
                    $order->newdatetime = $s;
                }  
                $order->br_id = $br_id->id;
                $order->domain = $client->domain;
                $order->client_name = $client->takeaway_name;
                $order->client_id = $clientId;
                $order->type = 'neworder';
                $jsonData = json_encode($order);
                $url = "https://fusionbooking.co.uk/order";
                $post = curl_init();
                curl_setopt($post, CURLOPT_URL, $url);
                curl_setopt($post, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($post, CURLOPT_SSL_VERIFYHOST, false);
                curl_setopt($post, CURLOPT_POST, true);
                curl_setopt($post, CURLOPT_POSTFIELDS, $jsonData);
                curl_setopt($post, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($post, CURLOPT_TIMEOUT, 10); // Set timeout to 10 seconds
                curl_setopt($post, CURLOPT_HTTPHEADER, [
                    'Content-Type: application/json',
                    'Content-Length: ' . strlen($jsonData)
                ]);
                $result = curl_exec($post);
                if ($result === false) {
                    $error_msg = curl_error($post);
                    // echo "Curl error: " . $error_msg;
                } else {
                    $isWebSocketTrigger = "Success!";
                }
                curl_close($post);
            }
        }

        if($data['status'] == true) {
            $responseData = [
                "status" => true,
                "type" => $data['type'],
                "message" => $data['response'],
                "websocket_status" => $isWebSocketTrigger,
                "response_code" => 200,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;  
        } else {
            $responseData = [
                "status" => false,
                "message" => $data['response'],
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;  
        }
    }


}

?>