<?php
namespace App\Controllers;
use App\Models\Menu;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;

class PostcodeController
{
    public function checkDeliveryPostcode(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'post_code' => $inputData['post_code'] ?? '',
			'client_path' => $inputData['client_path'] ?? '',
			'order_mode' => $inputData['order_mode'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!="0") {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        $menu = new Menu();
        $postCode = str_replace(' ','',$params['post_code']);
        $clientPath = $params['client_path'];
        $orderMode = $params['order_mode'];
        
        if (strlen($postCode) < 4) {
			$responseData = [
				"status" => false, 
				"message" => "The postcode must have at least 4 characters.",
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        # order mode = 1 is collection
        if($orderMode == "1") {
            $responseData = [
                "status" => false,
                "message" => "This is not a delivery order",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }

        # get client id using path
        $getClientId = $menu->getClientUsingB2cPath($clientPath);
        if($getClientId == false) {
            $responseData = [
                "status" => false,
                "message" => "Incorrect Path",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response; 
        }
        $clientId = $getClientId->id;

        # min order functions
        $getMin = $menu->getMin($clientId);

        # get client order config radius/postcode
        $getConfigOrder = $menu->getConfigOrder($clientId);
        if($getConfigOrder->deliveryType === 0) 
        {
            # postcode method
            $getPostcodeResponse = $this->checkPostcode($clientId, $postCode);
            if(!empty($getPostcodeResponse)) 
            {
                $getPostcodeResponse['status'] = true;
                $getPostCodeDetail = $this->getPostCodeDetail($postCode);
                $getPostCodeDetail = json_decode($getPostCodeDetail);
                $getPostcodeResponse['lessThenAmount'] = $getConfigOrder->lessThenAmount;
                $getPostcodeResponse['extraDeliveryCharge'] = $getConfigOrder->extraDeliveryCharge;
                $getPostcodeResponse['address1'] = $getPostCodeDetail->street;
                $getPostcodeResponse['address1'] = $getPostCodeDetail->street;
                $getPostcodeResponse['address2'] = $getPostCodeDetail->town;
                $getPostcodeResponse['country'] = "UK";
            }
            else 
            {
                $getPostcodeResponse['status'] = false;
                $getPostcodeResponse['minAmount'] = 0;
                $getPostcodeResponse['address1'] = "";
                $getPostcodeResponse['address2'] = "";
                $getPostcodeResponse['country'] = "";
            }
        } 
        else 
        {
            # radius method
            $getContacts =  $menu->getContacts($clientId);
            $restaurantPostcode = $getContacts->pincode;
            
            # get distance option and maximum radius
            $distanceOption = $getConfigOrder->distanceOption;
            $maximumRadius = $getConfigOrder->maximumRadius;
            $getPostcodeResponse = $this->checkRadius($clientId, $restaurantPostcode, $postCode, $distanceOption, $maximumRadius);
            
            if($getPostcodeResponse['status'] == true)
            {
                $getPostCodeDetail = $this->getPostCodeDetail($postCode);
                $getPostCodeDetail = json_decode($getPostCodeDetail);	
                $getPostcodeResponse['lessThenAmount'] = $getConfigOrder->lessThenAmount;
                $getPostcodeResponse['extraDeliveryCharge'] = $getConfigOrder->extraDeliveryCharge;
                $getPostcodeResponse['address1'] = $getPostCodeDetail->street;
                $getPostcodeResponse['address2'] = $getPostCodeDetail->town;
                $getPostcodeResponse['country'] = 'UK';
                $getPostcodeResponse['status'] = true;	
            }
            else
            {
                $getPostcodeResponse['status'] = false;
                $getPostcodeResponse['minAmount'] = 0;
                $getPostcodeResponse['address1'] = "";
                $getPostcodeResponse['address2'] = "";
                $getPostcodeResponse['country'] = "";
            }
        }

        if($orderMode == 0) {
            $minOrder = $getMin->delivery_min;
        } else if($orderMode == 1) {
            $minOrder = $getMin->collection_min;
        }

        if(floatval($getPostcodeResponse['minAmount']) >= floatval($minOrder)) {
            $getPostcodeResponse['minAmount'] = $getPostcodeResponse['minAmount'];
        } else {
            $getPostcodeResponse['minAmount'] = $minOrder;
        }

        $getStuartStatus = $menu->getStuartStatus($clientId);
        $getPostcodeResponse['stuartDeliveryCharge'] = "";
        if($getStuartStatus) {
            if($getStuartStatus->status == 1) {
                $getPostcodeResponse['stuartDeliveryCharge'] = $this->getStuartDeliveryCharge($clientId, $postCode,$orderMode);
            }
        }
        
        if($getPostcodeResponse['status'] == true) {
            $responseData = [
                "status" => true, 
                "message" => $getPostcodeResponse,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false, 
                "message" => $getPostcodeResponse,
                "response_code" => 400,
            ];
        }
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    public function getStuartDeliveryCharge($clientId, $postCode)
    {
        $menu = new Menu();
        # get client order config radius/postcode with stuart
        $getMin = $menu->getMin($clientId);
        $getConfigOrder = $menu->getStuartConfigOrder($clientId);
        if($getConfigOrder->deliveryType === 0) 
        {
            # postcode method
            $getPostcodeResponse = $this->checkStuartPostcode($clientId, $postCode);
            if(!empty($getPostcodeResponse)) 
            {
                $getPostcodeResponse['status'] = true;
                $getPostCodeDetail =  $this->getPostCodeDetail($postCode);
                $getPostCodeDetail = json_decode($getPostCodeDetail);
                $getPostcodeResponse['address1'] = $getPostCodeDetail->street;
                $getPostcodeResponse['address2'] = $getPostCodeDetail->town;
                $getPostcodeResponse['country'] = "UK";
            }
            else 
            {
                $getPostcodeResponse['status'] = false;
                $getPostcodeResponse['minAmount'] = 0;
                $getPostcodeResponse['address1'] = "";
                $getPostcodeResponse['address2'] = "";
                $getPostcodeResponse['country'] = "";
            }
        } 
        else 
        {
            # radius method
            $getContacts =  $menu->getContacts($clientId);
            $restaurantPostcode = $getContacts->pincode;
            
            # get distance option and maximum radius
            $distanceOption = $getConfigOrder->distanceOption;
            $maximumRadius = $getConfigOrder->maximumRadius;
            $getPostcodeResponse = $this->checkStuartRadius($clientId, $restaurantPostcode, $postCode, $distanceOption, $maximumRadius);
            
            if($getPostcodeResponse['status'] == true)
            {
                $getPostCodeDetail = $this->getPostCodeDetail($postCode);
                $getPostCodeDetail = json_decode($getPostCodeDetail);	
                $getPostcodeResponse['address1'] = $getPostCodeDetail->street;
                $getPostcodeResponse['address2'] = $getPostCodeDetail->town;
                $getPostcodeResponse['country'] = 'UK';
                $getPostcodeResponse['status'] = true;	
            }
            else
            {
                $getPostcodeResponse['status'] = false;
                $getPostcodeResponse['minAmount'] = 0;
                $getPostcodeResponse['address1'] = "";
                $getPostcodeResponse['address2'] = "";
                $getPostcodeResponse['country'] = "";
            }
        } 

        $minOrder = $getMin->delivery_min;
        if(floatval($getPostcodeResponse['minAmount']) >= floatval($minOrder)) {
            $getPostcodeResponse['minAmount'] = $getPostcodeResponse['minAmount'];
        } else {
            $getPostcodeResponse['minAmount'] = $minOrder;
        }

        return $getPostcodeResponse;
    }

    public function checkRadius($clientId, $restaurantPostcode, $postCode, $distanceOption, $maximumRadius)
    {
        $menu = new Menu();
        $result = array();
        # Distance matrix api
        $getDistanceMatrix =  $this->getDistanceMatrix($restaurantPostcode, $postCode);
        $getDistanceMatrix = json_decode($getDistanceMatrix, true);

        # converts the units
        $distance = array( 
            "meters" => $getDistanceMatrix["distance_value"],
            "kilometers" => $getDistanceMatrix["distance_value"] / 1000,
            "yards" => $getDistanceMatrix["distance_value"] * 1.0936133,
            "miles" => $getDistanceMatrix["distance_value"] * 0.000621371
        ); 

        if($distanceOption == '1') {
            $distance = intval($distance['kilometers']);
        } else {
            $distance = intval($distance['miles']);
        }

        # get radius meter
        $getRadius =  $menu->getRadius($clientId);
        $getRadius = json_decode(json_encode($getRadius), true);

        if($maximumRadius < $distance) {
            $result['status'] = false;
            $deliveryCharge = 0;	
            $minAmount = 0;
            $freeAbove = 0;
        } else {
            $result['status'] = true;
            $deliveryCharge = $getRadius[$distance]['deliveryCharge'];	
            $minAmount = $getRadius[$distance]['minAmount'];	
            $freeAbove = $getRadius[$distance]['freeAbove'];	
        }

        $result['distance'] = $distance;
        $result['minAmount'] = $minAmount;
        $result['deliveryCharge'] = $deliveryCharge;
        $result['freeAbove'] = $freeAbove;

        return $result;
    }

    public function checkStuartRadius($clientId, $restaurantPostcode, $postCode, $distanceOption, $maximumRadius)
    {
        $menu = new Menu();
        $result = array();
        # Distance matrix api
        $getDistanceMatrix =  $this->getDistanceMatrix($restaurantPostcode, $postCode);
        $getDistanceMatrix = json_decode($getDistanceMatrix, true);

        # converts the units
        $distance = array( 
            "meters" => $getDistanceMatrix["distance_value"],
            "kilometers" => $getDistanceMatrix["distance_value"] / 1000,
            "yards" => $getDistanceMatrix["distance_value"] * 1.0936133,
            "miles" => $getDistanceMatrix["distance_value"] * 0.000621371
        ); 

        if($distanceOption == '1') {
            $distance = intval($distance['kilometers']);
        } else {
            $distance = intval($distance['miles']);
        }

        # get radius meter
        $getStuartRadius =  $menu->getStuartRadius($clientId);
        $getStuartRadius = json_decode(json_encode($getStuartRadius), true);

        if($maximumRadius < $distance) {
            $result['status'] = false;
            $deliveryCharge = 0;	
            $minAmount = 0;
            $freeAbove = 0;
        } else {
            if($getStuartRadius == false) {
                $result['status'] = false;
                $deliveryCharge = 0;
                $minAmount = 0;
                $freeAbove = 0;
            } else {
                $result['status'] = true;
                $deliveryCharge = $getStuartRadius[$distance]['deliveryCharge'];	
                $minAmount = $getStuartRadius[$distance]['minAmount'];	
                $freeAbove = $getStuartRadius[$distance]['freeAbove'];	
            }
        }

        $result['distance'] = $distance;
        $result['minAmount'] = $minAmount;
        $result['deliveryCharge'] = $deliveryCharge;
        $result['freeAbove'] = $freeAbove;

        return $result;
    }

    public function checkPostcode($clientId, $postCode) 
    {
        $menu = new Menu();
        $getExcludePostcode = $menu->getExcludePostcode($clientId, $postCode);
        if($getExcludePostcode == false) {
            while(strlen($postCode) > 1){
                if($getPostcodeList = $menu->getPostcodeList($clientId, $postCode)){
                    $getPostcodeList = json_decode(json_encode($getPostcodeList), true);
                    return $getPostcodeList;
                }
                $postCode = substr($postCode, 0, strlen($postCode)-1);		
            }
        } else {
            return false;
        }
    }

    public function checkStuartPostcode($clientId, $postCode)
    {
        $menu = new Menu();
        $getStuartExcludePostcode = $menu->getStuartExcludePostcode($clientId, $postCode);
        if($getStuartExcludePostcode == false) {
            while(strlen($postCode) > 1){
                if($getStuartPostcodeList = $menu->getStuartPostcodeList($clientId, $postCode)){
                    $getStuartPostcodeList = json_decode(json_encode($getStuartPostcodeList), true);
                    return $getStuartPostcodeList;
                }
                $postCode = substr($postCode, 0, strlen($postCode)-1);		
            }
        } else {
            return false;
        }
    }

    public function getGoogleAddress(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $params = [
			'post_code' => $inputData['post_code'] ?? '',
		];

		$errors = [];
		foreach ($params as $key => $value) {
			if (empty($value) && $value!="0") {
				$errors[] = ucfirst($key) . ' is required';
			}
		}

		if (!empty($errors)) {
			$responseData = [
				"status" => false, 
				"message" => implode('. ', $errors),
                "response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

        if (strlen($params['post_code']) < 4) {
            $responseData = [
                "status" => false, 
                "message" => "The postcode must have at least 4 characters.",
                "response_code" => 400,
            ];
            $response->getBody()->write(json_encode($responseData));
            return $response;
        }

        $postCode = str_replace(' ','',$params['post_code']);

        $getPostCodeDetail = $this->getPostCodeDetail($postCode);

        $getPostCodeDetail = json_decode($getPostCodeDetail);	

        if($getPostCodeDetail->status == false) {
            $getPostcodeResponse['street'] = "";
            $getPostcodeResponse['city'] = "";
            $getPostcodeResponse['country'] = 'UK';
            $getPostcodeResponse['status'] = false;	
        } else {
            $getPostcodeResponse['street'] = $getPostCodeDetail->street;
            $getPostcodeResponse['city'] = $getPostCodeDetail->town;
            $getPostcodeResponse['country'] = 'UK';
            $getPostcodeResponse['status'] = true;	
        }

        $responseData = [
            "status" => $getPostcodeResponse['status'], 
            "message" => $getPostcodeResponse,
            "response_code" => 200,
        ];
        $response->getBody()->write(json_encode($responseData));
        return $response;

    }

	public function getPostCodeDetail($post_code)
	{
		$data = array('postcode' => $post_code);
		$url  = 'https://postcode.fusionkitchen.co.uk/addressbypostcode.php';
		$fields = '';
		foreach ($data as $key => $value) 
		{
            $fields .= $key . '=' . $value . '&';
        }
        rtrim($fields, '&');
        $post = curl_init();
        curl_setopt($post, CURLOPT_URL, $url);
        curl_setopt($post, CURLOPT_POST, count($data));
        curl_setopt($post, CURLOPT_POSTFIELDS, $fields);
        curl_setopt($post, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($post);
        $err    = curl_error($post);
        curl_close($post);
        return $result;
	}

    public function getDistanceMatrix($origin,$destination)
	{
	    $data = array('origins' => $origin,'destinations' => $destination); 
        $url  = 'https://postcode.fusionkitchen.co.uk/distancematrix.php';
        $fields = '';
        foreach ($data as $key => $value) 
        {
            $fields .= $key . '=' . $value . '&';
        }
        rtrim($fields, '&');
        $post = curl_init();
        curl_setopt($post, CURLOPT_URL, $url);
        curl_setopt($post, CURLOPT_POST, count($data));
        curl_setopt($post, CURLOPT_POSTFIELDS, $fields);
        curl_setopt($post, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($post);
        $err    = curl_error($post);
        curl_close($post);
        return $result;
    }
}

?>