<?php
namespace App\Controllers;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use App\Models\Seo;
require '../config/env.php';

class SeoController
{
	public function getSitemap(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $errorMessage = "";
        $successMessage = [];

        $currentDateTime = time(); // Get the current Unix timestamp
        $yesterdayDateTime = $currentDateTime - 86400; // 86400 seconds = 1 day
        $yesterdayFormatted = date('Y-m-d\TH:i:sP', $yesterdayDateTime);

        $seo = new Seo();
        $getClientDetails = $seo->getAllB2cActiveClients();
        $getPageList = $seo->getPageList();

        foreach ($getPageList as $data) {
            $sitemapArray = [
                "loc" => $_ENV["SOCIAL"]["FK"] ."/". $data['url'],
                "lastmod" => $yesterdayFormatted,
                "priority" => $data['url'] == "" ? "1.00" : "0.90",
            ];
            array_push($successMessage, $sitemapArray);
        }

        foreach($getClientDetails as $key => $data) {
            $path = str_replace(" ", "%20", $data['path']);
            $pages = ["menu", "about", "review"];
            foreach ($pages as $page) {
                $sitemapArray = $this->generateSitemapArray("$path/$page", $yesterdayFormatted, "0.80");
                array_push($successMessage, $sitemapArray);
            }
        }

        # final error / success message
        if($errorMessage == "") { 
            $responseData = [
                "status" => true, 
                "message" => $successMessage,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false, 
                "message" => $errorMessage,
                "response_code" => 400,
            ];
        }
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }

    function generateSitemapArray($path, $lastmod, $priority) {
        return [
            "loc" => $_ENV["SOCIAL"]["FK"] . "/" . $path,
            "lastmod" => $lastmod,
            "priority" => $priority,
        ];
    }

    public function getSeoData(Request $request, Response $response, $args)
    {
        $inputData = json_decode(file_get_contents('php://input'), true);
        $errorMessage = "";
        $successMessage = [];
        
        $seo = new Seo();
        $getSeoData = $seo->getSeoData();
        $getMeta = $seo->getMeta();
        $getPageTitle = $seo->getPageTitle();
        $getPageList = $seo->getPageList();
        
        $successMessage['analytic_code'] = $getSeoData->analytic_code;
        $successMessage['tracking_id'] = $getSeoData->tracking_id;
        $successMessage['meta'] = [];

        // Processing $getMeta
        foreach ($getMeta as $data) {
            $pageName = $this->findPageName($data['page_id'], $getPageList);
            if ($pageName !== null) {
                $successMessage['meta'][$pageName]['robots'] = $data['robots'];
                $successMessage['meta'][$pageName]['description'] = $data['description'];
                $successMessage['meta'][$pageName]['keywords'] = $data['keywords'];
            }
        }
        
        // Processing $getPageTitle
        foreach ($getPageTitle as $data) {
            $pageName = $this->findPageName($data['page_id'], $getPageList);
            if ($pageName !== null) {
                $successMessage['title'][$pageName] = $data['title'];
            }
        }
        
        # final error / success message
        if($errorMessage == "") { 
            $responseData = [
                "status" => true, 
                "message" => $successMessage,
                "response_code" => 200,
            ];
        } else {
            $responseData = [
                "status" => false, 
                "message" => $errorMessage,
                "response_code" => 400,
            ];
        }
        $response->getBody()->write(json_encode($responseData));
        return $response;
    }
    
    function findPageName($pageId, $getPageList) {
        $seo = new Seo();
        $getPageList = $seo->getPageList();
        foreach ($getPageList as $page) {
            if ($page['id'] === $pageId) {
                return $page['pages'];
            }
        }
        return null;
    }

}

?>