<?php
namespace App\Controllers;

use App\Models\Coupon;
use App\Models\DB;
use App\Models\Menu;
use App\Models\Client;
use App\Models\Subscriber;
use App\Models\User;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Stripe\Webhook;
use App\Models\Payment;

require '../config/env.php';

class WebHookController
{
	public function webhook(Request $request, Response $response, $args)
	{
		$inputData = json_decode(file_get_contents('php://input'), true);
		if ($inputData == null) {
			$responseData = [
				"status" => false,
				"message" => "Data not available",
				"response_code" => 400,
			];
			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		$payload = @file_get_contents('php://input');
		$sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
		$webhookSecret = $_ENV['WEB_HOOK_KEY'];
		$payment = new Payment();
		$mailController = new MailController();

		try {
			$event = Webhook::constructEvent($payload, $sig_header, $webhookSecret);
			
			$type = $event['type'];
			$object = $event['data']['object'];
			
			switch ($type) {
				case 'invoice.payment_succeeded':
					$stripeKey = '';

					if (!isset($object['subscription_details']['metadata']['client_id'])) {
						$responseData = [
							"status" => false,
							"message" => "Client ID not found",
							"response_code" => 400,
						];
						$response->getBody()->write(json_encode($responseData));
						return $response;
					}

					$menu = new Menu();
					$stripeConnection = $menu->getPaymentDetail($object['subscription_details']['metadata']['client_id']);
					$stripeKey = $stripeConnection->securitykey;

					$payment = new Payment();
					$stripeCustomer = $payment->retrieveStripeCustomer($stripeKey, $object['customer']);

					$user = new User();
					$user = $user->checkCustomerByEmail($stripeCustomer['email']);

					$client = (new DB())->query("call getClientList(" . $object['subscription_details']['metadata']['client_id'] . ")");
					$client = $menu->getClientUsingB2cPath($client->path);

					$client_conn = new Client($client->id);
					$contact = $client_conn->query("CALL getContacts()");
					$domain = $menu->getClientDatabase($client->id);

					$coupon = new Coupon($client->id);
					$coupon_code = $coupon->generateCoupon(6);
					$discount = 25;
					$type = '%';
					$from = $object['lines']['data'][0]['period']['start'];
					$to = $object['lines']['data'][0]['period']['end'];

					$subscriber = new Subscriber($client->id, $user->id, 1);

					$data['user_id'] = $user->id;
					$data['customer_id'] = $object['customer'];
					$data['subscription_id'] = $object['subscription'];
					$data['subscription_status'] = $object['status'];
					$data['is_live'] = $object['livemode'];

					# update subscription status
					$subscriber->updateSubscriberDetails($data);
					$subscriber->addCredit((float) number_format(($object['amount_paid'] / 100), 2));

					# add coupon information to table
					$coupon->addCoupon($user->id, 'Subscription', $coupon_code, $discount, $type, $from, $to, true);

					# send success invoice
					$mailData['store_mail'] = "info@" . $domain->domain;
					$mailData['store_domain'] = $domain->domain;
					$mailData['store'] = $client->takeaway_name;
					$mailData['social'] = $_ENV['SOCIAL'];
					$mailData['logo'] = $_ENV['FUSION_BUCKET'] . $contact->image;
					$mailData['pdf'] = $object['invoice_pdf'];
					$mailData['name'] = $user->fname . ' ' . $user->lname;
					$mailData['coupon_code'] = $coupon_code;
					$mailData['amount'] = $object['amount_paid'] / 100;
					$mailData['currency'] = strtoupper($object['currency']);
					$mailData['discount'] = $discount;
					$mailData['type'] = $type;
					$mailData['start'] = date("F d, Y", $from);
					$mailData['end'] = date("F d, Y", $to);
					$description = $object['lines']['data'][0]['description'];
					$startPosition = strpos($description, "×") + 2; // Add 2 to skip the "× " characters
					$endPosition = strpos($description, "(") - 1; // Subtract 1 to exclude the space before the opening parenthesis
					$plan = substr($description, $startPosition, $endPosition - $startPosition);
					$mailData['plan'] = $plan;
					$mailData['invoiceNumber'] = $object['number'];

					$mailController->sendSuccessInvoice($object['customer_email'], $mailData);

					$responseData = [
						"status" => true,
						"message" => "Invoice payment success",
						"response_code" => 200,
					];

					break;
				case 'invoice.created':
					$stripeKey = '';

					if (!isset($object['subscription_details']['metadata']['client_id'])) {
						$responseData = [
							"status" => false,
							"message" => "Client ID not found",
							"response_code" => 400,
						];
						$response->getBody()->write(json_encode($responseData));
						return $response;
					}

					$menu = new Menu();
					$stripeConnection = $menu->getPaymentDetail($object['subscription_details']['metadata']['client_id']);
					$stripeKey = $stripeConnection->securitykey;

					$payment = new Payment();
					$invoices = $payment->getAllInvoice($stripeKey, $object['subscription'], 2);

					if (count($invoices['data']) >= 2) {
						$stripeCustomer = $payment->retrieveStripeCustomer($stripeKey, $object['customer']);

						$user = new User();
						$user = $user->checkCustomerByEmail($stripeCustomer['email']);

						$client = (new DB())->query("call getClientList(" . $object['subscription_details']['metadata']['client_id'] . ")");
						$client = $menu->getClientUsingB2cPath($client->path);

						$client_conn = new Client($client->id);
						$contact = $client_conn->query("CALL getContacts()");
						$domain = $menu->getClientDatabase($client->id);

						$coupon = new Coupon($client->id);
						$coupon_code = $coupon->generateCoupon(6);
						$discount = 25;
						$type = '%';

						$subscriber = new Subscriber($client->id, $user->id, 1);

						# send success invoice
						$mailData['store_mail'] = "info@" . $domain->domain;
						$mailData['store_domain'] = $domain->domain;
						$mailData['store'] = $client->takeaway_name;
						$mailData['social'] = $_ENV['SOCIAL'];
						$mailData['logo'] = $_ENV['FUSION_BUCKET'] . $contact->image;
						$mailData['name'] = $user->fname . ' ' . $user->lname;
						$mailData['coupon_code'] = $coupon_code;
						$mailData['amount'] = $object['lines']['data'][0]['amount'] / 100;
						$mailData['currency'] = strtoupper($object['currency']);
						$mailData['discount'] = $discount;
						$mailData['date'] = $mailData['created'] = date("F d, Y", $object['next_payment_attempt']);
						$mailData['type'] = $type;

						$mailController->sendRemainderInvoice($object['customer_email'], $mailData);
					}

					$responseData = [
						"status" => true,
						"message" => "Remainder mail sended",
						"response_code" => 200,
					];

					break;
				default:
					$responseData = [
						"status" => true,
						"message" => "default response",
						"response_code" => 200,
					];
			}
		} catch (\Exception $e) {
			$mailData['data']['message'] = $e->getMessage();
			// $mailController->webhook($_ENV['SUPPORT'], $mailData);

			$responseData = [
				"status" => false,
				"message" => $e->getMessage(),
				"response_code" => 400,
			];

			$response->getBody()->write(json_encode($responseData));
			return $response;
		}

		$response->getBody()->write(json_encode($responseData));
		return $response;
	}
}
?>