<?php
namespace App\Helpers;

use Aws\Ses\SesClient;
use Aws\Exception\AwsException;
use Twig\Environment;
use Twig\Loader\FilesystemLoader;
use App\Models\ClientHome;
use SendGrid;
use SendGrid\Mail\Mail;

require '../config/env.php';

class SendEmail
{
    private $client;
    private $twig;

    public function __construct()
    {
        $loader = new FilesystemLoader(__DIR__ . '/../view/Templates');
        $this->twig = new Environment($loader, [
            'cache' => false,
        ]);
        $this->client = new SesClient([
            'version' => 'latest',
            'region' => $_ENV['SES_REGION'],  // Use the region of your SES instance
            'credentials' => [
                'key' => $_ENV['SES_ACCESS_KEY_ID'],
                'secret' => $_ENV['SES_SECRET_ACCESS_KEY'],
            ],
        ]);
    }

    function sendMail($to, $cc, $sub, $html, $reply = "", $replyName = "", $from = "", $fromName = "", $bcc = [])
    {
        if ($from === "") {
            $from = $_ENV['MAIL_FROM'];
        }
        if ($fromName === "") {
            $fromName = $_ENV['MAIL_FROM_NAME'];
        }
        if ($reply === "") {
            $reply = $from;
        }

        $toAddresses = [$to];
        $ccAddresses = count($cc) > 0 ? $cc : [];  // CC addresses
        $bccAddresses = count($bcc) > 0 ? $bcc : [];  // BCC addresses

        // Email body
        $plainTextBody = strip_tags($html);  // Fallback plain text body
        $htmlBody = $html;  // HTML email content
        $subject = $sub;

        try {
            $result = $this->client->sendEmail([
                'Destination' => [
                    'ToAddresses' => $toAddresses,
                    'CcAddresses' => $ccAddresses,
                    'BccAddresses' => $bccAddresses,
                ],
                'ReplyToAddresses' => [$reply],
                'Source' => "{$fromName} <{$from}>",
                'Message' => [
                    'Body' => [
                        'Html' => [
                            'Charset' => 'UTF-8',
                            'Data' => $htmlBody,
                        ],
                        'Text' => [
                            'Charset' => 'UTF-8',
                            'Data' => $plainTextBody,
                        ],
                    ],
                    'Subject' => [
                        'Charset' => 'UTF-8',
                        'Data' => $subject,
                    ],
                ],
            ]);

            return true;
        } catch (AwsException $e) {
            $sendgrid = $this->sendGridEmail($from, $to, $sub, $html, $fromName);
            if ($sendgrid['status']) {
                return $sendgrid;
            } else {
                return $e->getMessage();
            }
        }
    }

    function sendGridEmail($fromEmail, $to, $subject, $bodyContent, $takeaway)
    {
        $apiKey = 'U0cudVRHUDdjM2lTNkNWeHZ6ZGxVSFpKdy5hU0QwT0dWdDJIZ2RfNzlxYml6RW5IQTdpaXRUSXBiSVhMdHA3czJhXzRF';
        try {
            $sendgrid = new \SendGrid(base64_decode($apiKey));
        } catch (\Exception $e) {
            return [
                'status' => 'error',
                'message' => 'Failed to initialize SendGrid: ' . $e->getMessage()
            ];
        }

        $email = new \SendGrid\Mail\Mail();
        $email->setFrom('ordernow@fusionkitchen.co.uk', $takeaway);
        $email->addTo($to);
        if ($fromEmail) {
            $email->setReplyTo($fromEmail);
        }
        $email->setSubject($subject);
        $email->addContent("text/plain", $bodyContent);
        $email->addContent("text/html", $bodyContent);
        try {
            $response = $sendgrid->send($email);
            return [
                'status' => true,
                'response' => $response->statusCode(),
                'body' => $response->body(),
                'headers' => $response->headers()
            ];
        } catch (\Exception $e) {
            return [
                'status' => 'error',
                'message' => 'Failed to send email: ' . $e->getMessage()
            ];
        }
    }

    function sendreservation($data)
    {
        $imgurl = "https://fusionbucket.co.uk/img/";
        $data['logo'] = $imgurl . $data['logo'];
        $data['book_datetime'] = date("d/m/Y h:i A", strtotime($data['date'] . " " . $data['time']));
        $data['server_host'] = $_SERVER['HTTP_HOST'];
        $tableReservationHtml = $this->twig->render('tableReservation.twig', $data);
        $cc = array();
        return $this->sendMail($data['email'], $cc, "Thanks for Reservation on " . date("d/m/Y h:i A", strtotime($data['date'] . " " . $data['time'])) . " ", $tableReservationHtml);
    }



    function sentReservationToBoss($data)
    {
        $home = new ClientHome($data['domain']);
        $send = $home->getnotification($data['domain']);
        if (empty($send->reserv_mail)) {
            return false;
        }
        $imgurl = "https://fusionbucket.co.uk/img/";
        $data['logo'] = $imgurl . $data['logo'];
        $data['book_datetime'] = date("d/m/Y h:i A", strtotime($data['date'] . " " . $data['time']));
        $confirmReservationHtml = $this->twig->render('bookConfirmation.twig', $data);
        $cc = array();
        $connection_close = $home->connectionClose();
        return self::sendmail($send->reserv_mail, $cc, "You have received one table Booking on  " . date("d/m/Y h:i A", strtotime($data['date'] . " " . $data['time'])) . " ", $confirmReservationHtml);
    }
}