<?php

namespace App\Helpers;

use Aws\Ses\SesClient;
use Aws\Exception\AwsException;
use SendGrid;
use SendGrid\Mail\Mail;

require '../config/env.php';

class SendEposEmail
{
    private $client;

    public function __construct()
    {
        $this->client = new SesClient([
            'version' => 'latest',
            'region'  => $_ENV['SES_REGION'],
            'credentials' => [
                'key'    => $_ENV['SES_ACCESS_KEY_ID'],
                'secret' => $_ENV['SES_SECRET_ACCESS_KEY'],
            ],
        ]);
    }

    public function sendMail($to, $cc, $sub, $html, $reply = "", $replyName = "", $from = "", $fromName = "")
    {
        if ($from === "") {
            $from = $_ENV['MAIL_FROM'];
        }
        if ($fromName === "") {
            $fromName = $_ENV['MAIL_FROM_NAME'];
        }
        if ($reply === "") {
            $reply = $from;
        }
        if ($replyName === "") {
            $replyName = $fromName;
        }

        $toAddresses = [$to];  // Main recipient
        $ccAddresses = count($cc) > 0 ? $cc : [];  // CC recipients

        // Construct the email body
        $plainTextBody = strip_tags($html);  // Fallback plain text body
        $htmlBody = $html;  // HTML email body
        $subject = $sub;  // Email subject

        try {
            $result = $this->client->sendEmail([
                'Destination' => [
                    'ToAddresses' => $toAddresses,
                    'CcAddresses' => $ccAddresses,
                ],
                'ReplyToAddresses' => [$reply],
                'Source' => "{$fromName} <{$from}>",
                'Message' => [
                    'Body' => [
                        'Html' => [
                            'Charset' => 'UTF-8',
                            'Data' => $htmlBody,
                        ],
                        'Text' => [
                            'Charset' => 'UTF-8',
                            'Data' => $plainTextBody,
                        ],
                    ],
                    'Subject' => [
                        'Charset' => 'UTF-8',
                        'Data' => $subject,
                    ],
                ],
            ]);

            return true;
        } catch (AwsException $e) {
            
            $sendgrid = $this->sendGridEmail($from, $to, $sub, $html, $fromName);
            if($sendgrid['status']){
                return $sendgrid;
            }else{
               return $e->getMessage();
            }
        }
    }

    function sendGridEmail($fromEmail, $to, $subject, $bodyContent, $takeaway)
    {

        $apiKey = 'U0cudVRHUDdjM2lTNkNWeHZ6ZGxVSFpKdy5hU0QwT0dWdDJIZ2RfNzlxYml6RW5IQTdpaXRUSXBiSVhMdHA3czJhXzRF';
        try {
            $sendgrid = new \SendGrid(base64_decode($apiKey));
        } catch (\Exception $e) {
            return [
                'status' => 'error',
                'message' => 'Failed to initialize SendGrid: ' . $e->getMessage()
            ];
        }
        $email = new \SendGrid\Mail\Mail();
        $email->setFrom('ordernow@fusionkitchen.co.uk', 'Fusion Kitchen');
        $email->addTo($to);
        if ($fromEmail) {
            $email->setReplyTo($fromEmail);
        }
        $email->setSubject($subject);
        $email->addContent("text/plain", $bodyContent);
        $email->addContent("text/html", $bodyContent);
        try {
            $response = $sendgrid->send($email);
            return [
                'status' => true,
                'response' => $response->statusCode(),
                'body' => $response->body(),
                'headers' => $response->headers()
            ];
        } catch (\Exception $e) {
            return [
                'status' => 'error',
                'message' => 'Failed to send email: ' . $e->getMessage()
            ];
        }
    }
}
