<?php

namespace App\Models;

use Stripe\Stripe;
use Stripe\Exception\ApiErrorException;

class ClientPayment extends DB
{

    public function getStripeConnection($domain)
    {
        // $client = new Client($domain);
        $getClientDetails  = $this->getClientDetailsByDomain($domain);

        $client = New Client($getClientDetails->client_id);
        $getPaymentDetail = $client->query("CALL getPaymentDetail()");
        $paymentDetail['securityKey'] = $getPaymentDetail->securitykey;
        $paymentDetail['live'] = $getPaymentDetail->live;
        $paymentDetail['apiId'] = $getPaymentDetail->api_id;
        $paymentDetail['connectId'] = $getPaymentDetail->connectid;
        $paymentDetail['stripeCharge'] = $getPaymentDetail->stripecharge;
        $paymentDetail['stripeAdd'] = $getPaymentDetail->stripeadd;
        $client->connectionClose();
        return $paymentDetail;
    }

    public function createStripeCustomerWithCard($getStripeConnection, $description, $email, $paymentMethodId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try {
            $createCustomer = \Stripe\Customer::create([
                "description" => $description,
                "email" => $email,
                "payment_method" => $paymentMethodId
            ]);
            return $createCustomer;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function transferRetrieve($getStripeConnection, $transferKey)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try {
            # Retrieve a transfer ID
            $transfer = \Stripe\Transfer::retrieve($transferKey);
            return $transfer;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function createStripeCustomerWithAccount($getStripeConnection, $description, $email, $connectId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try {
            $createCustomer = \Stripe\Customer::create(
                array(
                    "description" => $description,
                    "email" => $email
                ),
                array(
                    "stripe_account" => $connectId
                )
            );
            return $createCustomer;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function chargesRetrieve($getStripeConnection, $chargeKey)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # charge retrieve
            $charge = \Stripe\Charge::retrieve($chargeKey);

            return [
                'status' => true,
                'data' => $charge
            ];
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return [
                'status' => false,
                'data' => $e->getMessage()
            ];
        }
    }

    public function chargesUpdate($getStripeConnection, $destinationPayment, $description, $createStripeCustomerId, $connectId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try {
            # charge update
            $chargesUpdate = \Stripe\Charge::update(
                $destinationPayment,
                [
                    'description' => $description,
                    'customer' => $createStripeCustomerId
                ],
                [
                    "stripe_account" => $connectId
                ]
            );

            // $balance = \Stripe\Balance::retrieve();
            $response['status'] = true;
            $response['data'] = $chargesUpdate;
            return $response;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function createPaymentIntentWithCard($getStripeConnection, $connectId, $domain, $takeawayName, $totalAmount, $clientTransferAmount, $description, $paymentMethodId, $customer)
    {
        $takeawayName = substr($takeawayName, 0, 22);
        if (!isset($customer->id)) {
            $response['status'] = false;
            $response['data'] = $customer;
            return $response;
        }

        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try {
            # create payment intent
            $paymentIntent = \Stripe\PaymentIntent::create([
                'payment_method' => $paymentMethodId,
                'metadata' => [
                    'domain' => $domain,
                    'portal' => 'Table booking'
                ],
                'amount' => $totalAmount,
                'statement_descriptor_suffix' => $takeawayName,
                'description' => $description,
                'currency' => 'gbp',
                'automatic_payment_methods' => [
                    'enabled' => true,
                    'allow_redirects' => 'never'
                ],
                'customer' => $customer->id,
                'confirm' => true,
                'capture_method' => 'automatic',
                "on_behalf_of" => $connectId,
                'transfer_data' => [
                    "amount" => $clientTransferAmount,
                    'destination' => $connectId,
                ],
            ], [
                "idempotency_key" => uniqid(),
            ]);


            $response['status'] = true;
            $response['data'] = $paymentIntent;
            return $response;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function retrievePaymentIntent($getStripeConnection, $intentId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try {
            $paymentIntent = \Stripe\PaymentIntent::retrieve($intentId);
            return $paymentIntent;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function confirmPaymentIntent($getStripeConnection, $intentId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try {
            $paymentIntent = \Stripe\PaymentIntent::retrieve($intentId);
            $confirmedPaymentIntent = $paymentIntent->confirm();
            return $confirmedPaymentIntent;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }
}
