<?php

namespace App\Models;

use App\Models\RestaurantPage;
use App\Models\Payment;
use App\Models\User;
use App\Models\Myaccount;
use App\Models\Klarna;
use App\Models\PayPal;
use App\Controllers\PostcodeController;
use App\Controllers\PagesController;
use App\Controllers\MenuController;
use App\Controllers\MyAccountController;
use Stripe\Exception\ApiErrorException;
use Stripe\Stripe;

require '../config/env.php';

class Menu extends DB
{
    private $db;
    function __construct()
    {
        $this->db = new DB();
    }

    public function getBusinessTime($currentTime, $getMode, $clientId, $orderMode, $pre, $tatStatus = "", $getCookingTime = "")
    {
        $getOrderMode = str_split($getMode);
        
        if ($orderMode == 0) {
            $type1 = "delivery_openat";
            $type2 = "delivery_closeat";
            $type = "delivery";
        } elseif ($orderMode == 1) {
            $type1 = "collection_openat";
            $type2 = "collection_closeat";
            $type = "collection";
        }

        // Get the current date based on current time
        $currentDate = strtotime(date('d-m-Y', $currentTime));

        // Retrieve today's shift times
        $getCurrentShiftTime = $this->getShiftTime($clientId, $currentDate);

        // Retrieve previous day's shift times if needed (before 8 AM)
        $getPreviousShiftTime = [];
        if (date('H', $currentTime) < 8) {
            $previousDate = strtotime('-1 day', $currentDate);
            $getPreviousShiftTime = $this->getShiftTime($clientId, $previousDate);

            // Check if previous day contains shift data after 11:55 PM
            foreach ($getPreviousShiftTime as $shift) {
                if ($shift->$type == 1) {
                    $time_1 = $shift->$type1;
                    $time_2 = $shift->$type2;

                    if ($time_1 != "" && $time_2 != "") {
                        $startTimestamp = strtotime($time_1);
                        $endTimestamp = strtotime($time_2);

                        // Check if the previous day shift ends after 11:55 PM
                        if ($startTimestamp < strtotime('23:55') && $endTimestamp > strtotime('23:55')) {
                            // Allow the start date to be 11:55 PM of the previous day
                            $previousDayStartTime = strtotime(date('d-m-Y', $previousDate) . " 23:55");

                            // If current time is within the previous day's shift range, adjust the start time
                            if ($currentTime >= $previousDayStartTime && $currentTime < $endTimestamp) {
                                $currentTime = $previousDayStartTime;
                            }
                        }
                    }
                }
            }
        }

        // Combine today's and previous day's shifts
        $getShiftTime = array_merge($getPreviousShiftTime, $getCurrentShiftTime);

        // Adjust current time for cooking time if applicable
        if ($getCookingTime && isset($getCookingTime[$type]) && !empty($getCookingTime[$type])) {
            $currentTime += ($getCookingTime[$type] * 60);
        }

        $timing = "";
        $firstValidShift = true;

        if (count($getShiftTime) > 0) {
            foreach ($getShiftTime as $shift) {
                if ($shift->$type == 1) { // If the shift is active
                    $time_1 = $shift->$type1; // Open time
                    $time_2 = $shift->$type2; // Close time

                    if ($time_1 != "" && $time_2 != "") {
                        $startTimestamp = strtotime($time_1);
                        $endTimestamp = strtotime($time_2);

                        // Add cooking time if applicable
                        if (isset($getCookingTime[$type])) {
                            $cookingTimeInSeconds = $getCookingTime[$type] * 60;
                            $startTimestamp += $cookingTimeInSeconds;
                        }

                        // Handle shifts that span into the next day (e.g., 13:00 to 07:55)
                        if ($startTimestamp > $endTimestamp) {
                            $endDate = date('d-m-Y', strtotime('+1 day', $currentDate));
                        } else {
                            $endDate = date('d-m-Y', $currentDate);
                        }

                        $startDate = ($startTimestamp < strtotime('08:00')) ? date('d-m-Y', strtotime('-1 day', $currentDate)) : date('d-m-Y', $currentDate);

                        // Calculate the actual start and end times
                        $startTime = strtotime($startDate . " " . date('H:i', $startTimestamp));
                        $endTime = strtotime($endDate . " " . date('H:i', $endTimestamp));

                        // Skip shifts that have fully passed
                        if ($endTime < $currentTime) {
                            continue;
                        }

                        // Adjust the start time if the shift has already started
                        if ($startTime < $currentTime) {
                            $startTime = $currentTime;
                        }

                        $duration = $endTime - $startTime;

                        // Handle the first valid shift separately
                        // if ($firstValidShift) {
                        //     $timing = ""; // Clear the previous timing data
                        //     $firstValidShift = false; // Turn off the first valid shift flag
                        // }
                    
                    if (!$firstValidShift) {
                        // Generate time slots in 5-minute intervals
                        for ($i = 0; $i <= $duration; $i += 300) { // 300 seconds = 5 minutes
                            $intervalTime = $startTime + $i;
                            if ($pre == 0) {
                                $intervalTime = date('d-m-Y H:i', $intervalTime);
                            } else {
                                $intervalTime = date('H:i', $intervalTime);
                            }
                            $timing .= !empty($timing) ? ',' . $intervalTime : $intervalTime;
                        }

                    }
                    else{
                        $timing = ""; // Clear the previous timing data
                        $firstValidShift = false;
                    }
                    }
                }
            }
        }

        

        return $timing ?: "";
    }



    public function getBusinessTimeLater($currentTime, $getMode, $clientId, $orderMode, $pre, $tatStatus = "", $getCookingTime = "")
    {
        $getOrderMode = str_split($getMode);
        if ($orderMode == 0) {
            $type1 = "delivery_openat";
            $type2 = "delivery_closeat";
            $type = "delivery";
        } elseif ($orderMode == 1) {
            $type1 = "collection_openat";
            $type2 = "collection_closeat";
            $type = "collection";
        }

        # get current date
        $hours = date('H', $currentTime);
        $hourMinutes = date('H:i', $currentTime);

        // return date('d-m-Y H:i A',$currentTime);

        if ($pre == 1 || $pre == 0) {
            // $currentDate = strtotime(date('d-m-Y', $currentTime));
            // if($hours < 8) 
            // {
            //     $currentDate = strtotime(date('d-m-Y',(strtotime ( '0 day' , $currentTime))));
            // } 
            // $getShiftTime = $this->getShiftTime($clientId, $currentDate);

            if (intval($hours) < 8) {
                $currentDate = strtotime(date('d-m-Y', $currentTime));
                $previousDate = strtotime(date('d-m-Y', (strtotime('-1 day', $currentTime))));
                $getPreShiftTime = $this->getShiftTime($clientId, $previousDate);
                $getCurrentShiftTime = $this->getShiftTime($clientId, $currentDate);
                $getShiftTime = array_merge($getPreShiftTime, $getCurrentShiftTime);
            } else {
                $currentDate = strtotime(date('d-m-Y', $currentTime));
                $getShiftTime = $this->getShiftTime($clientId, $currentDate);
            }
        } else {
            $currentDate = strtotime(date('d-m-Y', $currentTime));
            $previousDate = strtotime(date('d-m-Y', (strtotime('-1 day', $currentTime))));
            $getPreShiftTime = $this->getShiftTime($clientId, $previousDate);
            $getCurrentShiftTime = $this->getShiftTime($clientId, $currentDate);
            $getShiftTime = array_merge($getPreShiftTime, $getCurrentShiftTime);
        }

        # business shift functionality
        if (count($getShiftTime) != 0) {
            $timing = "";
            foreach ($getShiftTime as $shift) {
                $time_1 = $shift->$type1;
                $time_2 = $shift->$type2;

                if ($time_1 != "" && $time_2 != "") {
                    if ($pre == 1 || $pre == 0) {
                        # get start time based on current time
                        # this if statement for preorder starting time
                        if ($hourMinutes <= $time_2) {
                            // $startTimestamp = $hourMinutes;
                            if ($hourMinutes >= $time_1) {
                                $startTimestamp = $hourMinutes;
                            } else {
                                $startTimestamp = $time_1;
                            }
                        } else if ($hourMinutes <= $time_1) {
                            $startTimestamp = $time_1;
                        } else {
                            $startTimestamp = $hourMinutes;
                        }

                        if (isset($getCookingTime[$type])) {
                            $getCookingTime = $getCookingTime[$type];
                        }
                        if ($getCookingTime) {
                            $startTimestamp = date('H:i', strtotime($startTimestamp) + ($getCookingTime * 60));
                        } else {
                            $startTimestamp = date('H:i', strtotime($startTimestamp));
                        }

                        # open/close status with starting time 
                        $checkOpenStatus = $this->checkOpenStatus($clientId, $currentTime);
                        if ($checkOpenStatus != "") {
                            if (is_numeric($checkOpenStatus)) {
                                $currentDate = strtotime(date('d-m-Y', $currentTime));
                                $openDate = strtotime(date('d-m-Y', $checkOpenStatus));
                                if ($currentTime <= $checkOpenStatus && $currentDate == $openDate) {
                                    $startTimestamp = date('H:i', $checkOpenStatus);
                                } else {
                                    return "closed";
                                }
                            } else {
                                return "closed";
                            }
                        }

                        # get end time based on current time
                        $endTimestamp = $time_2;

                        # get start and end hours
                        $startHours = date('H', strtotime($time_1));
                        $endHours = date('H', strtotime($time_2));

                        # get start and end date
                        $startDate = $startHours < 8 ? date('d-m-Y', (strtotime('+1 day', $currentDate))) : date('d-m-Y', $currentDate);
                        $endDate = $endHours < 8 ? date('d-m-Y', (strtotime('+1 day', $currentDate))) : date('d-m-Y', $currentDate);
                    } else {
                        # closed status for open/close, holiday and bank holiday
                        $yesterdayTime = strtotime(date('d-m-Y', $previousDate) . " " . "08:00");
                        $todayTime = strtotime(date('d-m-Y', $currentTime) . " " . "08:00");
                        $yesterdayCheckStatus = $this->checkOpenStatus($clientId, $yesterdayTime);
                        $todayCheckStatus = $this->checkOpenStatus($clientId, $todayTime);

                        # get start time based on current time
                        $day = date('N', $currentDate);

                        if ($day > $shift->day || $day < $shift->day) {
                            # previous days start and end times
                            $startTimestamp = ("08:00" < $time_1) ? "00:00" : $time_1;
                            $endTimestamp = ("08:00" > $time_2) ? $time_2 : "";
                            // $startTimestamp = "00:00";
                            // $endTimestamp = $time_2;
                            if ($yesterdayCheckStatus != "") {
                                $startTimestamp = $time_1;
                            }
                        } else if ($day == $shift->day) {
                            # current days start and end times
                            $startTimestamp = $time_1;
                            $endTimestamp = ("08:00" < $time_2) ? $time_2 : "23:59";
                            # open/close time with start and end times 
                            if ($todayCheckStatus != "") {
                                $endTimestamp = $time_2;
                                if (is_numeric($todayCheckStatus)) {
                                    $currentDate = strtotime(date('d-m-Y', $currentTime));
                                    $openDate = strtotime(date('d-m-Y', $todayCheckStatus));
                                    if ($currentTime <= $todayCheckStatus && $currentDate == $openDate) {
                                        $startTimestamp = date('H:i', $todayCheckStatus);
                                        $endTimestamp = $time_2 < "23:59" && $time_2 > "08:00" ? $time_2 : "23:59";
                                    }
                                }
                            }
                        }
                        # get start and end date
                        $startDate = date('d-m-Y', $currentDate);
                        $endDate = date('d-m-Y', $currentDate);
                    }

                    if (in_array($orderMode, $getOrderMode)) {
                        if ($shift->$type == 1) {
                            if ($startTimestamp != "" && $endTimestamp != "") {
                                $startTime = strtotime($startDate . " " . $startTimestamp);

                                # calculation for between the start and end time
                                # adding cooking time to today orders while disable the tat time
                                $cookingTime = "";
                                if ($pre == 1 && $tatStatus == 1) {
                                    $cookingTime = " +" . $getCookingTime . " minutes";
                                }
                                $targetTime = strtotime($endDate . " " . $endTimestamp . $cookingTime);
                                $duration = $targetTime - $startTime;

                                # shift times between 5 min gap
                                for ($i = 0; $i <= $duration; $i += 300) { # 300 seconds = 5 minutes
                                    $intervalTime = $startTime + $i;
                                    if ($pre == 0) {
                                        $intervalTime = date('d-m-Y H:i', $intervalTime);
                                    } else {
                                        $intervalTime = date('H:i', $intervalTime);
                                    }
                                    # $intervalTime = date('Y-m-d H:i', $intervalTime);
                                    $timing .= !empty($timing) ? ',' . $intervalTime : $intervalTime;
                                }
                            }
                        } else {
                            // return $this->getOrderModeErrorMessage($orderMode);
                        }
                    } else {
                        // return $this->getOrderModeErrorMessage($orderMode);
                    }
                } else {
                    // return $this->getOrderModeErrorMessage($orderMode);
                }
            }
            return $timing;
        } else {
            return "";
        }
    }

    function checkOpenStatus($clientId, $orderTime)
    {
        $openStatus = "";
        $getActiveOpenClose = $this->getOpenClose($clientId, $orderTime);
        $getActiveHoliday = $this->getHoliday($clientId, $orderTime);
        $getActiveBankHoliday = $this->getBankHoliday($clientId, $orderTime);

        if (count($getActiveOpenClose) != 0) {
            $openStatus = $getActiveOpenClose['openTime'];
        } elseif (count($getActiveHoliday) != 0) {
            $openStatus = $getActiveHoliday['message'];
        } elseif (count($getActiveBankHoliday) != 0) {
            $openStatus = $getActiveBankHoliday['message'];
        }
        return $openStatus;
    }

    function getOrderModeErrorMessage($orderMode)
    {
        if ($orderMode == 0) {
            $orderMode = "Delivery";
        } else if ($orderMode == 1) {
            $orderMode = "Collection";
        }
        return $orderMode . " not available";
    }

    public function getCookingTime($clientId)
    {
        $record = array();
        $client = new Client($clientId);
        $cooking_time = $client->query("call getCollectionTime()");
        $client->connectionClose();

        if ($cooking_time->collection || $cooking_time->delivery) {
            $record['delivery'] = $cooking_time->delivery;
            $record['collection'] = $cooking_time->collection;
        } else {
            $day = date('w');
            if ($day == 0) {
                $col_cooking = "collection7";
                $del_cooking = "delivery7";
            } else {
                $col_cooking = "collection" . $day;
                $del_cooking = "delivery" . $day;
            }
            $record['collection'] = $cooking_time->$col_cooking;
            $record['delivery'] = $cooking_time->$del_cooking;
        }

        // $stuart = $this->getStuartStatus($clientId);
        // if($stuart->status == 1) {
        //     if($cooking_time->stuart_time)
        //         $record['stuart_time'] = $cooking_time->stuart_time;
        //     else
        //         $record['stuart_time'] = "20";

        //     $record['extraPrepTime'] = $cooking_time->extraPrepTime;
        // }

        return $record;
    }

    public function getTatStartEnd($clientId)
    {
        $cooking = $this->getCookingTime($clientId);
        $default = 10;

        $record['deliveryCookingTimeStart'] = intval($cooking["delivery"]);
        $record['deliveryCookingTimeEnd'] = intval($cooking["delivery"]) + $default;
        $record['collectionCookingTimeStart'] = intval($cooking["collection"]);
        $record['collectionCookingTimeEnd'] = intval($cooking["collection"]) + $default;

        return $record;
    }

    public function getTatEnableStatus($clientId)
    {
        $client = new Client($clientId);
        $tat = $client->query("call getTatEnableStatus()");
        $client->connectionClose();
        return $tat;
    }

    public function getMode($clientId)
    {
        $client = new Client($clientId);
        $mode = $client->query("call getOrderMode()");
        $client->connectionClose();
        return $mode;
    }

    public function getOpenClose($clientId, $orderTime)
    {
        $client = new Client($clientId);
        $openTime = array();
        $open = $client->query("call getOpenClose()");
        $client->connectionClose();
        if ($open == true) {
            if ($open->opentime > $orderTime) {
                $openTime['message'] = $open->info;
                $openTime['status'] = "0";
                $openTime['openTime'] = $open->opentime;
            }
        }
        return $openTime;
    }

    public function getholidayDates($clientId)
    {
        $client = new Client($clientId);
        $holidayResponse = array();
        $holidayDatas = $client->queryAll("call getHoliday()");
        $client->connectionClose();
        foreach ($holidayDatas as $holidayData) 
        {
             $today = strtotime("today");
             if($holidayData->date >= $today)
             {
                $dateform = date('Y-m-d',$holidayData->date);
                array_push($holidayResponse,$dateform);
             }
        }
        return $holidayResponse;
    }

    public function getHoliday($clientId, $orderTime)
    {
        $client = new Client($clientId);
        $holidayResponse = array();
        $holidayDatas = $client->queryAll("call getHoliday()");
        $client->connectionClose();
        if ($holidayDatas == true) {
            $hours = date('H', $orderTime);
            if ($hours >= 8) {
                $orderDate = $orderTime;
            } else {
                $orderDate = strtotime(date('m/d/Y', strtotime('-1 day', $orderTime)));
            }
            foreach ($holidayDatas as $holidayData) {
                $holidayDate = date('Y-m-d', $holidayData->date);
                $currentDate = date('Y-m-d', $orderDate);
                if ($currentDate == $holidayDate) {
                    $holidayResponse['message'] = $holidayData->desc;
                    $holidayResponse['status'] = "0";
                }
            }
        }
        return $holidayResponse;
    }

    public function getBankHoliday($clientId, $orderTime)
    {
        $client = new Client($clientId);
        $bankHolidayResponse = array();
        $bankHolidayDatas = $client->queryAll("call getBankHoliday()");
        $client->connectionClose();
        if ($bankHolidayDatas == true) {
            $hours = date('H', $orderTime);
            if ($hours >= 8) {
                $orderDate = $orderTime;
            } else {
                $orderDate = strtotime(date('m/d/Y', (strtotime('-1 day', $orderTime))));
            }
            foreach ($bankHolidayDatas as $bankHolidayData) {
                $holidayDate = date('Y-m-d', $bankHolidayData->date);
                $currentDate = date('Y-m-d', $orderDate);
                if ($currentDate == $holidayDate) {
                    $bankHolidayResponse['message'] = $bankHolidayData->desc;
                    $bankHolidayResponse['status'] = "0";
                }
            }
        }
        return $bankHolidayResponse;
    }

    public function getShiftTime($clientId, $time = "")
    {
        if ($time == "")
            $day = date('N');
        else
            $day = date('N', $time);

        # 1 - mon, 2 - tues ,3 - wed, 4 - thur, 5 - fri, 6 - sat, 7 - sun
        $client = new Client($clientId);
        return $client->queryAll("call getShift($day)");
    }

    public function getStuartStatus($clientId)
    {
        $stuartType = 3;
        $client = new Client($clientId);
        $stuart = $client->query("call getStuartStatus($stuartType)");
        $client->connectionClose();
        return $stuart;
    }

    public function getItemDetail($clientId, $itemId, $orderMode, $day)
    {
        $visible = 0;
        $item = $this->query("CALL getItemDetail($clientId, $itemId, $orderMode, $day, $visible)");
        $item = json_decode(json_encode($item), true);
        return $item;
    }

    public function getItemsList(int $clientId, int $day)
    {
        $item = $this->queryAll("CALL GetItemsByVisibilityAndDay($clientId, 0, $day);");

        if ($item === false) {
            return [];
        }

        $items = json_decode(json_encode($item), true);

        return $items;
    }

    public function getClientUsingB2cPath($clientPath)
    {
        return $this->query("CALL getClientUsingB2cPath('$clientPath')");
    }

    # New menu price condition checking
    public function getItemPriceValue($itemPrice, $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, $orderMode)
    {
        if ($percentageType == 1) {
            $extra = $extraPrice;
        } else {
            $extra = $itemPrice * $extraPrice / 100;
        }

        if (strpos($priceMode, "12") !== false) {
            $priceMode = 2;
        } else {
            if (strpos($priceMode, "1") !== false) {
                $priceMode = 0;
            }
            if (strpos($priceMode, "2") !== false) {
                $priceMode = 1;
            }
        }

        if (isset($orderMode) && (($orderMode == 0 && ($priceMode == 0 || $priceMode == 2)) || ($orderMode == 1 && ($priceMode == 1 || $priceMode == 2)))) {
            if (strpos($priceEnable, "1") !== false) {
                if ($increaseType == '1') {
                    return number_format($itemPrice - $extra, 2);
                } else {
                    return number_format($itemPrice + $extra, 2);
                }
            } else {
                return number_format($itemPrice, 2);
            }
        } else {
            return number_format($itemPrice, 2);
        }
    }

    public function getItemofferpercentage($productPrice,$selleingprice){
        if ($productPrice == 0) {
            return 0;
        }
        $percentage = (($productPrice - $selleingprice) / $productPrice) * 100;
        return $percentage;
    }

    public function getContacts($clientId)
    {
        $client = new Client($clientId);
        $contact = $client->query("CALL getContacts()");
        $client->connectionClose();
        return $contact;
    }

    public function getHygieneRating($clientId)
    {
        $client = new Client($clientId);
        $hygiene = $client->query("CALL getReservation()");
        $client->connectionClose();
        return $hygiene;
    }

    public function getDays($clientId)
    {
        $client = new Client($clientId);
        $getDays = $client->queryAll("CALL getDays()");
        $client->connectionClose();
        return $getDays;
    }

    public function getPagesWithAbout($clientId)
    {
        $client = new Client($clientId);
        $getDays = $client->query("CALL getPages('about')");
        $client->connectionClose();
        return $getDays;
    }

    public function getFeedbackRate($clientId)
    {
        $client = new Client($clientId);
        $feedback = $client->query("CALL getFeedbackRate()");
        $client->connectionClose();
        return $feedback;
    }

    # review page function
    public function getReviewData($clientId)
    {
        $review_array = array();
        $client = new Client($clientId);
        $reviews = $client->queryAll("CALL getFeedback()");
        foreach ($reviews as $review) {
            $data = array();
            $data['id'] = (int)$review->id;
            $data['rate'] = (int)$review->rate;
            $data['name'] = $review->name;
            $data['date'] = date("d M Y", strtotime($review->datetime));
            $data['comment'] = $review->comment;
            array_push($review_array, $data);
        }
        $client->connectionClose();

        return $review_array;
    }

    #about page function
    public function getAboutData($clientId)
    {
        $data = array();
        $about_array = array();
        $contact = $this->getContacts($clientId);
        $reservation = $this->getHygieneRating($clientId);
        $getAboutContent = $this->query("CALL getAboutContent($clientId)");

        $data['food_rate_img'] = ($reservation->image1 != "1" && $reservation->image1 != "0" && $reservation->image1 != "")
            ? $_ENV['FUSION_BUCKET'] . $reservation->image1
            : "";

        $data['client']['email'] = $contact->email;
        $data['client']['name'] = $contact->name;
        $data['client']['address1'] = $contact->address1;
        $data['client']['address2'] = $contact->address2;
        $data['client']['city'] = $contact->city;
        $data['client']['state'] = $contact->state;
        $data['client']['pincode'] = $contact->pincode;
        $data['client']['city'] = $contact->city;

        $data['aboutAs'] = $getAboutContent;

        // Get the current date
        $currentDate = date('Y-m-d');

        // Calculate the start and end dates of the current week
        $startOfWeek = strtotime('this week', strtotime($currentDate));
        $endOfWeek = strtotime('next week', strtotime($currentDate));
        $shiftDate = strtotime(date('d-m-Y', $startOfWeek) . " " . "08:00");
        $openingHours = array();
        while ($shiftDate < $endOfWeek) {
            $time = array();
            $day = date('l', $shiftDate);
            $time[$day]['collection'] = $this->getAboutBusinessTime($clientId, 1, $shiftDate);
            $time[$day]['delivery'] = $this->getAboutBusinessTime($clientId, 0, $shiftDate);
            array_push($openingHours, $time);
            $shiftDate = strtotime('+1 day', $shiftDate);
        }
        $data['day'] = $openingHours;
        return $data;
    }

    public function getAboutBusinessTime($clientId, $orderMode, $currentTime)
    {
        $getMode = "01";
        $getOrderMode = str_split($getMode);
        if ($orderMode == 0) {
            $type1 = "delivery_openat";
            $type2 = "delivery_closeat";
            $type = "delivery";
        } elseif ($orderMode == 1) {
            $type1 = "collection_openat";
            $type2 = "collection_closeat";
            $type = "collection";
        }

        # get current date
        $hours = date('H', $currentTime);
        $hourMinutes = date('H:i', $currentTime);

        $currentDate = strtotime(date('d-m-Y', $currentTime));
        $previousDate = strtotime(date('d-m-Y', (strtotime('-1 day', $currentTime))));
        $getPreShiftTime = $this->getShiftTime($clientId, $previousDate);
        $getCurrentShiftTime = $this->getShiftTime($clientId, $currentDate);
        $getShiftTime = array_merge($getPreShiftTime, $getCurrentShiftTime);

        # business shift functionality
        if (count($getShiftTime) != 0) {
            $timing = array();
            foreach ($getShiftTime as $shift) {
                $time_1 = $shift->$type1;
                $time_2 = $shift->$type2;

                if ($time_1 != "" && $time_2 != "") {
                    # get start time based on current time
                    $day = date('N', $currentDate);
                    if ($day > $shift->day || $day < $shift->day) {
                        $startTimestamp = ("08:00" < $time_1) ? "00:00" : $time_1;
                        $endTimestamp = ("08:00" > $time_2) ? $time_2 : "";
                    } else if ($day == $shift->day) {
                        $startTimestamp = $time_1;
                        $endTimestamp = ("08:00" < $time_2) ? $time_2 : "23:59";
                    }
                    # get start and end date
                    $startDate = date('d-m-Y', $currentDate);
                    $endDate = date('d-m-Y', $currentDate);
                    if (in_array($orderMode, $getOrderMode)) {
                        if ($shift->$type == 1) {
                            if ($startTimestamp != "" && $endTimestamp != "") {
                                $startTime = strtotime($startDate . " " . $startTimestamp);
                                $targetTime = strtotime($endDate . " " . $endTimestamp);
                                $time =  date('H:i', $startTime) . ' - ' . date('H:i', $targetTime);
                                array_push($timing, $time);
                            }
                        }
                    }
                }
            }
            return $timing;
        } else {
            return [];
        }
    }

    public function getRestaurantDetails($postCode, $clientPath, $orderMode, $customerId, $orderTime)
    {
        $current_time = time();
        if ($postCode != "") {
            $googleApi = new PostcodeController();
            $getPostCodeDetail = $googleApi->getPostCodeDetail($postCode);
            $getPostCodeDetail = json_decode($getPostCodeDetail);
            if (!empty($getPostCodeDetail)) {
                if ($getPostCodeDetail->status) {
                    $latitude = $getPostCodeDetail->latitude;
                    $longitude = $getPostCodeDetail->longitude;
                } else {
                    $latitude = 0;
                    $longitude = 0;
                }
            } else {
                $latitude = 0;
                $longitude = 0;
            }
        } else {
            $latitude = 0;
            $longitude = 0;
        }

        $menuData = array();
        $restaurantPage = new RestaurantPage();
        $getClientId = $this->getClientUsingB2cPath($clientPath);

        if ($getClientId == false) {
            return "Incorrect Path";
        }

        $client = $this->query("call getB2cClientData($latitude,$longitude,$getClientId->id)");
        $getFeedbackRate = $this->getFeedbackRate($getClientId->id);
        $getCookingTime = $this->getCookingTime($getClientId->id);
        $getTatStartEnd = $this->getTatStartEnd($getClientId->id);
        $getMin = $this->getMin($getClientId->id);
        $getStuartStatus = $this->getStuartStatus($getClientId->id);

        $menuData['clientId'] = $client->client_id;
        $menuData['clientPath'] = $clientPath;
        $menuData['restaurantName'] = $getClientId->takeaway_name;

        $shiftTimes = $this->getShiftTime($getClientId->id);

        $menuData['deliveryShiftTime'] = [
            'openAt' => isset($shiftTimes[0]->delivery_openat) ? $shiftTimes[0]->delivery_openat : 0,
            'closeAt' => isset($shiftTimes[0]->delivery_closeat) ? $shiftTimes[0]->delivery_closeat : 0,
        ];

        $menuData['collectionShiftTime'] = [
            'openAt' => isset($shiftTimes[0]->collection_openat) ? $shiftTimes[0]->collection_openat : 0,
            'closeAt' => isset($shiftTimes[0]->collection_closeat) ? $shiftTimes[0]->collection_closeat : 0,
        ];

        # get logo image 
        $menuData['logo'] = $client->logo
            ? $_ENV['FUSION_BUCKET'] . $client->logo
            : $_ENV['FUSION_BUCKET'] . "takeaway1.jpg";

        # get background image 
        $menuData['backgroundImage'] = $client->bg_image
            ? $_ENV['FUSION_BUCKET'] . $client->bg_image
            : $_ENV['FUSION_BUCKET'] . "takeaway1.jpg";

        # get favourite list
        $menuData['favourite'] = $customerId ? (($restaurantPage->getFavouriteById($customerId, $getClientId->id)) ? true : false) : false;

        # get miles and rating
        $menuData['miles'] = number_format($client->distance, 2) . ' Miles away';
        $menuData['rating'] = $getFeedbackRate;

        # get minimum order and tat time
        if ($orderMode == 0) {
            if ($getStuartStatus && $getStuartStatus->status == "1") 
            {
                $menuData['minOrder'] = (floatval($getMin->stuart_delivery_min) >= floatval($getMin->delivery_min)) 
                                        ? $getMin->stuart_delivery_min : $getMin->delivery_min;
                $menuData['stuart']   = true;
            } else {
                $menuData['minOrder'] = $getMin->delivery_min;
                $menuData['stuart']   = false;
            }
        } elseif ($orderMode == 1) {
            $menuData['minOrder'] = $getMin->collection_min;
            $menuData['stuart']   = false;
        }

        # get new minimum order and tat time
        if ($getStuartStatus && $getStuartStatus->status == "1") 
        {
            $menuData['newMinOrder']['deliveryMinOrder'] = (floatval($getMin->stuart_delivery_min) >= floatval($getMin->delivery_min))
                                                           ? $getMin->stuart_delivery_min : $getMin->delivery_min ; 
            $menuData['newStuart']['deliveryStuart']     = true;
        } else {
            $menuData['newMinOrder']['deliveryMinOrder'] = $getMin->delivery_min;
            $menuData['newStuart']['deliveryStuart'] = false;
        }

        $menuData['newMinOrder']['collectionMinOrder'] = $getMin->collection_min;
        $menuData['newStuart']['collectionStuart'] = false;

        $menuData['deliveryCookingTimeStart'] = $getTatStartEnd["deliveryCookingTimeStart"];
        $menuData['deliveryCookingTimeEnd'] = $getTatStartEnd["deliveryCookingTimeEnd"];
        $menuData['collectionCookingTimeStart'] = $getTatStartEnd["collectionCookingTimeStart"];
        $menuData['collectionCookingTimeEnd'] = $getTatStartEnd["collectionCookingTimeEnd"];

        # Check restaurant open close status functionalities
        $openCloseCollectionStatus = $restaurantPage->restaurantOpenCloseStatus($getClientId->id, $current_time, "1");
        $menuData['restaurantStatus']['Collection'] = $openCloseCollectionStatus;
        $openCloseDeliveryStatus = $restaurantPage->restaurantOpenCloseStatus($getClientId->id, $current_time, "0");
        $menuData['restaurantStatus']['Delivery'] = $openCloseDeliveryStatus;

        # get discount 
        $menuData['discount'] = $this->getOfferDetails($getClientId->id, $orderMode, $orderTime, $customerId, $clientPath, false);
        $menuData['deliveryDiscount'] = $this->getOfferDetails($getClientId->id, 0, $orderTime, $customerId, $clientPath, true);
        $menuData['collectionDiscount'] = $this->getOfferDetails($getClientId->id, 1, $orderTime, $customerId, $clientPath, true);

        # get cuisine list
        $cuisines = explode(",", $client->cuisine);
        $menuData['cuisine'] = array();
        foreach ($cuisines as $key => $cuisine) {
            $cuisineName = $this->query("CALL getCuisineImage($cuisine)");
            $menuData['cuisine'][$key]['name'] = str_replace("Â", "", $cuisineName->cuisine_name);
            $menuData['cuisine'][$key]['image'] = $cuisineName->cuisine_image;
        }

        $menuController = new MenuController();

        $menuData['timing']['Delivery'] = $menuController->getPreOrderTiming("0", $client->client_id);
        $menuData['timing']['Collection'] = $menuController->getPreOrderTiming("1", $client->client_id);

        return $menuData;
    }

    public function getMenuList($clientId, $orderMode, $day)
    {
        $client = new Client($clientId);
        $sort = $client->query("SELECT sorting FROM tbl_menu_sort LIMIT 1");
        $sort = $sort->sorting ?? 1;
        $visible = 0;
        $getMenu = $this->queryAll("CALL getMenuList($clientId, $orderMode, $day, $visible, $sort)");
        $getMenu = json_decode(json_encode($getMenu), true);
        $client->connectionClose();
        return $getMenu;
    }

    public function getBestSellerMustTry($clientId, $orderMode, $day)
    {
        # get best seller and must try items based on last 30 days orders
        $client = new Client($clientId);
        $startDate = strtotime(date('Y-m-d', strtotime('-30 days')));
        $endDate = time();
        $visible = 0;
        $getMostSold = $client->queryAll("CALL getMostSoldItem($startDate, $endDate, $orderMode, $day, $visible)");
        $getMostSold = json_decode(json_encode($getMostSold), true);
        $mostSold = array();
        $special = array();
        if (!empty($getMostSold)) {
            foreach ($getMostSold as $key => $value) {
                if ($value['categoryId'] != '') {
                    $mostSold[$value['categoryId']][] = $value;
                }
            }
            foreach ($mostSold as $kry => $kfk) {
                usort($kfk, function ($a, $b) {
                    return $a['count'] - $b['count'];
                });
                $data['bestSeller'] = array_slice($kfk, -1, 1);
                $data['mustTry'] = array_slice($kfk, -2, 1);
                $special['bestSeller'][] = $data['bestSeller'][0]['itemid'];
                $special['mustTry'][] = $data['bestSeller'][0]['itemid'] == $data['mustTry'][0]['itemid'] ? 0 : $data['mustTry'][0]['itemid'];
            }
        }
        return $special;
    }

    public function getMenus($clientId, $orderMode, $orderTime)
    {
        $zeroBasedDay = date('w', $orderTime);
        $oneBasedDay = date('N', $orderTime);

        // Fetch data
        $freeItem = $this->getFreeItem($clientId, $orderMode, $orderTime);
        $loyaltyItems = $this->getLoyalty($clientId, $orderMode, $orderTime);
        $getMenuList = $this->getMenuList($clientId, $orderMode, $zeroBasedDay);

        // Determine delivery and collection menus
        list($order_mode_delivery_menus, $order_mode_collection_menus, $free_item_delivery_menus, $free_item_collection_menus) = $this->getOrderModeMenus($clientId, $orderMode, $zeroBasedDay, $orderTime, $getMenuList, $freeItem);

        // Check availability for delivery and collection items
        $order_mode_delivery_items = $this->checkAvailability($order_mode_delivery_menus, $oneBasedDay, $orderTime);
        $order_mode_collection_items = $this->checkAvailability($order_mode_collection_menus, $oneBasedDay, $orderTime);

        // Get best seller and must try items
        $getBestSellerMustTry = $this->getBestSellerMustTry($clientId, $orderMode, $zeroBasedDay);
        $getBestSeller = $getBestSellerMustTry['bestSeller'] ?? [];
        $getMustTry = $getBestSellerMustTry['mustTry'] ?? [];

        // Process menu list
        $menuList = $this->processMenuList($getMenuList, $oneBasedDay, $orderTime, $order_mode_delivery_items, $order_mode_collection_items, $getBestSeller, $getMustTry, $orderMode);

        // Merge items based on category and subcategory
        $menuData = $this->mergeMenuItems($menuList);

        // Category List
        $categoryList = [];

        foreach ($menuData as $menu) {
            $categoryList[] = [
                "id" => $menu["categoryId"],
                "image" => $menu["categoryImage"],
                "name" => $menu["categoryName"]
            ];
        }
        
        // Prepare result
        $result = [
            'categoryList' => $categoryList,
            'freeItem' => $freeItem,
            'collectionFreeItem' => $free_item_collection_menus,
            'deliveryFreeItem' => $free_item_delivery_menus,
            'loyaltyItem' => $loyaltyItems,
            'menuData' => $menuData,
        ];

        return $result;
    }

    private function getOrderModeMenus($clientId, $orderMode, $zeroBasedDay, $orderTime, $getMenuList, $freeItem)
    {
        if ($orderMode == "0") {
            $order_mode_delivery_menus = $getMenuList;
            $order_mode_collection_menus = $this->getMenuList($clientId, "1", $zeroBasedDay);
            $free_item_delivery_menus = $freeItem;
            $free_item_collection_menus = $this->getFreeItem($clientId, "1", $orderTime);
        } else {
            $order_mode_collection_menus = $getMenuList;
            $order_mode_delivery_menus = $this->getMenuList($clientId, "0", $zeroBasedDay);
            $free_item_delivery_menus = $this->getFreeItem($clientId, "0", $orderTime);
            $free_item_collection_menus = $freeItem;
        }

        return [$order_mode_delivery_menus, $order_mode_collection_menus, $free_item_delivery_menus, $free_item_collection_menus];
    }

    private function checkAvailability($menus, $oneBasedDay, $orderTime)
    {
        $availableItems = [];
        foreach ($menus as $menu) {
            $currentTime = strtotime(date('h:i A', $orderTime));
            $categoryTime = $menu["categoryTime" . $oneBasedDay];
            $subCategoryTime = $menu["subCategoryTime" . $oneBasedDay];
            $itemTime = $menu["itemTime" . $oneBasedDay];
            $available = $this->checkAvailableItem($categoryTime, $subCategoryTime, $itemTime, $currentTime);
            $availableItems[$menu["itemId"]] = $available;
        }
        return $availableItems;
    }

    private function processMenuList($getMenuList, $oneBasedDay, $orderTime, $order_mode_delivery_items, $order_mode_collection_items, $getBestSeller, $getMustTry, $orderMode)
    {
        $menuList = [];
        foreach ($getMenuList as $menu) {
            $currentTime = strtotime(date('h:i A', $orderTime));
            $categoryTime = $menu["categoryTime" . $oneBasedDay];
            $subCategoryTime = $menu["subCategoryTime" . $oneBasedDay];
            $itemTime = $menu["itemTime" . $oneBasedDay];
            $available = $this->checkAvailableItem($categoryTime, $subCategoryTime, $itemTime, $currentTime);

            $productPrice = $this->getItemPriceValue($menu['productPrice'], $menu['fkPriceType'], $menu['fkPercentageType'], $menu['fkPrice'], $menu['fkPriceStatus'], $menu['fkPriceMode'], $orderMode);
            $price = $this->getItemPriceValue($menu['itemPrice'], $menu['fkPriceType'], $menu['fkPercentageType'], $menu['fkPrice'], $menu['fkPriceStatus'], $menu['fkPriceMode'], $orderMode);
            $deliveryPrice = $this->getItemPriceValue($menu['deliveryPrice'], $menu['fkPriceType'], $menu['fkPercentageType'], $menu['fkPrice'], $menu['fkPriceStatus'], $menu['fkPriceMode'], 0);
            $collectionPrice = $this->getItemPriceValue($menu['collectionPrice'], $menu['fkPriceType'], $menu['fkPercentageType'], $menu['fkPrice'], $menu['fkPriceStatus'], $menu['fkPriceMode'], 1);
            $offerpercentage = $this->getItemofferpercentage($menu['productPrice'], $menu['itemPrice']);

            # best seller and must try getting values
            if (in_array($menu['itemId'], $getBestSeller)) {
                $bestSeller = true;
            } else {
                $bestSeller = false;
            }
            if (in_array($menu['itemId'], $getMustTry)) {
                $mustTry = true;
            } else {
                $mustTry = false;
            }

            if($menu['item_18plus'] == 1 || $menu['item_18plus'] == '1'){
                $menu['item_18plus'] = true;
            }else{
                $menu['item_18plus'] = false;
            }
            if (isset($menu['fmb_sts']) && ($menu['fmb_sts'] == 1 || $menu['fmb_sts'] == '1'|| $menu['fmb_sts'] == 3 || $menu['fmb_sts'] == '3')) {
                $menu['Delivery_ItemStock'] = true;
                $menu['collection_ItemStock'] = true;
            } else {
                $menu['Delivery_ItemStock'] = false;
                $menu['collection_ItemStock'] = false;
            }
            
           $customizeNew = false;
           if(isset($menu['priority_type'])){
                if (strpos($menu['priority_type'], '0') !== false) {
                    }else{
                    if (strpos($menu['priority_type'], '1') !== false) {
                        $mustTry = true;
                    }
                    if (strpos($menu['priority_type'], '2') !== false) {
                        $bestSeller = true;
                    }
                    if (strpos($menu['priority_type'], '3') !== false) {
                        $customizeNew = true;
                    }
                }
            }
            
            $multiImage = explode(",", $menu['multiImage']);
            $image1 = "";
            $multipleImages = [];
            if ($menu['itemImage']) {
                $multipleImages[] = $_ENV['FUSION_BUCKET'] . 'menu/' . $menu['itemImage'];
            }
            foreach ($multiImage as $key => $image) {
                if ($key == 0) {
                    $image1 = $image
                        ? $_ENV['FUSION_BUCKET'] . 'menu/' . $image
                        : "";
                }
                if ($image) {
                    $multipleImages[] = $_ENV['FUSION_BUCKET'] . 'menu/' . $image;
                }
            }

            if ($menu['categoryCoupon'] == 1 && $menu['subCategoryCoupon'] == 1 && $menu['itemCoupon'] == 1) {
                $allowCoupon = 1;
            } else {
                $allowCoupon = 0;
            }
            $collection_available = "Collection Not Available";
            $delivery_available = "Delivery Not Available";
            // echo json_encode($order_mode_delivery_items);
            // echo json_encode($order_mode_collection_items);
            if (isset($order_mode_delivery_items[$menu['itemId']])) {
                $delivery_available = $order_mode_delivery_items[$menu['itemId']];
            }
            if (isset($order_mode_collection_items[$menu['itemId']])) {
                $collection_available = $order_mode_collection_items[$menu['itemId']];
            }
            $imageIntrinsicSize="";
            $imageAspectRatio="";
            $imageTypePath=isset($menu['itemImage']) && $menu['itemImage']
            ? $_ENV['FUSION_BUCKET'] . 'menu/' . $menu['itemImage']
            : "";
            if($imageTypePath!='')
            {
                //list($width, $height) = getimagesize($imageTypePath);
                // Function to calculate the greatest common divisor (GCD)
            
                $width=500;
                $height=500;
                // Check if dimensions are valid
                if ($width > 0 && $height > 0) {
                    // Calculate the GCD of the width and height
                    //$gcd = gcd($width, $height);

                    // Calculate the simplified aspect ratio
                    // $simplifiedWidth = $width / $gcd;
                    // $simplifiedHeight = $height / $gcd;

                    $simplifiedWidth =1;
                    $simplifiedHeight = 1;

                    $imageIntrinsicSize="{$width} × {$height}";  
                    $imageAspectRatio="{$simplifiedWidth}∶{$simplifiedHeight}";
                      
                } 
            }
            

            // Function to calculate the greatest common divisor (GCD)
           
            $menuList[] = array(
                'categoryId' => $menu['categoryId'],
                'categoryName' => str_replace(["<br>", "Â"], "", $menu['categoryName']),
                'categoryAnotherName' => str_replace(["<br>", "Â"], "", $menu['categoryAnotherName']),
                'categoryImage' => $menu['categoryImage'] ? $_ENV['FUSION_BUCKET'] . 'menu/' . $menu['categoryImage'] : "",
                'subCategory' => [
                    'subCategoryId' => str_replace(["<br>", "Â"], "", $menu['subCategoryId']),
                    'subCategoryName' => str_replace(["<br>", "Â"], "", $menu['subCategoryName']),
                    'subCategoryAnotherName' => str_replace(["<br>", "Â"], "", $menu['subCategoryAnotherName']),
                    'subCategoryDesc' => str_replace(["<br>", "Â"], "", $menu['subCategoryDesc']),
                    'subCategoryImg' => $menu['subCategoryImage']
                    ? $_ENV['FUSION_BUCKET'] . 'menu/' . $menu['subCategoryImage']
                    : "",
                    'item' => array(
                        'itemId' => $menu['itemId'],
                        'itemImage' => $menu['itemImage'] ? $_ENV['FUSION_BUCKET'] . 'menu/' . $menu['itemImage'] : "",
                        'ItemImageIntrinsicSize' => $imageIntrinsicSize,
                        'ItemImageAspectRatio' => $imageAspectRatio,
                        'itemImage1' => $image1,
                        'multipleImages' => $multipleImages,
                        'itemName' => str_replace(["<br>", "Â"], "", $menu['itemName']),
                        'itemEmojiName' => str_replace(["<br>", "Â"], "", $menu['emojiName']),
                        'itemAnotherName' => str_replace(["<br>", "Â"], "", $menu['itemAnotherName']),
                        'itemDesc' => str_replace(["<br>", "Â"], "", $menu['itemDesc']),
                        'available' => $available,
                        'collection_available' => $collection_available,
                        'delivery_available' => $delivery_available,
                        'product_price' => $productPrice,
                        'selling_price' => $price,
                        'price' => $price,
                        'delivery_price' => $deliveryPrice,
                        'collection_price' => $collectionPrice,
                        'type' => $menu['itemType'] ?? 'NONE',
                        'addon' => $menu['itemAddon'],
                        'freeAddon' => $menu['itemFreeAddon'],
                        'itemAllowCoupon' => $allowCoupon,
                        'bestSeller' => $bestSeller,
                        'mustTry' => $mustTry,
                        "customizeNew" => $customizeNew,
                        "priority_type" => $menu['priority_type'],
                        'age_restriction' => $menu['item_18plus'],
                        'Delivery_ItemStock' => $menu['Delivery_ItemStock'],
                        'collection_ItemStock' => $menu['collection_ItemStock'],
                        'offerpercentage' =>round($offerpercentage)
                    )
                ]);
        }
        return $menuList;
    }

    private function getMultipleImages($menu)
    {
        $multipleImages = [];
        if ($menu['itemImage']) {
            $multipleImages[] = $_ENV['FUSION_BUCKET'] . 'menu/' . $menu['itemImage'];
        }
        $multiImage = explode(",", $menu['multiImage']);
        foreach ($multiImage as $image) {
            if ($image) {
                $multipleImages[] = $_ENV['FUSION_BUCKET'] . 'menu/' . $image;
            }
        }
        return $multipleImages;
    }

    private function mergeMenuItems($menuList)
    {
        $categoryData = [];
        foreach ($menuList as $menu) {
            $categoryId = $menu['categoryId'];
            $subCategoryId = $menu['subCategory']['subCategoryId'];

            if (!isset($categoryData[$categoryId])) {
                $categoryData[$categoryId] = [
                    'categoryId' => $menu['categoryId'],
                    'categoryName' => $menu['categoryName'],
                    'categoryAnotherName' => $menu['categoryAnotherName'],
                    'categoryImage' => $menu['categoryImage'],
                    'subCategory' => [],
                ];
            }

            if (!isset($categoryData[$categoryId]['subCategory'][$subCategoryId])) {
                $categoryData[$categoryId]['subCategory'][$subCategoryId] = [
                    'subCategoryId' => $menu['subCategory']['subCategoryId'],
                    'subCategoryName' => $menu['subCategory']['subCategoryName'],
                    'subCategoryAnotherName' => $menu['subCategory']['subCategoryAnotherName'],
                    'subCategoryDesc' => $menu['subCategory']['subCategoryDesc'],
                    'item' => [],
                ];
            }

            $categoryData[$categoryId]['subCategory'][$subCategoryId]['item'][] = $menu['subCategory']['item'];
            $categoryData[$categoryId]['subCategory'][$subCategoryId]['subCategoryImg'] = $menu['subCategory']['subCategoryImg'];
        }

        $menuData = [];
        foreach ($categoryData as $category) {
            $category['subCategory'] = array_values($category['subCategory']);
            $menuData[] = $category;
        }

        return $menuData;
    }

    public function getLoyalty($clientId, $orderMode, $orderTime)
    {
        $zeroBasedDay = date('w', $orderTime);
        $oneBasedDay = date('N', $orderTime);
        $getOfferList = $this->query("CALL getOfferBanner($clientId)");

        if ($getOfferList && $getOfferList->loyalty != 0) {
            $loyaltyItem = array();
            # get free gift details
            $getLoyaltyGift = $this->getLoyaltyGift($clientId);
            if ($getLoyaltyGift != false) {
                foreach ($getLoyaltyGift as $key => $items) {
                    # get item details
                    $getItemDetail = $this->getItemDetail($clientId, $items->itemId, $orderMode, $zeroBasedDay);
                    if ($getItemDetail != false) {
                        $itemDetail['point'] = $items->points;
                        $itemDetail['itemId'] = $getItemDetail['itemId'];
                        $itemDetail['description'] = str_replace(["<br>", "Â"], "", $getItemDetail['itemDesc']);
                        $itemDetail['allowoffer'] = $getItemDetail['itemAllowOffer'];
                        $itemDetail['allowgift'] = $getItemDetail['itemAllowGift'];
                        $itemDetail['addon'] = $getItemDetail['itemAddon'];
                        $itemDetail['anotherName'] = str_replace(["<br>", "Â"], "", $getItemDetail['itemAnotherName']);
                        $itemDetail['itemName'] = str_replace(["<br>", "Â"], "", $getItemDetail['itemName']);

                        if ($getItemDetail['itemImage'] != "") {
                            $itemDetail['image'] = $_ENV["FUSION_BUCKET"] . "menu/" . $getItemDetail['itemImage'];
                        } else {
                            $itemDetail['image'] = "";
                        }
                        $itemDetail['price'] = "0.00";

                        # item available function based on category,sub category and item working times 
                        $categoryTime = $getItemDetail["categoryTime" . $oneBasedDay];
                        $subCategoryTime = $getItemDetail["subCategoryTime" . $oneBasedDay];
                        $itemTime = $getItemDetail["itemTime" . $oneBasedDay];
                        $available = $this->checkAvailableItem($categoryTime, $subCategoryTime, $itemTime, $orderTime);
                        $itemDetail['available'] = $available;
                        array_push($loyaltyItem, $itemDetail);
                    } else {
                        return false;
                    }
                }
            } else {
                return false;
            }
            return $loyaltyItem;
        } else {
            return false;
        }
    }

    public function getLoyaltyGift($clientId)
    {
        return $this->queryAll("CALL getLoyaltyGift($clientId)");
    }

    public function createLoyaltyWithOrder($customerId, $clientId, $totalAmount, $itemsOrOrderId, $point)
    {
        $datetime = time();
        if ($point == "1") {
            # get ordered points count
            $orderedPoint = 0;
            $description = "";
            foreach ($itemsOrOrderId as $key => $item) {
                $getItemPoint = $this->query("CALL getLoyaltyGiftWithItem($clientId, $key)");
                if ($getItemPoint != false) {
                    $orderedPoint += $getItemPoint->points;
                    $description = isset($description) && !empty($description)
                        ? $description . ', ' . $item
                        : $item;
                }
            }
            $creditPoint = "0";
            $debitPoint = $orderedPoint;

            $createLoyalty = $this->query("CALL createLoyalty($customerId, $creditPoint, $debitPoint, $clientId, '$description', $datetime, @insert_id)");
            return $createLoyalty;
        }

        if ($point == "2") {
            # get customer points
            $getLoyaltyPoints = $this->query("CALL getLoyaltyPoints($totalAmount)");
            if ($getLoyaltyPoints->success != 0) {
                $point = $getLoyaltyPoints->point;

                $creditPoint = $point;
                $debitPoint = "0";
                $description = "Order# " . $itemsOrOrderId . " Earn";

                $createLoyalty = $this->query("CALL createLoyalty($customerId, $creditPoint, $debitPoint, $clientId, '$description', $datetime, @insert_id)");
                return $createLoyalty;
            }
        }
    }

    public function getLoyaltyAmount($orderedAmount)
    {
        # get customer points
        $getLoyaltyPoints = $this->query("CALL getLoyaltyPoints($orderedAmount)");
        if ($getLoyaltyPoints->success != 0) {
            $points['minAmount'] = $getLoyaltyPoints->amount;
            $points['point'] = $getLoyaltyPoints->point;
        } else {
            $points['minAmount'] = "";
            $points['point'] = "";
        }
        $points['status'] = $getLoyaltyPoints->success;
        return $points;
    }

    public function getFreeItem($clientId, $orderMode, $orderTime)
    {
        $zeroBasedDay = date('w', $orderTime);
        $oneBasedDay = date('N', $orderTime);
        $getOfferList = $this->query("CALL getOfferBanner($clientId)");
        if ($getOfferList && $getOfferList->freeGift != 0) {
            # get free gift details
            $getFreeGift = $this->getFreeGift($clientId, $orderMode, $zeroBasedDay);
            if ($getFreeGift != false) {
                if ($getFreeGift->validityFrom <= $orderTime && $orderTime <= $getFreeGift->validityTo) {
                    $freeGift['detail']['minAmount'] = $getFreeGift->minAmount;
                    $freeGift['detail']['orderType'] = $getFreeGift->orderType;
                    $freeGift['detail']['paymentType'] = $getFreeGift->paymentType;
                    $freeGift['item'] = array();
                    # get free gift item details
                    $getFreeItems = $this->getFreeGiftItems($clientId);
                    foreach ($getFreeItems as $items) {
                        # get item details
                        $getItemDetail = $this->getItemDetail($clientId, $items->itemId, $orderMode, $zeroBasedDay);
                        if ($getItemDetail != false) {
                            $itemDetail['itemId'] = $getItemDetail['itemId'];
                            $itemDetail['description'] = $getItemDetail['itemDesc'];
                            $itemDetail['allowoffer'] = $getItemDetail['itemAllowOffer'];
                            $itemDetail['allowgift'] = $getItemDetail['itemAllowGift'];
                            $itemDetail['addon'] = $getItemDetail['itemAddon'];
                            $itemDetail['type'] = $getItemDetail['type'] ?? null;
                            $itemDetail['anotherName'] = str_replace("Â", "", $getItemDetail['itemAnotherName']);
                            $itemDetail['itemName'] =  str_replace(["<br>", "Â"], "", $getItemDetail['itemName']);

                            if ($getItemDetail['itemImage'] != "") {
                                $itemDetail['image'] = $_ENV["FUSION_BUCKET"] . "menu/" . $getItemDetail['itemImage'];
                            } else {
                                $itemDetail['image'] = "";
                            }
                            $itemDetail['price'] = "0.00";
                            $itemDetail['minPrice'] = $items->price;

                            # item available function based on category,sub category and item working times 
                            $categoryTime = $getItemDetail["categoryTime" . $oneBasedDay];
                            $subCategoryTime = $getItemDetail["subCategoryTime" . $oneBasedDay];
                            $itemTime = $getItemDetail["itemTime" . $oneBasedDay];
                            $available = $this->checkAvailableItem($categoryTime, $subCategoryTime, $itemTime, $orderTime);
                            $itemDetail['available'] = $available;

                            array_push($freeGift['item'], $itemDetail);
                        }
                    }
                    return $freeGift;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    public function getFreeGift($clientId, $orderMode, $zeroBasedDay)
    {
        $client = new Client($clientId);
        $paymentMode = 1;
        $getFreeGift = $client->query("CALL getFreeGift($orderMode, $paymentMode, $zeroBasedDay)");
        $client->connectionClose();
        return $getFreeGift;
    }

    public function getFreeGiftItems($clientId)
    {
        $client = new Client($clientId);
        $getAddonDetail = $client->queryAll("CALL getFreeGiftItems()");
        $client->connectionClose();
        return $getAddonDetail;
    }

    public function checkAvailableItem($categoryTime, $subCategoryTime, $itemTime, $currentTime)
    {
        // Helper function to check if current time is within a given time range
        $isAvailable = function ($timeRange, $currentTime)
        {
            if (!$timeRange)
                return true; // If no time range, consider available
            [$from, $to] = explode('-', $timeRange);
            return strtotime($from) <= $currentTime && strtotime($to) >= $currentTime;
        };

        // Check availability at each level
        if (!$isAvailable($categoryTime, $currentTime)) {
            return "Available at " . $categoryTime;
        }

        if (!$isAvailable($subCategoryTime, $currentTime)) {
            return "Available at " . $subCategoryTime;
        }

        if (!$isAvailable($itemTime, $currentTime)) {
            return "Available at " . $itemTime;
        }

        return ""; // Available
    }

    public function getAddonDetail($clientId, $addonId)
    {
        $getAddonDetail = $this->query("CALL getAddonDetail($clientId, $addonId)");
        return $getAddonDetail;
    }

    public function getAddonItemDetailWithAddon($clientId, $addonId)
    {
        $getAddonItemDetailWithAddon = $this->queryAll("CALL getAddonItemDetailWithAddon($clientId, $addonId)");
        return $getAddonItemDetailWithAddon;
    }

    public function getAddonItemDetailWithId($clientId, $addonItemId)
    {
        $getAddonItemDetailWithId = $this->query("CALL getAddonItemDetailWithId($clientId, $addonItemId)");
        $getAddonItemDetailWithId = json_decode(json_encode($getAddonItemDetailWithId), true);
        return $getAddonItemDetailWithId;
    }

    public function getAddonList($data)
    {
        $addonItems = array();
        $selectedItem = array();
        $result = array();
        $addonItems["orderItemName"] = $data["orderItem"]["itemName"];
        $addonItems["orderItemId"] = $data["orderItem"]["itemId"];
        # get select items list
        if ($data["selected"]) {
            foreach ($data["selected"] as $key => $item) {
                $selectedItem[$key] = $item;
            }
        }
        $addonItems["selectedItem"] = $selectedItem;
        if ($data["addon"]) {
            $addonItems["addonDescription"] = $data["addon"]->description;
            if (strlen($data["addon"]->extra) > 0) {
                $values = array(
                    "No" => 0,
                    "Less" => 1,
                    "Half" => 2,
                    "On" => 3,
                    "With" => 4,
                    "OnBurger" => 5,
                    "OnChips" => 6,
                );
                $addExtra = array();
                foreach ($values as $key => $value) {
                    if (strpos($data["addon"]->extra, (string)$value) !== false) {
                        $addExtra[$key . 'id'] = $value;
                    }
                }
                $addonItems["addExtra"] = $addExtra;
            } else {
                $addonItems["addExtra"] = "";
            }
            $addonItems["addonLimit"] = $data["addon"]->limit;
            $addonItems["addonId"] = $data["addon"]->id;
            $addonItems["orderItemPrice"] = number_format($data["orderItem"]["itemPrice"], 2);
            $addonItems["orderDeliveryPrice"] = number_format($data["orderItem"]["deliveryPrice"], 2);
            $addonItems["orderCollectionPrice"] = number_format($data["orderItem"]["collectionPrice"], 2);
            $addonItems["nextAid"] = $data["addon"]->next;
        } else {
            $addonItems["addonDescription"] = "";
            $addonItems["addExtra"] = "";
            $addonItems["addonLimit"] = "";
            $addonItems["addonId"] = "";
            $addonItems["orderItemPrice"] = "";
            $addonItems["orderDeliveryPrice"] = "";
            $addonItems["orderCollectionPrice"] = "";
            $addonItems["nextAid"] = "";
        }

        foreach ($data["addonItems"] as $key => $addonItem) {
            $result[$key]["name"] = $addonItem->name;
            $result[$key]["price"] = ($addonItem->price == '0.0') ? "" : "+ ". $data['currency']['symbol'] ." " . $addonItem->price;
            $result[$key]["btnNext"] = $addonItem->next;
            $result[$key]["itemid"] = $addonItem->id;
            $result[$key]["type"] = $addonItem->type;
        }
        $addonItems["addonItems"] = $result;

        return $addonItems;
    }

    public function countUpcomingAddonSteps(int $clientId, int $addonId): int
    {
        $count = 0;

        while ($addon = $this->getAddonDetail($clientId, $addonId)) {
            $count += 1;

            if (empty($addon->next)) {
                break;
            }

            $addonId = $addon->next;
        }

        return $count;
    }

    public function getConfigOrder($clientId)
    {
        $client = new Client($clientId);
        $getConfigOrder = $client->query("CALL getConfigOrder()");
        $client->connectionClose();
        return $getConfigOrder;
    }

    public function getStuartConfigOrder($clientId)
    {
        $client = new Client($clientId);
        $getStuartConfigOrder = $client->query("CALL getStuartConfigOrder()");
        $client->connectionClose();
        return $getStuartConfigOrder;
    }

    public function getExcludePostcode($clientId, $postCode)
    {
        $client = new Client($clientId);
        $getExcludePostcode = $client->query("CALL getExcludePostcode('$postCode')");
        $client->connectionClose();
        return $getExcludePostcode;
    }

    public function getStuartExcludePostcode($clientId, $postCode)
    {
        $client = new Client($clientId);
        $getStuartExcludePostcode = $client->query("CALL getStuartExcludePostcode('$postCode')");
        $client->connectionClose();
        return $getStuartExcludePostcode;
    }

    public function getPostcodeList($clientId, $postCode)
    {
        $client = new Client($clientId);
        $getPostcodeList = $client->query("CALL getPostcodeList('$postCode')");
        $client->connectionClose();
        return $getPostcodeList;
    }

    public function getStuartPostcodeList($clientId, $postCode)
    {
        $client = new Client($clientId);
        $getStuartPostcodeList = $client->query("CALL getStuartPostcodeList('$postCode')");
        $client->connectionClose();
        return $getStuartPostcodeList;
    }

    public function getRadius($clientId)
    {
        $client = new Client($clientId);
        $getRadius = $client->queryAll("CALL getRadius()");
        $client->connectionClose();
        return $getRadius;
    }

    public function getStuartRadius($clientId)
    {
        $client = new Client($clientId);
        $getStuartRadius = $client->queryAll("CALL getStuartRadius()");
        $client->connectionClose();
        return $getStuartRadius;
    }

    public function getMin($clientId)
    {
        $client = new Client($clientId);
        $getMin = $client->query("CALL getMin()");
        $client->connectionClose();
        return $getMin;
    }

    public function getServiceCharge()
    {
        return $this->query("CALL getServiceCharge()");
    }

    public function getBaggageCharge()
    {
        return $this->query("CALL getBaggageCharge()");
    }

    public function cashCardStatus()
    {
        return $this->query("CALL cashCardStatus()");
    }

    public function getBank($clientId)
    {
        $client = new Client($clientId);
        $getBank = $client->query("CALL getBank()");
        $client->connectionClose();
        return $getBank;
    }

    public function getPaymentDetail($clientId)
    {
        $client = new Client($clientId);
        $getPaymentDetail = $client->query("CALL getPaymentDetail()");
        $client->connectionClose();
        return $getPaymentDetail;
    }

    public function createPaymentIntent($clientId, $intentDetail, $types, $dateTime, $status, $createdAt)
    {
        $client = new Client($clientId);
        $createPaymentIntent = $client->query("CALL createPaymentIntent('$intentDetail', '$types', '$dateTime', '$status', '$createdAt', @insert_id)");
        $client->connectionClose();
        return $createPaymentIntent;
    }

    public function clientData($clientId)
    {
        return $this->query("call getClientList($clientId)");
    }

    public function getChargesDetail($clientId)
    {
        $getServiceCharge = $this->getServiceCharge();
        $getBaggageCharge = $this->getBaggageCharge();
        $getBank = $this->getBank($clientId);

        if ($getBank->is_enable != 1) {
            if ($getBank->include_clientBank == 1) {
                $data['serviceCharge'] = number_format($getBank->bank_surcharge + $getServiceCharge->charge, 2);
                $data['clientServiceCharge'] = number_format($getBank->bank_surcharge, 2);
                $data['fkServiceCharge'] = number_format($getServiceCharge->charge, 2);
            } else {
                $data['serviceCharge'] = number_format($getServiceCharge->charge + 0, 2);
                $data['clientServiceCharge'] = "0.00";
                $data['fkServiceCharge'] = number_format($getServiceCharge->charge, 2);
            }
            if ($getBank->domainStatus == 1) {
                $data['clientFkCharge'] = number_format($getServiceCharge->charge, 2);
                $data['FkCharge'] = number_format($getBank->fusion_bank, 2);
                $data['ClientCharge'] = number_format($getBank->bank_surcharge, 2);
                $data['is_FkCharge'] = 1;
            } else {
                $data['clientFkCharge'] = number_format($getBank->bank_surcharge + $getBank->fusion_bank, 2);
                $data['FkCharge'] = number_format($getBank->fusion_bank, 2);
                $data['ClientCharge'] = number_format($getBank->bank_surcharge, 2);
                $data['is_FkCharge'] = 2;
            }
        } else {
            $data['serviceCharge'] = "0.00";
            $data['clientServiceCharge'] = "0.00";
            $data['fkServiceCharge'] = "0.00";
            $data['clientFkCharge'] = '0.00';
            $data['FkCharge'] = '0.00';
            $data['ClientCharge'] = '0.00';
            $data['is_FkCharge'] = 2;
        }

        if ($getBank->is_bag_enable != 1) {
            $data['baggageCharge'] = number_format($getBank->bag_surcharge + $getBaggageCharge->charge, 2);
            $data['clientBaggageCharge'] = number_format($getBank->bag_surcharge, 2);
            $data['fkBaggageCharge'] = number_format($getBaggageCharge->charge, 2);
        } else {
            $data['baggageCharge'] = "0.00";
            $data['clientBaggageCharge'] = "0.00";
            $data['fkBaggageCharge'] = "0.00";
        }
        return $data;
    }

    public function placeOrderResponse($clientPath, $data)
    {
        $payment = new Payment();

        # get client id using path and domain name
        $getClientId = $this->getClientUsingB2cPath($clientPath);
        if ($getClientId == false) {
            $data = array(
                "status" => false,
                "response" => "Invalid path"
            );
            return $data;
        }
        $domain = $this->getClientDatabase($getClientId->id);
        $clientId = $getClientId->id;
        $takeawayName = $getClientId->takeaway_name;
        $data['orderDetail']['takeawayName'] = $takeawayName;
        $data['orderDetail']['domain'] = $domain->domain;

        $orderCharges = $this->getChargesDetail($clientId);
        $stripeCharges = $this->getPaymentDetail($clientId);

        $data['chargesDetail']['clientServiceCharge'] = $orderCharges['clientServiceCharge'];
        $data['chargesDetail']['fkServiceCharge'] = $orderCharges['fkServiceCharge'];
        $data['chargesDetail']['clientBaggageCharge'] = $orderCharges['clientBaggageCharge'];
        $data['chargesDetail']['fkBaggageCharge'] = $orderCharges['fkBaggageCharge'];
        $data['chargesDetail']['baggageCharge'] = $orderCharges['baggageCharge'];
        $data['chargesDetail']['serviceCharge'] = $orderCharges['serviceCharge'];

        $subTotal = floatval($data['chargesDetail']['subTotal']);
        $subTotal = 0;
        $preOrder = isset($data['orderDetail']['preOrder']) ? strtotime($data['orderDetail']['preOrder']) : "0";
        
        if ($preOrder == "0" || $preOrder == "") {
            $orderTime = time();
        } else {
            $orderTime = $preOrder;
        }
        $zeroBasedDay = date('w', $orderTime);
        
        $loyaltyItems = array();
        $orderType = $data['orderDetail']['orderType'];
        foreach ($data['itemsDetail'] as $itemKey => $item) {
            $loyalty = isset($item['loyalty']) ? $item['loyalty'] : "0";
            $getItemDetail = $this->getItemDetail($clientId, $item['id'], $orderType, $zeroBasedDay);
            if ($getItemDetail != false) {
                if ($loyalty == "0") {
                    # New menu price functionalities
                    if ($orderType == 0) {
                        $itemPrice = $getItemDetail['deliveryPrice'];
                    } else if ($orderType == 1) {
                        $itemPrice = $getItemDetail['collectionPrice'];
                    } else {
                        $itemPrice = $getItemDetail['itemPrice'];
                    }
                    $increaseType = $getItemDetail['fkPriceType'];
                    $percentageType = $getItemDetail['fkPercentageType'];
                    $extraPrice = $getItemDetail['fkPrice'];
                    $priceEnable = $getItemDetail['fkPriceStatus'];
                    $priceMode = $getItemDetail['fkPriceMode'];
                    $convertedPrice = $this->getItemPriceValue($itemPrice, $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, $orderType);
                    $item['price'] = $convertedPrice;
                } else {
                    # New loyalty price
                    $item['price'] = "0.00";
                    $loyaltyItems[$item['id']] = str_replace(["<br>", "Â"], "", $getItemDetail['itemName']);
                }
            }
            $itemTotal = $item['price'] * $item['qty'];
            $subTotal += $itemTotal;

            if (!empty($item['addon']) && is_array($item['addon'])) {
                $addonCount = count($item['addon']);
                $i = 0;
                foreach ($item['addon'] as $addonKey => $addon) {
                    $getAddonDetail = $this->getAddonItemDetailWithId($clientId, $addon['id']);
                    if ($getAddonDetail != false) {
                        
                        $addonTotal = $getAddonDetail['price'] * $addon['qty'];
                        $subTotal += $addonTotal;
                        
                    }
                    $i++;
                }
            }
        }

        $data['chargesDetail']['subTotal'] = floatval($subTotal);


        $deliveryCharge = floatval($data['chargesDetail']['deliveryCharge']);
        $clientStuartCharge = floatval($data['chargesDetail']['clientStuartCharge']);
        $totalDeliveryCharge = floatval($deliveryCharge + $clientStuartCharge);
        $serviceCharge = floatval($orderCharges['serviceCharge']);
        $fkServiceCharge = floatval($orderCharges['fkServiceCharge']);
        $clientServiceCharge = floatval($orderCharges['clientServiceCharge']);
        $fkBaggageCharge = floatval($orderCharges['fkBaggageCharge']);
        $baggageCharge = floatval($orderCharges['baggageCharge']);
        $roundAmount = floatval($data['chargesDetail']['roundAmount']);
        $extraDonate = floatval($data['chargesDetail']['extraDonate']);
        $driverTip = floatval($data['chargesDetail']['driverTip']);
        $discount = floatval($data['chargesDetail']['discount']);
        $promo = floatval($data['chargesDetail']['promo']);
        $voucher = floatval($data['chargesDetail']['voucher']);
        $stripeCharge = floatval($stripeCharges->stripecharge);
        $stripeAdd = floatval($stripeCharges->stripeadd);

        $klarnaCharge = floatval($stripeCharges->klarnaStripeCharge);
        $klarnaAdd = floatval($stripeCharges->klarnaStripeAdd);

        $paypalStripeAdd = floatval($stripeCharges->paypalStripeAdd);
        $paypalStripeCharge = floatval($stripeCharges->paypalStripeCharge);

        $securityKey = $stripeCharges->securitykey;
        $connectId = $stripeCharges->connectid;
        $provider = $stripeCharges->provider;
        $stripeMode = $stripeCharges->live;

        $getStuartStatus = $this->getStuartStatus($getClientId->id);

        $SessionalOfferStatus = isset($data['chargesDetail']['sessionalOffer']) ? $data['chargesDetail']['sessionalOffer'] : "";
        $SessionalOfferTotal = isset($data['chargesDetail']['sessionalOffer']) ? $data['chargesDetail']['sessionalOfferTotal'] : "";
        $platform = isset($data['chargesDetail']['platform']) ? 1 : 0;

        if ($platform == 1 && $orderCharges['is_FkCharge'] == 2 && $orderCharges['is_FkCharge'] != 1) {
            $data['chargesDetail']['fkServiceCharge'] = floatval($orderCharges['FkCharge']);
            $serviceCharge = floatval($orderCharges['FkCharge']);
            $clientServiceCharge = floatval($orderCharges['ClientCharge']);
            $data['chargesDetail']['clientServiceCharge'] = floatval($orderCharges['ClientCharge']);
            $data['is_clientWebsite'] = 1;
        } else {
            $data['is_clientWebsite'] = 0;
        }

        # stripe calculations
        if ($SessionalOfferStatus == 1) {
            $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip) - ($SessionalOfferTotal)), 2);
        } else {
            $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip) - ($discount + $promo + $voucher)), 2);
        }

        if ($platform == 1 && $orderCharges['is_FkCharge'] == 2 && $orderCharges['is_FkCharge'] != 1) {
            if ($SessionalOfferStatus == 1) {
                $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip + $clientServiceCharge) - ($SessionalOfferTotal)), 2);
            } else {
                $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip + $clientServiceCharge) - ($discount + $promo + $voucher)), 2);
            }
        }

        if ($getStuartStatus && $getStuartStatus->status == "1") {
            $stripeCalculation = number_format($totalDeliveryCharge + $serviceCharge + $roundAmount + $extraDonate + $driverTip, 2);
        } else {
            $stripeCalculation = number_format($serviceCharge + $roundAmount + $extraDonate + $driverTip, 2);
        }

        $additionalPremiumCharges = 0;

        if ($data['orderDetail']['paymentType'] == 10) {
            $transferCalculation = number_format((($totalAmount - $stripeCalculation) * $klarnaCharge / 100 + $klarnaAdd + $stripeCalculation), 2);
        } else if ($data['orderDetail']['paymentType'] == 14) {
            $transferCalculation = number_format((($totalAmount - $stripeCalculation) * $paypalStripeCharge / 100 + $paypalStripeAdd + $stripeCalculation), 2);
        } else {
            $transferCalculation = number_format((($totalAmount - $stripeCalculation) * $stripeCharge / 100 + $stripeAdd + $stripeCalculation), 2);

            if(isset($stripeCharges->premiumstripecharge)) {
                $additionalStripeCharge = floatval($stripeCharges->premiumstripecharge);

                $additionalPremiumCharges = number_format((($totalAmount - $stripeCalculation) * $additionalStripeCharge / 100), 2);
            }
        }

        $transferCalculationString = str_replace(".", "", $transferCalculation);
        $totalAmountString = str_replace(".", "", $totalAmount);
        $clientTransferAmountString = $totalAmountString - $transferCalculationString;
        $clientTransferAmount = number_format($clientTransferAmountString / 100, 2);

        if ($SessionalOfferStatus == 1) {
            $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip) - ($discount + $promo + $voucher)), 2);
            $totalAmountString = str_replace(".", "", $totalAmount);
        }

        $data['chargesDetail']['totalAmount'] = $totalAmount;
        $data['chargesDetail']['clientTransferAmount'] = $clientTransferAmount;

        if ($data['userDetail']['email'])
            $description = $data['userDetail']['firstName'] . " " . $data['userDetail']['lastName'] . " " . $data['userDetail']['email'];
        else
            $description = $data['userDetail']['firstName'] . " " . $data['userDetail']['lastName'];

        $customerId = $data['userDetail']['customerId'];

        # wallet
        $tableOrder = isset($data['orderDetail']['isTableOrder']) ? $data['orderDetail']['isTableOrder'] : '';
        if(isset($data['orderDetail']['useWalletBalance']) && $data['orderDetail']['useWalletBalance'] && $data['orderDetail']['useWalletBalance'] !== "0" && empty($tableOrder)) {
            $balance = (float) $payment->getWallet($data['userDetail']['customerId']);
            $totalAmount = (float) $totalAmount;
            if ($balance > 0) {
                if ($balance >= $totalAmount) {                    
                    $createTransfer = $this->processWalletPayment($payment, $clientId, $customerId, $totalAmount, $clientTransferAmountString, $connectId, $securityKey, $description, $data);

                    if(isset($createTransfer['status']) && $createTransfer['status'] == false) {
                        return $createTransfer;
                    }

                    $data['orderDetail']['wallet'] = [
                        'has_to_use' => true,
                        'is_used' => true,
                        'amount_used' => $totalAmount,
                        'client_transfer' => $clientTransferAmountString
                    ];
        
                    # order placed functionalities
                    $placeOrder = $this->placeOrder($clientId, $data);
                    $data = array(
                        "status" => true,
                        "type" => "orderPlaced",
                        "response" => $placeOrder
                    );
                    return $data;
                } else {
                    if ($data['orderDetail']['paymentType'] == "9") {
                        return [
                            'status' => false,
                            'response' => "Insufficiant wallet balance."
                        ];
                    }
                    
                    $percentage = $balance / $totalAmount;
                    $CTA = number_format(($clientTransferAmount) * $percentage, 2);
                    $CTAS = str_replace(".", "", $CTA);

                    $data['orderDetail']['wallet'] = [
                        'has_to_use' => true,
                        'is_used' => false,
                        'amount_used' => $balance,
                        'client_transfer' => $CTAS,
                        'payment' => $payment,
                        'clientId' => $clientId,
                        'customerId' => $customerId,
                        'connectId' => $connectId,
                        'securityKey' => $securityKey,
                        'description' => $description
                    ];
                    
                    $totalAmount = number_format($totalAmount - $balance, 2);
                    $totalAmountString = str_replace(".", "", $totalAmount);
                    $clientTransferAmount = number_format($clientTransferAmount - $CTA, 2);
                    $clientTransferAmountString = str_replace(".", "", $clientTransferAmount);
                }
            }
        }

        // Get currency info
        $location = new Location($clientId);
        $currency = $location->getCurrency();

        if ($data['orderDetail']['paymentType'] == '0') {
            # wallet
            if (isset($data['orderDetail']['wallet']['is_used']) && $data['orderDetail']['wallet']['is_used'] === false) {
                $createTransfer = $this->processWalletPayment($data['orderDetail']['wallet']['payment'], $data['orderDetail']['wallet']['clientId'], $data['orderDetail']['wallet']['customerId'], $data['orderDetail']['wallet']['amount_used'], $data['orderDetail']['wallet']['client_transfer'], $data['orderDetail']['wallet']['connectId'], $data['orderDetail']['wallet']['securityKey'], $data['orderDetail']['wallet']['description'], $data);
    
                if (isset($createTransfer['status']) && $createTransfer['status'] == false) {
                    return $createTransfer;
                }
    
                $data['orderDetail']['wallet']['is_used'] = true;
            }
    
            $placeOrder = $this->placeOrder($clientId, $data);

            $data = array(
                "status" => true,
                "type" => "orderPlaced",
                "response" => $placeOrder
            );
            return $data;
        } else if ($data['orderDetail']['paymentType'] == "9") {
            # wallet payment functionalities
            $description = "Wallet " . $description;
            # create transfer amount stripe api
            $createTransfer = $payment->createTransfer($securityKey, $clientTransferAmountString, $connectId, $description, strtolower($currency['code']));
            if (!isset($createTransfer['id'])) {
                $data = array(
                    "status" => false,
                    "response" => $createTransfer
                );
                return $data;
            }
            # create customer account with account
            $createStripeCustomer = $payment->createStripeCustomerWithAccount($securityKey, $description, $data['userDetail']['email'], $connectId);
            # update charges
            $chargesUpdate = $payment->chargesUpdate($securityKey, $createTransfer['destination_payment'], $description, $createStripeCustomer->id, $connectId);

            if ($chargesUpdate['status'] == false) {
                $data = array(
                    "status" => false,
                    "response" => $chargesUpdate['data']
                );
                return $data;
            }

            # balanced wallet amount stored to DB
            $getWallet = $payment->getWallet($customerId);
            $balanceAmount = $getWallet - $totalAmount;
            $payment->updateWalletAmount($balanceAmount, $customerId);

            $payment->addTransaction([
                'client_id' => $clientId,
                'customer_id' => $data['userDetail']['customerId'],
                'amount' => $payment->encrypt($totalAmount),
                'type' => 2,
                'status' => 1,
                'datatime' => time(),
            ]);

            $data['orderDetail']['refno'] = json_encode(
                array(
                    'transfer' => $createTransfer->id,
                    'transfer_group' => $createTransfer->transfer_group,
                    '3d_secure' => false
                )
            );

            # order placed functionalities
            $placeOrder = $this->placeOrder($clientId, $data);

            $data = array(
                "status" => true,
                "type" => "orderPlaced",
                "response" => $placeOrder
            );
            return $data;
        } else if ($data['orderDetail']['paymentType'] == "1") {
            # card payment functionality
            if ($provider == "Stripe") {

                # create payment method functions
                if ($data['paymentDetail']['type'] == "paymentMethodId") {
                    if (empty($data['paymentDetail']['paymentMethodId'])) {
                        $data = array(
                            "status" => false,
                            "response" => "Payment method id is missing"
                        );
                        return $data;
                    }
                    # create customer account with account
                    $savecardStatus = $data['paymentDetail']['sva'];
                    if ($savecardStatus == 1) {
                        $createStripeCustomer = $payment->createStripeCustomerWithAccount($securityKey, $description, $data['userDetail']['email'], $connectId);
                    } else {
                        $createStripeCustomer = $payment->createStripeCustomerWithCard($securityKey, $description, $data['userDetail']['email'], $data['paymentDetail']['paymentMethodId']);
                    }

                    try {
                        if (!empty($data['paymentDetail']['paymentMethodId'])) {
                            # Set your Stripe API key
                            Stripe::setApiKey($securityKey);
                            $paymentMethod = \Stripe\PaymentMethod::retrieve($data['paymentDetail']['paymentMethodId']);
                            if ($paymentMethod->card) {
                                $card = $paymentMethod->card;

                                // Other than domestic visa and mastercard, we are charging addition charge
                                if (!(($card['brand'] == 'visa' || $card['brand'] == 'mastercard') && $card['country'] == 'GB')) {
                                    if ($additionalPremiumCharges) {
                                        $clientTransferAmount = number_format($clientTransferAmount - $additionalPremiumCharges, 2);
                                        $clientTransferAmountString = str_replace(".", "", $clientTransferAmount);
                                        $data['chargesDetail']['clientTransferAmount'] = $clientTransferAmount;
                                    }
                                }
                            }
                        }
                    } catch (ApiErrorException $e) {
                        # Handle any errors from the Stripe API
                        $response['status'] = false;
                        $response['response'] = $e->getMessage();
                        return $response;
                    }

                    # create payment intents
                    $paymentIntent = $payment->createPaymentIntentWithCard($securityKey, $connectId, $data['orderDetail']['domain'], $data['orderDetail']['takeawayName'], $totalAmountString, $clientTransferAmountString, $description, $data['paymentDetail'], $createStripeCustomer, strtolower($currency['code']));
                    if ($paymentIntent['status'] == false) {
                        $data = array(
                            "status" => false,
                            "response" => $paymentIntent['data']
                        );
                        return $data;
                    } else {
                        $paymentIntent = $paymentIntent['data'];
                    }

                    if ($paymentIntent['status']) {
                        if ($paymentIntent['status'] == 'requires_action' && $paymentIntent['next_action']['type'] == 'use_stripe_sdk') {
                            $response['id'] = $paymentIntent['id'];
                            $response['clientSecret'] = $paymentIntent['client_secret'];
                            $data = array(
                                "status" => true,
                                "type" => "3dSecure",
                                "response" => $response
                            );
                            return $data;
                        } else if ($paymentIntent['status'] == 'succeeded' && $paymentIntent['charges']['data'][0]['status'] == 'succeeded') {
                            $createPaymentIntent = $this->createCommonPaymentIntent($payment, $data, $clientId, $customerId, $securityKey, $paymentIntent['id'], $description, $connectId, $stripeMode);
                            return $createPaymentIntent;
                        } else {
                            $data = array(
                                "status" => false,
                                "response" => $paymentIntent
                            );
                            return $data;
                        }
                    } else {
                        $data = array(
                            "status" => false,
                            "response" => $paymentIntent
                        );
                        return $data;
                    }
                } else
                    # create payment intent functions for save card
                    if ($data['paymentDetail']['type'] == "saveCardPaymentIntent") {
                        # get savecard detail
                        $getSaveCard = $payment->getSaveCard($customerId);

                        # get payment customer key
                        $key = ($stripeMode == "0") ? $getSaveCard->test_pay_customer_id : $getSaveCard->pay_customer_id;
                        $customerKey = $payment->decrypt($key);

                        # create payment intents
                        $paymentIntent = $payment->createPaymentIntentWithSaveCard($securityKey, $connectId, $data['orderDetail']['domain'], $data['orderDetail']['takeawayName'], $totalAmountString, $clientTransferAmountString, $description, $customerKey, strtolower($currency['code']));
                        if ($paymentIntent['status'] == false) {
                            $data = array(
                                "status" => false,
                                "response" => $paymentIntent['data']
                            );
                            return $data;
                        } else {
                            $paymentIntent = $paymentIntent['data'];
                        }
                        $response['id'] = $paymentIntent['id'];
                        $response['clientSecret'] = $paymentIntent['client_secret'];
                        $data = array(
                            "status" => true,
                            "type" => "3dSecure",
                            "response" => $response
                        );
                        return $data;
                    } else if ($data['paymentDetail']['type'] == "paymentIntentId") {
                        $createPaymentIntent = $this->createCommonPaymentIntent($payment, $data, $clientId, $customerId, $securityKey, $data['paymentDetail']['paymentIntentId'], $description, $connectId, $stripeMode);
                        return $createPaymentIntent;
                    } else {
                        $data = array(
                            "status" => false,
                            "response" => "Invalid types for card payment"
                        );
                        return $data;
                    }
            } else {
                $data = array(
                    "status" => false,
                    "response" => $provider . " provider is not available"
                );
                return $data;
            }
        } else if ($data['orderDetail']['paymentType'] == "7" || $data['orderDetail']['paymentType'] == "8") {

            # create payment method functions
            if ($data['paymentDetail']['type'] == "applePayGooglePayPaymentMethodId") {
                if (empty($data['paymentDetail']['paymentMethodId'])) {
                    $data = array(
                        "status" => false,
                        "response" => "Payment method id is missing"
                    );
                    return $data;
                }

                # apple pay and google pay payment functionality
                $type = array(
                    "7" => "apple_pay",
                    "8" => "google_pay"
                );
                $portal = $type[$data['orderDetail']['paymentType']];

                # create customer account with account
                $createStripeCustomer = $payment->createStripeCustomerWithCard($securityKey, $description, $data['userDetail']['email'], $data['paymentDetail']['paymentMethodId']);

                // additional charge for premium cards
                try {
                    if (!empty($data['paymentDetail']['paymentMethodId'])) {
                        # Set your Stripe API key
                        Stripe::setApiKey($securityKey);
                        $paymentMethod = \Stripe\PaymentMethod::retrieve($data['paymentDetail']['paymentMethodId']);
                        if (isset($paymentMethod->card)) {
                            $card = $paymentMethod->card;

                            // Other than domestic visa and mastercard, we are charging addition charge
                            if (!(($card['brand'] == 'visa' || $card['brand'] == 'mastercard') && $card['country'] == 'GB')) {
                                if ($additionalPremiumCharges) {
                                    $clientTransferAmount = number_format($clientTransferAmount - $additionalPremiumCharges, 2);
                                    $clientTransferAmountString = str_replace(".", "", $clientTransferAmount);
                                    $data['chargesDetail']['clientTransferAmount'] = $clientTransferAmount;
                                }
                            }
                        }
                    }
                } catch (ApiErrorException $e) {
                    # Handle any errors from the Stripe API
                    $response['status'] = false;
                    $response['response'] = $e->getMessage();
                    return $response;
                }

                # create payment intents
                $paymentIntent = $payment->createPaymentIntentWithApGp($securityKey, $connectId, $data['orderDetail']['domain'], $data['orderDetail']['takeawayName'], $totalAmountString, $clientTransferAmountString, $description, $data['paymentDetail'], $portal, $createStripeCustomer, strtolower($currency['code']));
                if ($paymentIntent['status'] == false) {
                    $data = array(
                        "status" => false,
                        "response" => $paymentIntent['data']
                    );
                    return $data;
                } else {
                    $paymentIntent = $paymentIntent['data'];
                }

                if ($paymentIntent['status']) {
                    if ($paymentIntent['status'] == 'requires_action' && $paymentIntent['next_action']['type'] == 'use_stripe_sdk') {
                        $response['id'] = $paymentIntent['id'];
                        $response['clientSecret'] = $paymentIntent['client_secret'];
                        $data = array(
                            "status" => true,
                            "type" => "3dSecure",
                            "response" => $response
                        );
                        return $data;
                    } else if ($paymentIntent['status'] == 'succeeded') {
                        $createPaymentIntent = $this->createCommonPaymentIntent($payment, $data, $clientId, $customerId, $securityKey, $paymentIntent['id'], $description, $connectId, $stripeMode);
                        return $createPaymentIntent;
                    } else {
                        $data = array(
                            "status" => false,
                            "response" => $paymentIntent
                        );
                        return $data;
                    }
                } else {
                    $data = array(
                        "status" => false,
                        "response" => $paymentIntent
                    );
                    return $data;
                }
            }
        } else if ($data['orderDetail']['paymentType'] == 10) {
            if ($data['paymentDetail']['type'] == "klarnaIntent") {
                $klarna = new Klarna();
                $createKlarna = $klarna->createKlarna($securityKey, $totalAmountString, $connectId, $clientTransferAmountString, $description, strtolower($currency['code']));
                $response['id'] = $createKlarna['id'];
                $response['clientSecret'] = $createKlarna['client_secret'];
                $data = array(
                    "status" => true,
                    "type" => "klarnaIntent",
                    "response" => $response
                );
                return $data;
            } else if ($data['paymentDetail']['type'] == "klarnaOrderPlaced") {
                # retrieve payment intent
                $paymentIntent = $payment->retrievePaymentIntent($securityKey, $data['paymentDetail']['paymentIntentId']);

                $data['orderDetail']['refno'] = json_encode(
                    array(
                        'id' => $paymentIntent['id'],
                        'transfer_group' => $paymentIntent['transfer_group'],
                        'type' => $paymentIntent['payment_method_types'],
                        '3d_secure' => true
                    )
                );

                # wallet
                if (isset($data['orderDetail']['wallet']['is_used']) && $data['orderDetail']['wallet']['is_used'] === false) {
                    $createTransfer = $this->processWalletPayment($data['orderDetail']['wallet']['payment'], $data['orderDetail']['wallet']['clientId'], $data['orderDetail']['wallet']['customerId'], $data['orderDetail']['wallet']['amount_used'], $data['orderDetail']['wallet']['client_transfer'], $data['orderDetail']['wallet']['connectId'], $data['orderDetail']['wallet']['securityKey'], $data['orderDetail']['wallet']['description'], $data);

                    if (isset($createTransfer['status']) && $createTransfer['status'] == false) {
                        return $createTransfer;
                    }

                    $data['orderDetail']['wallet']['is_used'] = true;
                }

                # order placed functionalities
                $placeOrder = $this->placeOrder($clientId, $data);

                $data = array(
                    "status" => true,
                    "type" => "orderPlaced",
                    "response" => $placeOrder
                );
                return $data;
            }
        } else if ($data['orderDetail']['paymentType'] == 14) {
            if ($data['paymentDetail']['type'] == "PaypalMethodId") {
                $paypal = new PayPal();
                $createPayPal = $paypal->createPayPalMID($securityKey, $totalAmountString, $connectId, $clientTransferAmountString, strtolower($currency['code']));
                if (!isset($createPayPal['id'])) {
                    $data = array(
                        "status" => false,
                        "response" => $createPayPal
                    );
                    return $data;
                }
                $response['id'] = $createPayPal['id'];
                $response['clientSecret'] = $createPayPal['client_secret'];
                $data = array(
                    "status" => true,
                    "type" => "PayPalIntent",
                    "response" => $response
                );
                return $data;
            } else if ($data['paymentDetail']['type'] == "PaypalIntent") {
                $paymentIntent = $payment->retrievePaymentIntent($securityKey, $data['paymentDetail']['paymentIntentId']);
                $data['orderDetail']['refno'] = json_encode(
                    array(
                        'id' => $paymentIntent['id'],
                        'transfer_group' => $paymentIntent['transfer_group'],
                        'type' => $paymentIntent['payment_method_types'],
                        '3d_secure' => true
                    )
                );

                # wallet
                if (isset($data['orderDetail']['wallet']['is_used']) && $data['orderDetail']['wallet']['is_used'] === false) {
                    $createTransfer = $this->processWalletPayment($data['orderDetail']['wallet']['payment'], $data['orderDetail']['wallet']['clientId'], $data['orderDetail']['wallet']['customerId'], $data['orderDetail']['wallet']['amount_used'], $data['orderDetail']['wallet']['client_transfer'], $data['orderDetail']['wallet']['connectId'], $data['orderDetail']['wallet']['securityKey'], $data['orderDetail']['wallet']['description'], $data);
        
                    if (isset($createTransfer['status']) && $createTransfer['status'] == false) {
                        return $createTransfer;
                    }
        
                    $data['orderDetail']['wallet']['is_used'] = true;
                }
        
                $placeOrder = $this->placeOrder($clientId, $data);
                $data = array(
                    "status" => true,
                    "type" => "orderPlaced",
                    "response" => $placeOrder
                );
                return $data;
            }
        }

        # create payment intent functions for every payment orders without wallet orders
        if ($data['orderDetail']['paymentType'] != "9" && $data['paymentDetail']['type'] == "paymentIntentId") {
            $createPaymentIntent = $this->createCommonPaymentIntent($payment, $data, $clientId, $customerId, $securityKey, $data['paymentDetail']['paymentIntentId'], $description, $connectId, $stripeMode);
            return $createPaymentIntent;
        }
    }

    private function processWalletPayment($payment, $clientId, $customerId, $totalAmount, $clientTransferAmountString, $connectId, $securityKey, $description, $data) {
        # balanced wallet amount stored to DB
        $getWallet = $payment->getWallet($customerId);
        $balanceAmount = $getWallet - $totalAmount;
        $payment->updateWalletAmount($balanceAmount, $customerId);

        $payment->addTransaction([
            'client_id' => $clientId,
            'customer_id' => $data['userDetail']['customerId'],
            'amount' => $payment->encrypt($totalAmount),
            'type' => 2,
            'status' => 1,
            'datatime' => time(),
        ]);

        return [
            "status" => true,
        ];
    }

    public function createCommonPaymentIntent($payment, $data, $clientId, $customerId, $securityKey, $paymentIntentId, $description, $connectId, $stripeMode)
    {
        # retrieve payment intent
        $paymentIntent = $payment->retrievePaymentIntent($securityKey, $paymentIntentId);
        # confirm the payment intent
        if ($paymentIntent['status'] != "succeeded") {
            $paymentIntent = $payment->confirmPaymentIntent($securityKey, $paymentIntentId);
        }

        if (!isset($paymentIntent['latest_charge']) || empty($paymentIntent['latest_charge'])) {
            $data = array(
                "status" => false,
                "response" => "No charge information available for the PaymentIntent",
            );
            return $data;
        }

        // additional charge for premium cards
        try {
            if (isset($paymentIntent['payment_method']) || !empty($paymentIntent['payment_method'])) {
                # Set your Stripe API key
                Stripe::setApiKey($securityKey);
                $paymentMethod = \Stripe\PaymentMethod::retrieve($paymentIntent['payment_method']);
                if (isset($paymentMethod->card)) {
                    $card = $paymentMethod->card;

                    $cardDetails = [];
                    $cardDetails['brand'] = $card['brand'];
                    $cardDetails['country'] = isset($card['country']) ? $card['country'] : '';
                    $cardDetails['display_brand'] = isset($card['display_brand']) ? $card['display_brand'] : '';
                    $cardDetails['exp_month'] = $card['exp_month'];
                    $cardDetails['exp_year'] = $card['exp_year'];
                    $cardDetails['last4'] = $card['last4'];
                    $cardDetails['three_d_secure_usage'] = isset($card['three_d_secure_usage']) ? $card['three_d_secure_usage'] : '';
                    $cardDetails['wallet'] = isset($card['wallet']) ? $card['wallet'] : '';

                    $data['paymentDetail']['cardDetails'] = $cardDetails;
                }
            }
        } catch (ApiErrorException $e) {
            // ignore errors
        }

        # save card data stored to table
        if ($data['orderDetail']['paymentType'] == "1") {
            $createCardPaymentIntent = $this->createCardPaymentIntent($payment, $data, $customerId, $securityKey, $paymentIntent, $stripeMode, $description);
            if (isset($createCardPaymentIntent)) {
                $data = array(
                    "status" => false,
                    "response" => $createCardPaymentIntent['response']
                );
                return $data;
            }
        }
        # apple pay and google pay data stored to table
        if ($data['orderDetail']['paymentType'] == "7" || $data['orderDetail']['paymentType'] == "8") {

            $type = array(
                "7" => "apple_pay",
                "8" => "google_pay"
            );
            $portal = $type[$data['orderDetail']['paymentType']];

            $datetime = time();
            $createdAt = date('Y-m-d');
            $status = 0;
            $type = "Apple_Pay/Google_Pay";

            $pi = json_encode(
                array(
                    'id' => $paymentIntent['id'],
                    'transfer_group' => $paymentIntent['transfer_group'],
                    'status' => $paymentIntent['status'],
                    '3d_secure' => true,
                    'type' => $portal
                )
            );

            # create payment intent table data
            $this->createPaymentIntent($clientId, $pi, $type, $datetime, $status, $createdAt);

            # apple pay and google pay refno data
            $data['orderDetail']['refno'] = json_encode(
                array(
                    'id' => $paymentIntent['id'],
                    'transfer_group' => $paymentIntent['transfer_group'],
                    'status' => $paymentIntent['status'],
                    '3d_secure' => true,
                    'type' => $portal,
                    'datetime' => $datetime,
                    'created_at' => $createdAt
                )
            );

            // $this->createApGpPaymentIntent($data, $clientId, $paymentIntent);
        }
        
        $chargesRetrieve = $payment->chargesRetrieve($securityKey, $paymentIntent['latest_charge']);

        if ($chargesRetrieve['status'] == false) {
            $data = array(
                "status" => false,
                "response" => $chargesRetrieve
            );
            return $data;
        }

        if(isset($chargesRetrieve->transfer) && !empty($chargesRetrieve->transfer)) {
            $transfer = $payment->transferRetrieve($securityKey, $chargesRetrieve->transfer);
            
            if(isset($transfer->destination) && isset($transfer->destination_payment)) {
                # create customer account with account
                $createStripeCustomer = $payment->createStripeCustomerWithAccount($securityKey, $description, $data['userDetail']['email'], $connectId);
                # update charges
                $charge = $payment->chargesUpdate($securityKey, $transfer->destination_payment, $description, $createStripeCustomer->id, $connectId);

                if ($charge['status'] == false) {
                    $data = array(
                        "status" => false,
                        "response" => $charge['data']
                    );
                    return $data;
                }
            }
        }

        # card and save card refno data
        if ($data['orderDetail']['paymentType'] != "7" && $data['orderDetail']['paymentType'] != "8") {
            $data['orderDetail']['refno'] = json_encode(
                array(
                    'id' => $paymentIntent['id'],
                    'transfer' => $chargesRetrieve['transfer'],
                    'transfer_group' => $chargesRetrieve['transfer_group'],
                    '3d_secure' => true
                )
            );
        }

        # wallet
        if (isset($data['orderDetail']['wallet']['is_used']) && $data['orderDetail']['wallet']['is_used'] === false) {
            $createTransfer = $this->processWalletPayment($data['orderDetail']['wallet']['payment'], $data['orderDetail']['wallet']['clientId'], $data['orderDetail']['wallet']['customerId'], $data['orderDetail']['wallet']['amount_used'], $data['orderDetail']['wallet']['client_transfer'], $data['orderDetail']['wallet']['connectId'], $data['orderDetail']['wallet']['securityKey'], $data['orderDetail']['wallet']['description'], $data);

            if (isset($createTransfer['status']) && $createTransfer['status'] == false) {
                return $createTransfer;
            }

            $data['orderDetail']['wallet']['is_used'] = true;
        }

        # order placed functionalities
        $placeOrder = $this->placeOrder($clientId, $data);

        $data = array(
            "status" => true,
            "type" => "orderPlaced",
            "response" => $placeOrder
        );
        return $data;
    }

    public function createCardPaymentIntent($payment, $data, $customerId, $securityKey, $paymentIntent, $stripeMode, $description)
    {
        $savecardStatus = 0;
        if ($data['paymentDetail']['sva'] == "" && $data['paymentDetail']['sva'] != 0) {
            $data = array(
                "status" => false,
                "response" => "sva is missing"
            );
            return $data;
        }
        $savecardStatus = $data['paymentDetail']['sva'];
        if ($savecardStatus == "1") {
            # get savecard detail 
            $getCustomerStripeConnection = $payment->getCustomerStripeConnection($customerId);
            $getSaveCard = $payment->getSaveCard($customerId);

            # Retrieve a payment method by ID
            $retrieveCard = $payment->retrievePaymentMethod($securityKey, $paymentIntent['payment_method']);
            $saveCardFingerPrint = $retrieveCard['card']['fingerprint'];

            if (!empty($getSaveCard)) {
                $key = ($stripeMode == "0") ? $getSaveCard->test_pay_customer_id : $getSaveCard->pay_customer_id;
                $keys = $payment->decrypt($key);
                # Call the Stripe API to get a list of payment methods
                $customerCard = $payment->stripePaymentMethod($securityKey, $keys);

                $saveAllCardDetail = [];
                foreach ($customerCard as $cardDetail) {
                    $saveAllCardDetail[] = $cardDetail['card']['fingerprint']; # array of unique card fingerprint id
                }

                # check duplicate card entry
                if (!in_array($saveCardFingerPrint, $saveAllCardDetail)) {
                    $attachedCardDetail = $payment->attachStripeCustomer($retrieveCard, $keys);
                    if (!isset($attachedCardDetail['id'])) {
                        $data = array(
                            "status" => false,
                            "response" => $attachedCardDetail
                        );
                        return $data;
                    }
                }
            } else {
                # create customer from stripe and encrypt
                $createCustomer = $payment->createStripeCustomer($securityKey, $description, $data['userDetail']['email']);
                $paymentCustomerId = $payment->encrypt($createCustomer['id']);
                $fingerPrint = $retrieveCard['card']['fingerprint'];
                if ($stripeMode == "0") {
                    # test customer payment mode
                    $createSavecard = $payment->createSavecard($customerId, $paymentCustomerId, $fingerPrint, 0);
                } else {
                    # live customer payment mode
                    $createSavecard = $payment->createSavecard($customerId, $paymentCustomerId, $fingerPrint, 1);
                }
                $attachedCardDetail = $payment->attachStripeCustomer($retrieveCard, $createCustomer['id']);
            }
        }
    }

    // public function createApGpPaymentIntent($data, $clientId, $paymentIntent)
    // {
    //     $type = array(
    //         "7" => "apple_pay",
    //         "8" => "google_pay"
    //     );
    //     $portal = $type[$data['orderDetail']['paymentType']];

    //     $datetime = time();
    //     $createdAt = date('Y-m-d');
    //     $status = 0;
    //     $type = "Apple_Pay/Google_Pay";

    //     $pi = json_encode(
    //         array(
    //             'id' => $paymentIntent['id'],
    //             'transfer_group' => $paymentIntent['transfer_group'],
    //             'status' => $paymentIntent['status'],
    //             '3d_secure' => true,
    //             'type' => $portal
    //         )
    //     );

    //     # create payment intent table data
    //     $this->createPaymentIntent($clientId, $pi, $type, $datetime, $status, $createdAt);

    //     # apple pay and google pay refno data
    //     $data['orderDetail']['refno'] = json_encode(
    //         array(
    //             'id' => $paymentIntent['id'],
    //             'transfer_group' => $paymentIntent['transfer_group'],
    //             'status' => $paymentIntent['status'],
    //             '3d_secure' => true,
    //             'type' => $portal,
    //             'datetime' => $datetime, 
    //             'status' => 0, 
    //             'created_at' => $createdAt
    //         )
    //     );
    // }

    public function placeOrder($clientId, $data,$incomplete=false)
    {
        $client = new Client($clientId);
        $myaccount = new Myaccount();
        $user = new User();
        $pagesController = new PagesController();
        $getTodayOrderCount = $this->getTodayOrderCount($clientId);
        $getStuartStatus = $this->getStuartStatus($clientId);
        $mailData = $data;

        $customerId = $data['userDetail']['customerId'];
        $sno = $getTodayOrderCount;
        $deliveryCharge = isset($data['chargesDetail']['deliveryCharge']) ? $data['chargesDetail']['deliveryCharge'] : "0.00";
        $subTotal = isset($data['chargesDetail']['subTotal']) ? $data['chargesDetail']['subTotal'] : "0.00";
        $totalAmount = isset($data['chargesDetail']['totalAmount']) ? $data['chargesDetail']['totalAmount'] : "0.00";
        $status = 0;
        $orderType = $data['orderDetail']['orderType'];
        $paymentType = $data['orderDetail']['paymentType'];
        $dateTime = time();
        $instruction = isset($data['orderDetail']['instruction']) ? $data['orderDetail']['instruction'] : "";
        date_default_timezone_set('Europe/London');
        $preOrder = isset($data['orderDetail']['preOrder']) ? strtotime($data['orderDetail']['preOrder']) : "0";
        $hide = 0;
        $online = 2;
        $refno = isset($data['orderDetail']['refno']) ? $data['orderDetail']['refno'] : '';
        $giftName = isset($data['orderDetail']['gift']) ? $data['orderDetail']['gift'] : "";
        $giftQty = isset($data['orderDetail']['giftQty']) ? $data['orderDetail']['giftQty'] : "";
        $discount = isset($data['chargesDetail']['discount']) ? $data['chargesDetail']['discount'] : "0.00";
        $discountDescription = isset($data['chargesDetail']['discountDescription']) ? $data['chargesDetail']['discountDescription'] : "";
        $promo = isset($data['chargesDetail']['promo']) ? $data['chargesDetail']['promo'] : "0.00";
        $promoDescription = isset($data['chargesDetail']['promoDescription']) ? $data['chargesDetail']['promoDescription'] : "";
        $voucher = isset($data['chargesDetail']['voucher']) && !empty($data['chargesDetail']['voucher']) ? $data['chargesDetail']['voucher'] : "0.00";
        $voucherDescription = isset($data['chargesDetail']['voucherDescription']) ? $data['chargesDetail']['voucherDescription'] : "";
        $roundAmount = $data['chargesDetail']['roundAmount'];
        $clientServiceCharge = $data['chargesDetail']['clientServiceCharge'];
        $clientBaggageCharge = $data['chargesDetail']['clientBaggageCharge'];
        $fkServiceCharge = $data['chargesDetail']['fkServiceCharge'];
        $fkBaggageCharge = $data['chargesDetail']['fkBaggageCharge'];
        $serviceCharge = $data['chargesDetail']['serviceCharge'];
        $baggageCharge = $data['chargesDetail']['baggageCharge'];
        $driverTip = $data['chargesDetail']['driverTip'];
        $loginType = $data['orderDetail']['loginType'];
        $extraDonate = $data['chargesDetail']['extraDonate'];
        $cutlery = isset($data['orderDetail']['cutlery']) ? $data['orderDetail']['cutlery'] : "";
        $SessionalOfferStatus = isset($data['chargesDetail']['sessionalOffer']) ? $data['chargesDetail']['sessionalOffer'] : "";
        $isCollected = 1;
        if ($getStuartStatus && $getStuartStatus->status == "1") {
            $isStuartEnable = $getStuartStatus->status;
        } else {
            $isStuartEnable = "0";
        }
        $isOrderMoved = 0;
        $clientStripeAmount = $data['chargesDetail']['clientTransferAmount'];
        $stuartClientCharge = $data['chargesDetail']['clientStuartCharge'];


        if ($data['is_clientWebsite'] == 1) {
            $orderTotalAmount = ($subTotal +  $baggageCharge + $roundAmount + $deliveryCharge + $clientServiceCharge) - ($discount + $promo + $voucher);
            $online = 0;
        } else {
            $orderTotalAmount = ($subTotal +  $baggageCharge + $roundAmount + $deliveryCharge) - ($discount + $promo + $voucher);
        }

        if ($loginType == "5") {
            $orderCustomerId = "0";
        } else {
            $orderCustomerId = $customerId;
            # create voucher data
            $this->createVoucherList($clientId, $customerId, $voucherDescription);
        }

        $firstName = $data['userDetail']['firstName'];
        $lastName = $data['userDetail']['lastName'];
        $phone = $data['userDetail']['phone'];
        $email = $data['userDetail']['email'];
        if ($data['orderDetail']['orderType'] == 0) {
            $doorNo = $data['userDetail']['doorNo'];
            $streetName = $data['userDetail']['streetName'];
            $town = $data['userDetail']['town'];
            $postcode = $data['userDetail']['postcode'];
        }
        if ($data['orderDetail']['orderType'] == 1) {
            $doorNo = "";
            $streetName = "";
            $town = "";
            $postcode = "";
        }
        if ($preOrder == "0" || $preOrder == "") {
            $orderTime = $dateTime;
        } else {
            $orderTime = $preOrder;
        }
        $zeroBasedDay = date('w', $orderTime);

        $userAgent = $data['userDetail']['userAgent'];
        $ip = $data['userDetail']['ip'];
        $server = json_encode($_SERVER);


        $tableOrder = isset($data['orderDetail']['isTableOrder']) ? $data['orderDetail']['isTableOrder'] : '';
        $tableName = isset($data['orderDetail']['tableName']) ? $data['orderDetail']['tableName'] : '';

        if (!empty($tableOrder) && isset($tableOrder)) {
            $dateTime = time();
            $preOrder = 0;
            $order = $client->query("CALL createTableOrder('$orderCustomerId', '$sno', '$deliveryCharge', '$subTotal', '$orderTotalAmount', '$status', '$orderType', '$paymentType', '$dateTime', '" . addslashes($instruction) . "', '$preOrder', '$hide', '$clientServiceCharge', '$baggageCharge', '$online', '$refno', '$giftName', '$giftQty', '$discount', '" . addslashes($discountDescription) . "', '$promo', '" . addslashes($promoDescription) . "', '$voucher', '" . addslashes($voucherDescription) . "', '$fkServiceCharge', '$roundAmount', '$fkBaggageCharge', '$driverTip', '$loginType', '$extraDonate', '$cutlery', '$isCollected', '$isStuartEnable', '$isOrderMoved', '$clientStripeAmount', '$stuartClientCharge','$tableName','$tableOrder',@insert_id)");
        } else {
            $order = $client->query("CALL createOrder('$orderCustomerId', '$sno', '$deliveryCharge', '$subTotal', '$orderTotalAmount', '$status', '$orderType', '$paymentType', '$dateTime', '" . addslashes($instruction) . "', '$preOrder', '$hide', '$clientServiceCharge', '$baggageCharge', '$online', '$refno', '$giftName', '$giftQty', '$discount', '" . addslashes($discountDescription) . "', '$promo', '" . addslashes($promoDescription) . "', '$voucher', '" . addslashes($voucherDescription) . "', '$fkServiceCharge', '$roundAmount', '$fkBaggageCharge', '$driverTip', '$loginType', '$extraDonate', '$cutlery', '$isCollected', '$isStuartEnable', '$isOrderMoved', '$clientStripeAmount', '$stuartClientCharge', @insert_id)");

            # wallet
            if(isset($order->insert_id) && isset($data['orderDetail']['useWalletBalance']) && $data['orderDetail']['useWalletBalance'] && $data['orderDetail']['useWalletBalance'] !== "0" && isset($data['orderDetail']['wallet']['has_to_use']) && isset($data['orderDetail']['wallet']['is_used']) && $data['orderDetail']['wallet']['is_used'] === true) {
                $client->query("UPDATE `tbl_order` SET `useWalletBalance`=1,`wallet_amt_used`='". $data['orderDetail']['wallet']['amount_used'] ."' WHERE `id`=$order->insert_id");
            }
        }

        $orderId = $order->insert_id;

        $orderUser = $client->query("CALL createOrderUser('$orderId', '" . addslashes($firstName) . "', '" . addslashes($lastName) . "', '$phone', '" . addslashes($doorNo) . "', '" . addslashes($streetName) . "', '" . addslashes($town) . "', '$email', '$postcode', @insert_id)");

        $loyaltyItems = array();
        foreach ($data['itemsDetail'] as $itemKey => $item) {            
            $loyalty = isset($item['loyalty']) ? $item['loyalty'] : "0";

            $getItemDetail = $this->getItemDetail($clientId, $item['id'], $orderType, $zeroBasedDay);
            if ($getItemDetail != false) {
                if ($loyalty == "0") {
                    # New menu price functionalities
                    if ($orderType == 0) {
                        $itemPrice = $getItemDetail['deliveryPrice'];
                    } else if ($orderType == 1) {
                        $itemPrice = $getItemDetail['collectionPrice'];
                    } else {
                        $itemPrice = $getItemDetail['itemPrice'];
                    }
                    $increaseType = $getItemDetail['fkPriceType'];
                    $percentageType = $getItemDetail['fkPercentageType'];
                    $extraPrice = $getItemDetail['fkPrice'];
                    $priceEnable = $getItemDetail['fkPriceStatus'];
                    $priceMode = $getItemDetail['fkPriceMode'];
                    $convertedPrice = $this->getItemPriceValue($itemPrice, $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, $orderType);
                    $item['price'] = $convertedPrice;
                } else {
                    # New loyalty price
                    $item['price'] = "0.00";
                    $loyaltyItems[$item['id']] = str_replace(["<br>", "Â"], "", $getItemDetail['itemName']);
                }

                $itemFree = 0;
                $itemOffer = "";
                $itemTotal = $item['price'] * $item['qty'];
                if ($getItemDetail['itemName'] == 1) {
                    $itemDiscount = 1;
                } else {
                    $itemDiscount = 0;
                }

                $mailData['itemsDetail'][$itemKey]['itemName'] = str_replace(["<br>", "Â"], "", $getItemDetail['itemName']);
                $mailData['itemsDetail'][$itemKey]['itemTotal'] = $itemTotal;
                $mailData['itemsDetail'][$itemKey]['addonName'] = "";

                # create order item tables
                $createOrderItem = $client->query("CALL createOrderItem('$orderId', '$item[id]', '" . addslashes($getItemDetail['itemName']) . "', '" . addslashes($getItemDetail['subCategoryName']) . "', '$item[qty]', '$itemFree', '$itemOffer', '" . addslashes($item['instruction']) . "', '$item[price]', '$itemDiscount', '$itemTotal', '" . addslashes($getItemDetail['itemAnotherName']) . "', '" . addslashes($getItemDetail['itemType']) . "', @insert_id)");
                if (!empty($item['addon']) && is_array($item['addon'])) {
                    $addonCount = count($item['addon']);
                    $i = 0;
                    foreach ($item['addon'] as $addonKey => $addon) {
                        $getAddonDetail = $this->getAddonItemDetailWithId($clientId, $addon['id']);
                        if ($getAddonDetail != false) {
                            $addonName = $getAddonDetail['name'];
                            if (!empty($addon['extra'])) {
                                $values = array("No", "Less", "Half", "On", "With", "OnBurger", "OnChips");
                                $addonName = $values[$addon['extra']] . ' ' . $getAddonDetail['name'];
                            }
                            $addonTotal = $getAddonDetail['price'] * $addon['qty'];
                            $addonType = "addon";
                            $mailData['itemsDetail'][$itemKey]['itemTotal'] += $addonTotal;
                            $createOrderAddon = $client->query("CALL createOrderAddon('$createOrderItem->insert_id', '$getAddonDetail[id]', '" . addslashes($addonName) . "', '$addon[qty]', '$getAddonDetail[price]', '$addonTotal', '$addonType', '" . addslashes($getAddonDetail['aname']) . "', @insert_id)");
                        }
                        if ($i == $addonCount - 1) {
                            $mailData['itemsDetail'][$itemKey]['addonName'] .= $addonName . '.';
                        } else {
                            $mailData['itemsDetail'][$itemKey]['addonName'] .= $addonName . ',';
                        }
                        $i++;
                    }
                }

            }
        }

        # create loyalty points
        if (!empty($loyaltyItems) && $orderCustomerId != "0") {
            $point = "1";
            $this->createLoyaltyWithOrder($orderCustomerId, $clientId, $totalAmount, $loyaltyItems, $point);
        }

        $getOfferList = $this->query("CALL getOfferBanner($clientId)");
        if (!empty($getOfferList)) {
            if ($getOfferList->loyalty != 0) {
                $point = "2";
                $this->createLoyaltyWithOrder($orderCustomerId, $clientId, $totalAmount, $orderId, $point);
            }
        }

        # create order device info, order transaction, order history
        $createOrderDeviceInfo = $client->query("CALL createOrderDeviceInfo('$orderId', '$userAgent', '$ip', '$server', @insert_id)");
        if ($paymentType == "1") {
            if(!$incomplete)
            {
                $reference = json_decode($refno);         
                $token = $reference->id;
                $this->createPaymentInfo($customerId, $orderId, $token, $totalAmount, $refno);
            }

        }
        if ($paymentType == "9") {
            $type = "2";
            $status = "2";
            $createTransaction = $this->createTransaction($clientId, $customerId, $orderId, $totalAmount, $type, $status, $dateTime);
        }
        $createOrderHistory = $this->createOrderHistory($customerId, $data['clientPath'], $orderId, $dateTime, $loginType);

        if ($SessionalOfferStatus == 1) {
            $SessionalOfferInsert = $this->createSessionalOfferInsert($customerId, $SessionalOfferStatus);
        }

        $getOrderDetail = $client->query("call getOrderDetail($orderId)");
        
        // Get currency info
        $location = new Location($clientId);
        $currency = $location->getCurrency();

        $expectedTime = $myaccount->tatTime($getOrderDetail, $clientId);
        $contact = $this->getContacts($clientId);
        $Userid = $data['userDetail']['customerId'];
        $Userdetails = $this->query("SELECT * FROM tbl_customer_address WHERE customer_id=$Userid");
        $mailData['Address']['door_number'] = $Userdetails->door_number;
        $mailData['Address']['address1'] = $Userdetails->address1;
        $mailData['Address']['address2'] = $Userdetails->address2;
        $paymentTypeArray = array("0" => "Cash", "1" => "Card", "7" => "Apple Pay", "8" => "Google Pay", "9" => "Wallet", "10" => "Klarna", "14" => "Paypal");
        $mailData['orderDetail']['paymentType'] = $paymentTypeArray[$data['orderDetail']['paymentType']];
        $mailData['orderDetail']['dateTime'] = date("M d,Y h:i A ", $dateTime);
        $mailData['orderDetail']['estimated'] = $expectedTime['estimated'];
        $myAccount = new Myaccount();
        $mailData['data'] = $myAccount->OrderacceptDetail($clientId, $orderId);
        $mailData['orderDetail']['id']= $orderId;
        $mailData['orderDetail']['phone'] = $contact->phone;
        $mailData['chargesDetail']['totalServiceCharge'] = number_format($data['chargesDetail']['clientServiceCharge'] + $data['chargesDetail']['fkServiceCharge'], 2);
        $mailData['chargesDetail']['totalBagCharge'] = number_format($data['chargesDetail']['clientBaggageCharge'] + $data['chargesDetail']['fkBaggageCharge'], 2);
        $mailData['currency'] = $currency['symbol'];

        // adding gift to item list for bill
        if ($getOrderDetail->gift != "") {
            $giftData = array(
                "itemName" => $getOrderDetail->gift,
                "qty" => $getOrderDetail->gqty,
                "itemTotal" => "0.00"
            );
            array_push($mailData['itemsDetail'], $giftData);
        }

        $placeOrderMail = $pagesController->placeOrderMail($mailData);

        $order = $client->query("SELECT o.preorder, o.id AS ord_id, o.otype, o.total, o.datetime, o.online, o.isstuart_enable, u.fname, u.lname, u.phone, u.postcode FROM tbl_order o INNER JOIN tbl_order_user u ON o.id = u.oid WHERE o.online IN ('0', '2', '4', '5') AND o.status = '0' AND o.hide = 0 AND o.id = $orderId");
        $user = $client->query("SELECT * from tbl_order_user where `oid` = $orderId");
        $notify = $client->query("SELECT * from tbl_notify");
        $collection = $client->query("select collection_time,delivery_time from `tbl_collection_time`");

        $result['orderId'] = $orderId;
        $result['clientId'] = $clientId;
        $result['order'] = $order;
        $result['order']->user = $user;
        $result['order']->notify = $notify;
        $result['order']->collection = $collection;

        if (isset($data['paymentDetail']['cardDetails']) && !empty($data['paymentDetail']['cardDetails'])) {
            try {
                $client->query("INSERT INTO `tbl_stripe_card_orders` (`oid`, `card`) VALUES ('$orderId', '" . json_encode($data['paymentDetail']['cardDetails']) . "');");
            } catch (\PDOException $e) {
                //throw $th;
            }
        }
        
        $client->connectionClose();

        $push_notification_type="order_placed";
        $MyAccountController=new MyAccountController();
        $push_notification=$MyAccountController->pushNotificationCurl($push_notification_type,$clientId, $orderId);
        return $result;
    }


    public function encryptOrderId($orderId)
    {
        $payment = new Payment();
        return $payment->encrypt($orderId);
    }

    public function createOrderHistory($customerId, $path, $orderId, $dateTime, $loginType)
    {
        return $this->query("CALL createOrderHistory('$customerId', '$path', '$orderId', '$dateTime', '$loginType', @insert_id)");
    }

    public function createSessionalOfferInsert($customerId, $SessionalOfferStatus)
    {
        return $this->query("CALL SessionalOfferInsert('$customerId','$SessionalOfferStatus')");
    }

    public function createTransaction($clientId, $customerId, $orderId, $amount, $type, $status, $dateTime)
    {
        $payment = new Payment();
        $amount = $payment->encrypt($amount);
        return $this->query("CALL createTransaction('$clientId', '$customerId', '$orderId', '$amount', '$type', '$status', '$dateTime', @insert_id)");
    }

    public function createPaymentInfo($customerId, $orderId, $token, $totalAmount, $refno)
    {
        return $this->query("CALL createPaymentInfo('$customerId', '$orderId', '$token', '$totalAmount', '$refno', @insert_id)");
    }

    public function getTodayOrderCount($clientId)
    {
        $client = new Client($clientId);
        $today = strtotime('8:00 AM');
        $hour = strtotime(date("H:i"));
        if ($today < $hour) {
            $tomorrow = strtotime('+1 day', strtotime('7:59 AM'));
            $today = strtotime('8:00 AM');
        } else {
            $tomorrow = strtotime('7:59 AM');
            $today = strtotime('-1 day', strtotime('8:00 AM'));
        }
        $getTodayOrderCount = $client->query("CALL getTodayOrderCount($today, $tomorrow)");
        $client->connectionClose();
        $count = intval($getTodayOrderCount->count + 1);
        return $count;
    }

    public function createEmailFeedback($data)
    {
        $client = new Client($data['clientId']);

        # get ordered user details
        $getOrderUser = $client->query("call getOrderUser($data[orderId])");
        if ($getOrderUser != false) {
            $name = $getOrderUser->fname . " " . $getOrderUser->lname;
            $email = $getOrderUser->email;

            #update feedback and update order feedback status
            $client->query("CALL createFeedback('$name', '$data[comment]', '$data[rating]', '$email', '$data[agent]', '$data[orderId]', '$data[reason]', @insert_id)");
            $client->query("CALL updateFeedbackInOrder('$data[orderId]')");

            $error['status'] = true;
            $error['data'] = "Feedback added";
            return $error;
        } else {
            $error['status'] = false;
            $error['data'] = "Invalid order id";
            return $error;
        }
        $client->connectionClose();
    }

    public function getPromoDetail($clientId)
    {
        return $this->queryAll("call getPromoDetail($clientId)");
    }

    public function getPromoDetailByOrderMode($clientId, $orderMode)
    {
        return $this->queryAll("call getPromoDetailByOrderMode($clientId, $orderMode)");
    }

    public function getSessionalDetail($clientId)
    {
        return $this->queryAll("call getSessionalDetails($clientId)");
    }

    public function getOnlinSeasonalDiscount($clientId)
    {
        $day = date('w');
        return $this->query("call getOnlineDiscountMax($clientId,$day)");
    }

    public function checkSessionalOfferCustomer($custId)
    {
        if ($custId) {
            return $this->query("call checkSessionalOfferCustomer($custId)");
        } else {
            return false;
        }
    }

    public function getCommonDiscount($zeroBasedDay)
    {
        return $this->queryAll("call getCommonDiscount($zeroBasedDay)");
    }

    public function getCommonDiscountByOrderMode($zeroBasedDay, $orderMode)
    {
        return $this->queryAll("call getCommonDiscountByOrderMode($zeroBasedDay, $orderMode)");
    }

    public function getOnlineDiscount($clientId, $zeroBasedDay)
    {
        return $this->queryAll("call getOnlineDiscount($clientId, $zeroBasedDay)");
    }

    public function getOnlineDiscountByOrderMode($clientId, $zeroBasedDay, $orderMode)
    {
        return $this->queryAll("call getOnlineDiscountByOrderMode($clientId, $zeroBasedDay, $orderMode)");
    }

    public function getAutoDiscount($clientId, $zeroBasedDay, $orderMode)
    {
        $client = new Client($clientId);
        $getAutoDiscount = $client->queryAll("call getAutoDiscount($zeroBasedDay, $orderMode)");
        $client->connectionClose();
        return $getAutoDiscount;
    }

    public function getOfferDetails($clientId, $orderMode, $orderTime, $customerId, $clientPath, bool $only = false)
    {
        $isPromoAvailable = false;
        $myaccount = new Myaccount();
        if ($customerId != "") {
            $getLastOrder = $myaccount->getLastOrder($customerId, $clientId, $clientPath);
            if ($getLastOrder == false) {
                $isPromoAvailable = true;
            }
        }

        $zeroBasedDay = date('w', $orderTime);

        if ($only) {
            $getOnlineDiscount = $this->getOnlineDiscountByOrderMode($clientId, $zeroBasedDay, $orderMode);
            $getAutoDiscount = $this->getAutoDiscount($clientId, $zeroBasedDay, $orderMode);
            $getCommonDiscount = $this->getCommonDiscountByOrderMode($zeroBasedDay, $orderMode);
            $getPromoDetail = $this->getPromoDetailByOrderMode($clientId, $orderMode);
        } else {
            $getPromoDetail = $this->getPromoDetail($clientId);
            $getCommonDiscount = $this->getCommonDiscount($zeroBasedDay);
            $collection = $this->getAutoDiscount($clientId, $zeroBasedDay, 1);
            $delivery = $this->getAutoDiscount($clientId, $zeroBasedDay, 0);
            $getAutoDiscount = array_merge($collection, $delivery);
            $getOnlineDiscount = $this->getOnlineDiscount($clientId, $zeroBasedDay);
        }

        $getOfferList = $this->query("CALL getOfferBanner($clientId)");

        $data = array();

        $checkSessionalOffer = $this->checkSessionalOfferCustomer($customerId);
        $getSessionalOffer = $this->getSessionalDetail($clientId);

        // Get currency info
        $location = new Location($clientId);
        $currency = $location->getCurrency();

        $getOnlinSeasonalDiscount = $this->getOnlinSeasonalDiscount($clientId);
        $orderModeCode =  ($orderMode == 0) ? "Delivery" : "Collection";
        if ($getSessionalOffer != false && $checkSessionalOffer && $getOnlinSeasonalDiscount != false && ($getSessionalOffer[0]->order_type_id == $orderMode || $getSessionalOffer[0]->order_type_id == 2)) {
            if ($getSessionalOffer != false && $checkSessionalOffer->result) {
                if ($customerId != "" && $isPromoAvailable != false) {
                    foreach ($getSessionalOffer as $key => $list) {
                        $data["promo"][$key]["code"] = $list->promo_code;
                        $data["promo"][$key]["description"] = $list->description;

                        $data["promo"][$key]["terms1"] = "Coupon code applicable only on orders above " . $currency['symbol'] . $list->min_amount . ".";
                        $data["promo"][$key]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
                        $data["promo"][$key]["terms3"] = "No maximum limit to apply the coupon code.";
                        $data["promo"][$key]["terms4"] = "";
                        $data["promo"][$key]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

                        $data["promo"][$key]["discount"] = $list->discount;
                        $data["promo"][$key]["discountType"] = ($list->promotion_amount_type == 0) ? "%" : $currency['symbol'];
                        $data["promo"][$key]["orderType"] = $list->order_type_id;
                        $data["promo"][$key]["sessionalOffer"] = 1;
                        $data["promo"][$key]["minOrder"] = $list->min_amount;
                    }
                } else {
                    $data["promo"] = array();
                }

                $data["seasonal_online"][0]["code"] = $getOnlinSeasonalDiscount->discount_code;
                $data["seasonal_online"][0]["description"] = $getOnlinSeasonalDiscount->description;

                $data["seasonal_online"][0]["terms1"] = "Coupon code applicable only on orders above " . $currency['symbol'] . $getOnlinSeasonalDiscount->min_order_value . ".";
                $data["seasonal_online"][0]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
                $data["seasonal_online"][0]["terms3"] = "No maximum limit to apply the coupon code.";
                $data["seasonal_online"][0]["terms4"] = "Offer valid only till " . date('d/m/Y h:i A', $getOnlinSeasonalDiscount->to_validity) . ".";
                $data["seasonal_online"][0]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

                $data["seasonal_online"][0]["discount"] = $getOnlinSeasonalDiscount->discount_amount;
                $data["seasonal_online"][0]["discountType"] = ($getOnlinSeasonalDiscount->discount_amount_type == 0) ? "%" : $currency['symbol'];
                $data["seasonal_online"][0]["orderType"] = $getOnlinSeasonalDiscount->order_type_id;
                $data["seasonal_online"][0]["paymentType"] = $getOnlinSeasonalDiscount->payment_type_id;
                $data["seasonal_online"][0]["minOrder"] = $getOnlinSeasonalDiscount->min_order_value;
                $data["seasonal_online"][0]["from"] = date("d/m/Y h:i A", $getOnlinSeasonalDiscount->from_validity);
                $data["seasonal_online"][0]["to"] = date("d/m/Y h:i A", $getOnlinSeasonalDiscount->to_validity);

                $data["online"] = array();
                $data["common"] = array();
                $data["autoDiscount"] = array();
                return $data;
            }
        } else if ($getSessionalOffer != false && !$checkSessionalOffer && $getOnlinSeasonalDiscount != false && ($getSessionalOffer[0]->order_type_id == $orderMode || $getSessionalOffer[0]->order_type_id == 2)) {
            if ($customerId != "" && $isPromoAvailable != false) {
                foreach ($getSessionalOffer as $key => $list) {
                    $data["promo"][$key]["code"] = $list->promo_code;
                    $data["promo"][$key]["description"] = $list->description;

                    $data["promo"][$key]["terms1"] = "Coupon code applicable only on orders above " . $currency['symbol'] . $list->min_amount . ".";
                    $data["promo"][$key]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
                    $data["promo"][$key]["terms3"] = "No maximum limit to apply the coupon code.";
                    $data["promo"][$key]["terms4"] = "";
                    $data["promo"][$key]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

                    $data["promo"][$key]["discount"] = $list->discount;
                    $data["promo"][$key]["discountType"] = ($list->promotion_amount_type == 0) ? "%" : $currency['symbol'];
                    $data["promo"][$key]["orderType"] = $list->order_type_id;
                    $data["promo"][$key]["sessionalOffer"] = 1;
                    $data["promo"][$key]["minOrder"] = $list->min_amount;
                }
            } else {
                $data["promo"] = array();
            }
            $data["seasonal_online"][0]["code"] = $getOnlinSeasonalDiscount->discount_code;
            $data["seasonal_online"][0]["description"] = $getOnlinSeasonalDiscount->description;

            $data["seasonal_online"][0]["terms1"] = "Coupon code applicable only on orders above " . $currency['symbol'] . $getOnlinSeasonalDiscount->min_order_value . ".";
            $data["seasonal_online"][0]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
            $data["seasonal_online"][0]["terms3"] = "No maximum limit to apply the coupon code.";
            $data["seasonal_online"][0]["terms4"] = "Offer valid only till " . date('d/m/Y h:i A', $getOnlinSeasonalDiscount->to_validity) . ".";
            $data["seasonal_online"][0]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

            $data["seasonal_online"][0]["discount"] = $getOnlinSeasonalDiscount->discount_amount;
            $data["seasonal_online"][0]["discountType"] = ($getOnlinSeasonalDiscount->discount_amount_type == 0) ? "%" : $currency['symbol'];
            $data["seasonal_online"][0]["orderType"] = $getOnlinSeasonalDiscount->order_type_id;
            $data["seasonal_online"][0]["paymentType"] = $getOnlinSeasonalDiscount->payment_type_id;
            $data["seasonal_online"][0]["minOrder"] = $getOnlinSeasonalDiscount->min_order_value;
            $data["seasonal_online"][0]["from"] = date("d/m/Y h:i A", $getOnlinSeasonalDiscount->from_validity);
            $data["seasonal_online"][0]["to"] = date("d/m/Y h:i A", $getOnlinSeasonalDiscount->to_validity);

            $data["online"] = array();
            $data["common"] = array();
            $data["autoDiscount"] = array();
            return $data;
        } else {
            if ($customerId != "" && $isPromoAvailable != false) {
                if ($getPromoDetail != false) {
                    foreach ($getPromoDetail as $key => $list) {
                        $data["promo"][$key]["code"] = $list->promo_code;
                        $data["promo"][$key]["description"] = $list->description;

                        $data["promo"][$key]["terms1"] = "";
                        $data["promo"][$key]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
                        $data["promo"][$key]["terms3"] = "No maximum limit to apply the coupon code.";
                        $data["promo"][$key]["terms4"] = "";
                        $data["promo"][$key]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

                        $data["promo"][$key]["discount"] = $list->discount;
                        $data["promo"][$key]["discountType"] = ($list->promotion_amount_type == 0) ? "%" : $currency['symbol'];
                        $data["promo"][$key]["orderType"] = $list->order_type_id;
                        $data["promo"][$key]["sessionalOffer"] = 0;
                    }
                } else {
                    $data["promo"] = array();
                }
            } else {
                $data["promo"] = array();
            }
            $data["seasonal_online"] = array();

            if ($getOnlineDiscount != false) {
                $onlineKey = 0;
                foreach ($getOnlineDiscount as $key => $list) {
                    if ($list->from_validity <= $orderTime && $orderTime <= $list->to_validity) {
                        $data["online"][$onlineKey]["code"] = $list->discount_code;
                        $data["online"][$onlineKey]["description"] = $list->description;

                        $data["online"][$onlineKey]["terms1"] = "Coupon code applicable only on orders above " . $currency['symbol'] . $list->min_order_value . ".";
                        $data["online"][$onlineKey]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
                        $data["online"][$onlineKey]["terms3"] = "No maximum limit to apply the coupon code.";
                        $data["online"][$onlineKey]["terms4"] = "Offer valid only till " . date('d/m/Y h:i A', $list->to_validity) . ".";
                        $data["online"][$onlineKey]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

                        $data["online"][$onlineKey]["discount"] = $list->discount_amount;
                        $data["online"][$onlineKey]["discountType"] = ($list->discount_amount_type == 0) ? "%" : $currency['symbol'];
                        $data["online"][$onlineKey]["orderType"] = $list->order_type_id;
                        $data["online"][$onlineKey]["paymentType"] = $list->payment_type_id;
                        $data["online"][$onlineKey]["minOrder"] = $list->min_order_value;
                        $data["online"][$onlineKey]["from"] = date("d/m/Y h:i A", $list->from_validity);
                        $data["online"][$onlineKey]["to"] = date("d/m/Y h:i A", $list->to_validity);
                        $onlineKey++;
                    } else {
                        $data["online"] = array();
                    }
                }
                if ($getCommonDiscount != false) {
                    $commonKey = 0;
                    foreach ($getCommonDiscount as $key => $list) {
                        if ($list->from_validity <= $orderTime && $orderTime <= $list->to_validity) {
                            $data["common"][$commonKey]["code"] = str_replace("Â", "", $list->discount_code);
                            if ($list->discount_amount_type == 0) {
                                $data["common"][$commonKey]["description"] = "UPTO " . number_format($list->discount, 0) . "% exclusive offer";
                            } else {
                                $data["common"][$commonKey]["description"] = "UPTO " . $currency['symbol'] . number_format($list->discount, 0) . " exclusive offer";
                            }

                            $data["common"][$commonKey]["terms1"] = "Coupon code applicable only on orders above " . $currency['symbol'] . $list->min_order_value . ".";
                            $data["common"][$commonKey]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
                            $data["common"][$commonKey]["terms3"] = "No maximum limit to apply the coupon code.";
                            $data["common"][$commonKey]["terms4"] = "Offer valid only till " . date('d/m/Y h:i A', $list->to_validity) . ".";
                            $data["common"][$commonKey]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

                            $data["common"][$commonKey]["discount"] = $list->discount;
                            $data["common"][$commonKey]["discountType"] = ($list->discount_amount_type == 0) ? "%" : $currency['symbol'];
                            $data["common"][$commonKey]["orderType"] = $list->order_type_id;
                            $data["common"][$commonKey]["paymentType"] = $list->payment_type_id;
                            $data["common"][$commonKey]["minOrder"] = $list->min_order_value;
                            $data["common"][$commonKey]["from"] = date("d/m/Y h:i A", $list->from_validity);
                            $data["common"][$commonKey]["to"] = date("d/m/Y h:i A", $list->to_validity);
                            $commonKey++;
                        } else {
                            $data["common"] = array();
                        }
                    }
                } else {
                    $data["common"] = array();
                }
            } else {
                $data["online"] = array();
                $data["common"] = array();
            }

            if ($getOfferList && $getOfferList->noCode == 1) {
                if ($getAutoDiscount != false) {
                    $offlineKey = 0;
                    foreach ($getAutoDiscount as $key => $list) {
                        if ($list->discountFROM <= $orderTime && $orderTime <= $list->discountTo) {
                            $data["autoDiscount"][$offlineKey]["code"] = $list->discount_code;
                            $data["autoDiscount"][$offlineKey]["description"] = $list->description;

                            $data["autoDiscount"][$offlineKey]["terms1"] = "Coupon code applicable only on orders above " . $currency['symbol'] . $list->minAmount . ".";
                            $data["autoDiscount"][$offlineKey]["terms2"] = "Offer is valid only for this particular takeaway/restaurant.";
                            $data["autoDiscount"][$offlineKey]["terms3"] = "No maximum limit to apply the coupon code.";
                            $data["autoDiscount"][$offlineKey]["terms4"] = "Offer valid only till " . date('d/m/Y h:i A', $list->discountTo) . ".";
                            $data["autoDiscount"][$offlineKey]["terms5"] = "This coupon code is applicable only for " . $orderModeCode . " orders.";

                            $data["autoDiscount"][$offlineKey]["discount"] = $list->discount;
                            $data["autoDiscount"][$offlineKey]["discountType"] = ($list->dType == "%") ? "%" : $currency['symbol'];
                            if ($list->discount_code == "Online Collection") {
                                $data["autoDiscount"][$offlineKey]["orderType"] = 1;
                            } else if ($list->discount_code == "Online Delivery") {
                                $data["autoDiscount"][$offlineKey]["orderType"] = 0;
                            }
                            $data["autoDiscount"][$offlineKey]["paymentType"] = $list->pType;
                            $data["autoDiscount"][$offlineKey]["minOrder"] = $list->minAmount;
                            $data["autoDiscount"][$offlineKey]["from"] = date("d/m/Y h:i A", $list->discountFROM);
                            $data["autoDiscount"][$offlineKey]["to"] = date("d/m/Y h:i A", $list->discountTo);
                            $offlineKey++;
                        } else {
                            $data["autoDiscount"] = array();
                        }
                    }
                } else {
                    $data["autoDiscount"] = array();
                }
            } else {
                $data["autoDiscount"] = array();
            }

            return $data;
        }
    }

    function createVoucherList($clientId, $customerId, $voucherCode)
    {
        $checkVoucher = $this->checkVoucher($clientId, $customerId);
        if ($checkVoucher['status'] != false) {
            if (!empty($checkVoucher['voucherCode'])) {
                $code = explode(",", $checkVoucher['voucherCode']);
                if (!in_array(strtoupper($voucherCode), array_map('strtoupper', $code))) {
                    return $this->updateVoucher($clientId, $checkVoucher['email'], $voucherCode);
                } else {
                    return "voucher code was already Used";
                }
            } else {
                if ($voucherCode != "") {
                    return $this->createVoucher($clientId, $checkVoucher['email'], $voucherCode);
                }
            }
        }
    }

    function checkVoucher($clientId, $customerId)
    {
        $myaccount = new Myaccount();
        $customerDetail = $myaccount->getProfile($customerId);
        $getOfferList = $this->query("CALL getOfferBanner($clientId)");
        $data['email'] = $customerDetail->email;
        $data['status'] = true;
        if (!empty($getOfferList)) {
            if ($getOfferList->voucher == 1) {
                $voucher = $this->getVoucherCode($clientId, $customerDetail->email);
                $data['voucherCode'] = $voucher !== false ? $voucher->voucher_code : "";
            } else {
                $data['status'] = false;
                $data['voucherCode'] = "";
            }
        } else {
            $data['status'] = false;
            $data['voucherCode'] = "";
        }
        return $data;
    }

    function getVoucherCode($clientId, $email)
    {
        $client = new Client($clientId);
        $getVoucherCode = $client->query("CALL getVoucherCode('$email')");
        $client->connectionClose();
        return $getVoucherCode;
    }

    function createVoucher($clientId, $email, $voucherCode)
    {
        $client = new Client($clientId);
        $createVoucher = $client->query("CALL createVoucher('$email', '$voucherCode', @insert_id)");
        $client->connectionClose();
        return $createVoucher;
    }

    function updateVoucher($clientId, $email, $voucherCode)
    {
        $client = new Client($clientId);
        $updateVoucher = $client->query("CALL updateVoucher('$email', '$voucherCode')");
        $client->connectionClose();
        return true;
    }

    function getVoucherDetail($clientId, $voucherCode, $zeroBasedDay)
    {
        $client = new Client($clientId);
        $getVoucherDetail = $client->query("CALL getVoucherDetail('$voucherCode', $zeroBasedDay)");
        $client->connectionClose();
        return $getVoucherDetail;
    }

    function getOrderDriverStatus($clientId, $orderId)
    {
        $client = new Client($clientId);
        $getOrderDriver = $client->query("CALL getOrderDriver($orderId)");
        $client->connectionClose();
        return $getOrderDriver;
    }

    function checkOrderTime($data)
    {
        $menu = new Menu();
        $restaurantPage = new RestaurantPage();
        $getClientId = $this->getClientUsingB2cPath($data['clientPath']);
        $orderDetailPreOrder = $data['orderDetail']['preOrder'];
        if (!empty($orderDetailPreOrder)) {
            $orderedTime = strtotime(str_replace("\xc2\xa0", " ", ($orderDetailPreOrder)));
        } else {
            $orderedTime = time();
        }

        # Check restaurant open close status functionalities
        $openCloseStatus = $restaurantPage->restaurantOpenCloseStatus($getClientId->id, $orderedTime, $data['orderDetail']['orderType']);
        if ($openCloseStatus['status'] != "0" && strtotime($openCloseStatus['openAt']) < strtotime(date("h:i A", $orderedTime))) {
            return [
                'status' => false,
                'data' => $openCloseStatus
            ];
        }
        return [
            'status' => true,
            'data' => $openCloseStatus
        ];
    }

    public function getItemsNotAddOns($clientId)
    {
        $client = new Client($clientId);
        $numericDayOfWeek = date('N');
        $getItemsNotAddOns = $client->queryall("CALL getItemsList($numericDayOfWeek)");
        $client->connectionClose();
        return $getItemsNotAddOns;
    }

    public function getMenuCollapse($clientId)
    {
        $client = new Client($clientId);
        $getMenuCollapse = $client->query("CALL getMenuCollapse()");
        $client->connectionClose();
        return $getMenuCollapse;
    }
    public function getpromotionbannerimg($clientId){
        $client = new Client($clientId);
        $sql = "SELECT * FROM tbl_fk_promotionbanner";
        $result = $client->query($sql);
        $client->connectionClose();
        return $result;
    }

    function getDriverTip($clientId)
    {
        $client = new Client($clientId);
        $getDriverTip = $client->query("CALL getDriverTip()");
        $client->connectionClose();
        return $getDriverTip;
    }

   

    public function placeOrderResponseInComplete($clientPath, $data,$inCompleteId)
    {
        $payment = new Payment();
        
        # get client id using path and domain name
        $getClientId = $this->getClientUsingB2cPath($clientPath);
     
        if ($getClientId == false) {
            $data = array(
                "status" => false,
                "response" => "Invalid path"
            );
            return $data;
        }
        $domain = $this->getClientDatabase($getClientId->id);
        $clientId = $getClientId->id;
        $takeawayName = $getClientId->takeaway_name;
       
       
        $data['orderDetail']['takeawayName'] = $takeawayName;
        $data['orderDetail']['domain'] = $domain->domain;

        $orderCharges = $this->getChargesDetail($clientId);
        $stripeCharges = $this->getPaymentDetail($clientId);

        $data['chargesDetail']['clientServiceCharge'] = $orderCharges['clientServiceCharge'];
        $data['chargesDetail']['fkServiceCharge'] = $orderCharges['fkServiceCharge'];
        $data['chargesDetail']['clientBaggageCharge'] = $orderCharges['clientBaggageCharge'];
        $data['chargesDetail']['fkBaggageCharge'] = $orderCharges['fkBaggageCharge'];
        $data['chargesDetail']['baggageCharge'] = $orderCharges['baggageCharge'];
        $data['chargesDetail']['serviceCharge'] = $orderCharges['serviceCharge'];

        $subTotal = floatval($data['chargesDetail']['subTotal']);
        $subTotal = 0;
        $preOrder = isset($data['orderDetail']['preOrder']) ? strtotime($data['orderDetail']['preOrder']) : "0";
        
        if ($preOrder == "0" || $preOrder == "") {
            $orderTime = time();
        } else {
            $orderTime = $preOrder;
        }
        $zeroBasedDay = date('w', $orderTime);
        
        $loyaltyItems = array();
        $orderType = $data['orderDetail']['orderType'];
        foreach ($data['itemsDetail'] as $itemKey => $item) {
            
            $loyalty = isset($item['loyalty']) ? $item['loyalty'] : "0";
            
            $getItemDetail = $this->getItemDetail($clientId, $item['id'], $orderType, $zeroBasedDay);
            // print_r($getItemDetail);exit;
            if ($getItemDetail != false) {
                if ($loyalty == "0") {
                    # New menu price functionalities
                    if ($orderType == 0) {
                        $itemPrice = $getItemDetail['deliveryPrice'];
                    } else if ($orderType == 1) {
                        $itemPrice = $getItemDetail['collectionPrice'];
                    } else {
                        $itemPrice = $getItemDetail['itemPrice'];
                    }
                    $increaseType = $getItemDetail['fkPriceType'];
                    $percentageType = $getItemDetail['fkPercentageType'];
                    $extraPrice = $getItemDetail['fkPrice'];
                    $priceEnable = $getItemDetail['fkPriceStatus'];
                    $priceMode = $getItemDetail['fkPriceMode'];
                    $convertedPrice = $this->getItemPriceValue($itemPrice, $increaseType, $percentageType, $extraPrice, $priceEnable, $priceMode, $orderType);
                    $item['price'] = $convertedPrice;
                } else {
                    # New loyalty price
                    $item['price'] = "0.00";
                    $loyaltyItems[$item['id']] = str_replace(["<br>", "Â"], "", $getItemDetail['itemName']);
                }
            }
            $itemTotal = $item['price'] * $item['qty'];
            $subTotal += $itemTotal;

            if (!empty($item['addon']) && is_array($item['addon'])) {
                $addonCount = count($item['addon']);
                $i = 0;
                foreach ($item['addon'] as $addonKey => $addon) {
                    $getAddonDetail = $this->getAddonItemDetailWithId($clientId, $addon['id']);
                    if ($getAddonDetail != false) {
                        
                        $addonTotal = $getAddonDetail['price'] * $addon['qty'];
                        $subTotal += $addonTotal;
                        
                    }
                    $i++;
                }
            }
        }

        $data['chargesDetail']['subTotal'] = floatval($subTotal);


        $deliveryCharge = floatval($data['chargesDetail']['deliveryCharge']);
        $clientStuartCharge = floatval($data['chargesDetail']['clientStuartCharge']);
        $totalDeliveryCharge = floatval($deliveryCharge + $clientStuartCharge);
        $serviceCharge = floatval($orderCharges['serviceCharge']);
        $fkServiceCharge = floatval($orderCharges['fkServiceCharge']);
        $clientServiceCharge = floatval($orderCharges['clientServiceCharge']);
        $fkBaggageCharge = floatval($orderCharges['fkBaggageCharge']);
        $baggageCharge = floatval($orderCharges['baggageCharge']);
        $roundAmount = floatval($data['chargesDetail']['roundAmount']);
        $extraDonate = floatval($data['chargesDetail']['extraDonate']);
        $driverTip = floatval($data['chargesDetail']['driverTip']);
        $discount = floatval($data['chargesDetail']['discount']);
        $promo = floatval($data['chargesDetail']['promo']);
        $voucher = floatval($data['chargesDetail']['voucher']);
        $stripeCharge = floatval($stripeCharges->stripecharge);
        $stripeAdd = floatval($stripeCharges->stripeadd);

        $klarnaCharge = floatval($stripeCharges->klarnaStripeCharge);
        $klarnaAdd = floatval($stripeCharges->klarnaStripeAdd);

        $paypalStripeAdd = floatval($stripeCharges->paypalStripeAdd);
        $paypalStripeCharge = floatval($stripeCharges->paypalStripeCharge);

        $securityKey = $stripeCharges->securitykey;
        $connectId = $stripeCharges->connectid;
        $provider = $stripeCharges->provider;
        $stripeMode = $stripeCharges->live;

        $getStuartStatus = $this->getStuartStatus($getClientId->id);

        $SessionalOfferStatus = isset($data['chargesDetail']['sessionalOffer']) ? $data['chargesDetail']['sessionalOffer'] : "";
        $SessionalOfferTotal = isset($data['chargesDetail']['sessionalOffer']) ? $data['chargesDetail']['sessionalOfferTotal'] : "";
        $platform = isset($data['chargesDetail']['platform']) ? 1 : 0;

        if ($platform == 1 && $orderCharges['is_FkCharge'] == 2 && $orderCharges['is_FkCharge'] != 1) {
            $data['chargesDetail']['fkServiceCharge'] = floatval($orderCharges['FkCharge']);
            $serviceCharge = floatval($orderCharges['FkCharge']);
            $clientServiceCharge = floatval($orderCharges['ClientCharge']);
            $data['chargesDetail']['clientServiceCharge'] = floatval($orderCharges['ClientCharge']);
            $data['is_clientWebsite'] = 1;
        } else {
            $data['is_clientWebsite'] = 0;
        }

        # stripe calculations
        if ($SessionalOfferStatus == 1) {
            $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip) - ($SessionalOfferTotal)), 2);
        } else {
            $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip) - ($discount + $promo + $voucher)), 2);
        }

        if ($platform == 1 && $orderCharges['is_FkCharge'] == 2 && $orderCharges['is_FkCharge'] != 1) {
            if ($SessionalOfferStatus == 1) {
                $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip + $clientServiceCharge) - ($SessionalOfferTotal)), 2);
            } else {
                $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip + $clientServiceCharge) - ($discount + $promo + $voucher)), 2);
            }
        }

        if ($getStuartStatus && $getStuartStatus->status == "1") {
            $stripeCalculation = number_format($totalDeliveryCharge + $serviceCharge + $roundAmount + $extraDonate + $driverTip, 2);
        } else {
            $stripeCalculation = number_format($serviceCharge + $roundAmount + $extraDonate + $driverTip, 2);
        }

        $additionalPremiumCharges = 0;

        if ($data['orderDetail']['paymentType'] == 10) {
            $transferCalculation = number_format((($totalAmount - $stripeCalculation) * $klarnaCharge / 100 + $klarnaAdd + $stripeCalculation), 2);
        } else if ($data['orderDetail']['paymentType'] == 14) {
            $transferCalculation = number_format((($totalAmount - $stripeCalculation) * $paypalStripeCharge / 100 + $paypalStripeAdd + $stripeCalculation), 2);
        } else {
            $transferCalculation = number_format((($totalAmount - $stripeCalculation) * $stripeCharge / 100 + $stripeAdd + $stripeCalculation), 2);

            if(isset($stripeCharges->premiumstripecharge)) {
                $additionalStripeCharge = floatval($stripeCharges->premiumstripecharge);

                $additionalPremiumCharges = number_format((($totalAmount - $stripeCalculation) * $additionalStripeCharge / 100), 2);
            }
        }

        $transferCalculationString = str_replace(".", "", $transferCalculation);
        $totalAmountString = str_replace(".", "", $totalAmount);
        $clientTransferAmountString = $totalAmountString - $transferCalculationString;
        $clientTransferAmount = number_format($clientTransferAmountString / 100, 2);

        if ($SessionalOfferStatus == 1) {
            $totalAmount = number_format((($subTotal + $deliveryCharge + $serviceCharge + $baggageCharge + $roundAmount + $extraDonate + $driverTip) - ($discount + $promo + $voucher)), 2);
            $totalAmountString = str_replace(".", "", $totalAmount);
        }

        $data['chargesDetail']['totalAmount'] = $totalAmount;
        $data['chargesDetail']['clientTransferAmount'] = $clientTransferAmount;

        if ($data['userDetail']['email'])
            $description = $data['userDetail']['firstName'] . " " . $data['userDetail']['lastName'] . " " . $data['userDetail']['email'];
        else
            $description = $data['userDetail']['firstName'] . " " . $data['userDetail']['lastName'];

        $customerId = $data['userDetail']['customerId'];

        # wallet
        $tableOrder = isset($data['orderDetail']['isTableOrder']) ? $data['orderDetail']['isTableOrder'] : '';
        // print_r($data);exit;
        $placeOrder = $this->placeOrder($clientId, $data,true);
    
        $update_InCompleteOrder=$this->update_InCompleteOrder($inCompleteId,$placeOrder['orderId']);

        $data = array(
            "status" => true,
            "type" => "orderPlaced",
            "response" => $placeOrder
        );
        return $data;
             
    }


    public function getInCompleteOrder($id)
    {
        return $this->query("CALL  getInCompleteOrder($id);");        
    }

    public function getClientMenuBasedId($id)
    {
        return $this->query("CALL  getTblB2cBasedId($id)");  
    }

    public function update_InCompleteOrder($id,$order_id)
    {
        return $this->query("CALL  UpdateTblInCompleteOrders($order_id,$id)");
    }

   

}
