<?php

namespace App\Models;

use Stripe\Stripe;
use Stripe\Exception\ApiErrorException;

class Payment extends DB
{
    public function getCustomer($customerId)
    {
        return $this->query("CALL getCustomer($customerId)");
    }

    public function getCustomerStripeConnection($customerId)
    {
        $getCustomer = $this->getCustomer($customerId);
        $connection = $this->query("CALL getConnection()");

        if($getCustomer->isLive == 0)
        {
            # test user key
            $paymentDetail['skey'] = $connection->test_securitykey;
            $paymentDetail['live'] = 0;
            $paymentDetail['apiid'] = $connection->test_api_id;
            $paymentDetail['paymentMode'] = 0 ;  
        }
        else
        {
            # live user key           
            $paymentDetail['skey'] = $connection->security_key;
            $paymentDetail['live'] = $connection->live_status;
            $paymentDetail['apiid'] = $connection->api_id;  
            $paymentDetail['paymentMode'] = 1 ; 
        }
        return $paymentDetail;
    }

    public function payInvoice($stripeConnection, $invoice)
    {
        $response = array();
        # Set your Stripe API key
        $stripe = new \Stripe\StripeClient($stripeConnection);
        try 
        {
            $payInvoice = $stripe->invoices->pay(
                $invoice,
                []
            );
            $response['status'] = true;
            $response['data'] = $payInvoice;
            return $response;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function getWallet($customerId)
    {
        $getWallet = $this->query("CALL getWallet($customerId)");
        # wallet balance amount
        if (isset($getWallet) && isset($getWallet->amount)) {
            $walletAmount = $this->decrypt($getWallet->amount);
        } else {
            $walletAmount = 0;

            //if wallet not exist yet, create
            $data = $this->query("SELECT COUNT(*) AS count FROM `tbl_wallet` WHERE endCustomerId = $customerId");

            if (isset($data->count) && $data->count == 0) {
                $this->query("INSERT INTO `tbl_wallet`(`endCustomerId`, `amount`, `status`, `datetime`) VALUES ('$customerId','$walletAmount','1', '" . time() . "')");
            }
        }
        return $walletAmount;
    }

    public function updateWalletAmount($balanceAmount, $customerId)
    {
        $balanceAmount = $this->encrypt($balanceAmount);
        $this->query("CALL updateWalletAmount('$balanceAmount', $customerId)");
        return true;
    }

    public function getTransaction($customerId)
    {
        return $this->queryAll("CALL getTransaction($customerId)");
    }

    public function addTransaction(array $data): bool
    {
        $result = $this->query("INSERT INTO `tbl_wallet_transaction`(`clientId`, `endCustomerId`, `amount`, `type`, `status`, `datetime`) VALUES ('" . $data['client_id'] . "','" . $data['customer_id'] . "','" . $data['amount'] . "','" . $data['type'] . "','" . $data['status'] . "', '" . $data['datatime'] . "')");

        return $result ? true : false;
    }

    public function getSaveCard($customerId)
    {
        return $this->query("CALL getSaveCard($customerId)");
    }

    public function updatePrimarySaveCard($customerId, $fingerPrinter)
    {
        $this->query("CALL updatePrimarySaveCard($customerId, '$fingerPrinter')");
        return true;
    }

    public function createSavecard($customerId, $paymentCustomerId, $fingerPrinter, $isLive)
    {
        return $this->query("CALL createSavecard($customerId, '$paymentCustomerId', '$fingerPrinter', $isLive, @insert_id)");
    }

    public function updateSavecardCustomerId($customerId, $paymentCustomerId, $isLive)
    {
        return $this->query("CALL updateSavecardCustomerId($customerId, '$paymentCustomerId', $isLive)");
    }


    public function decrypt($value)
    {
        $c = base64_decode($value);
        $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
        $iv = substr($c, 0, $ivlen);
        $hmac = substr($c, $ivlen, $sha2len=32);
        $key = '123';
        $ciphertext_raw = substr($c, $ivlen+$sha2len);
        $original_plaintext = openssl_decrypt($ciphertext_raw, $cipher, $key, $options=OPENSSL_RAW_DATA, $iv);
        $calcmac = hash_hmac('sha256', $ciphertext_raw, $key, $as_binary=true);
        return $original_plaintext;
    }

    public function encrypt($savcd)
    {
        $plaintext = $savcd;
        $ivlen = openssl_cipher_iv_length($cipher="AES-128-CBC");
        $iv = openssl_random_pseudo_bytes($ivlen);
        $key = 123;
        $ciphertext_raw = openssl_encrypt($plaintext, $cipher, $key, $options=OPENSSL_RAW_DATA, $iv);
        $hmac = hash_hmac('sha256', $ciphertext_raw, $key, $as_binary=true);
        $ciphertext = base64_encode( $iv.$hmac.$ciphertext_raw );
        return $ciphertext;
    }

    public function stripePaymentMethod($getStripeConnection, $customerPaymentValue)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # Call the Stripe API to get a list of payment methods
            $customerCard = \Stripe\PaymentMethod::all(['customer' => $customerPaymentValue,'type' => 'card']);
            return $customerCard; 

        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function retrievePaymentMethod($getStripeConnection, $paymentMethodId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # Retrieve a payment method by ID
            $customerCard = \Stripe\PaymentMethod::retrieve($paymentMethodId);
            return $customerCard;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function detachPaymentMethod($getStripeConnection, $paymentMethodId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # Retrieve with detach a payment method by ID
            $customerCard = \Stripe\PaymentMethod::retrieve($paymentMethodId);
            $customerCard->detach();
            return $customerCard;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function createStripeCustomer($getStripeConnection, $description, $email, ?string $paymentMethodId = null, $client_id = null)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            $data = [
                "description" => $description,
                "email"=> $email
            ];

            if($paymentMethodId !== null) {
                $data["payment_method"] = $paymentMethodId;
                $data["invoice_settings"] = [
                    "default_payment_method" => $paymentMethodId
                ];
            }

            if($client_id) {
                $data["metadata"] = ['client_id' => $client_id];
            }

            $createCustomer = \Stripe\Customer::create($data);
            return $createCustomer;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    // public function attachPaymentMethodToCustomer($getStripeConnection, $paymentMethodId, $customerId)
    // {
    //     # Set your Stripe client
    //     $stripe = new \Stripe\StripeClient($getStripeConnection);

    //     try 
    //     {
    //         # Attach payment method with customer
    //         $attachPaymentMethod = $stripe->paymentMethods->attach($paymentMethodId, [
    //             'customer' => $customerId
    //         ]);

    //         $response['status'] = true;
    //         $response['data'] =  $attachPaymentMethod;
    //     } 
    //     catch (ApiErrorException $e) 
    //     {
    //         # Handle any errors from the Stripe API
    //         $response['status'] = false;
    //         $response['data'] = $e->getMessage();
    //     }

    //     return $response;
    // }

    public function createStripePrice($stripeConnection, $price, $type, $duration, $planName, $tax_behavior = 'exclusive', $currency_code = 'gbp')
    {
        $response = array();
        # Set your Stripe API key
        Stripe::setApiKey($stripeConnection);
        try 
        {
            $createPrice = \Stripe\Price::create(
                array(
                    "unit_amount" => $price, 
                    "currency"=> $currency_code,
                    "recurring"=> [
                        "interval" => $type, 
                        "interval_count" => $duration
                    ],
                    "product_data"=> [
                        "name" => $planName
                    ],
                    'tax_behavior' => $tax_behavior
                )
            );
            $response['status'] = true;
            $response['data'] = $createPrice;
            return $response;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function createStripeCustomerWithCard($getStripeConnection, $description, $email, $paymentMethodId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            $createCustomer = \Stripe\Customer::create([
                "description" => $description, 
                "email"=> $email,
                "payment_method" => $paymentMethodId
            ]);
            return $createCustomer;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function retrieveStripeCustomer($getStripeConnection, $customerId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            $customer = \Stripe\Customer::retrieve($customerId);
            return $customer;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function attachStripeCustomer($retrieveCard, $customerPaymentValue)
    {
        try 
        {
            $attach = $retrieveCard->attach(['customer' => $customerPaymentValue]);
            return $attach;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function transferRetrieve($getStripeConnection, $transferKey)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # Retrieve a transfer ID
            $transfer = \Stripe\Transfer::retrieve($transferKey);
            return $transfer;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function createReversal($getStripeConnection, $transferKey, $amount)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # Create transfer reversal
            $reversal = \Stripe\Transfer::createReversal($transferKey, ['amount' => $amount]);
            return $reversal;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function createTransfer($getStripeConnection, $amount, $connectid, $description, $currency_code = 'gbp')
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # Create transfer
            $createTransfer = \Stripe\Transfer::create(['amount' => $amount, 'currency' => $currency_code, 'destination' => $connectid, 'transfer_group' => $description]);
            return $createTransfer;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function createSubscription($stripeConnection, $customerId, $priceId, $clientTransferPct, $card, $destination, $default_tax_rates = null, $client_id = null)
    {
        $response = array();
        # Set your Stripe API key
        Stripe::setApiKey($stripeConnection);
        try 
        {
            $data = [
                "customer" => $customerId,
                "items" => [["price" => $priceId]],
                "payment_behavior" => "default_incomplete",
                "payment_settings" => [
                    "save_default_payment_method" => "on_subscription",
                    "payment_method_types" => ['card']
                ],
                "transfer_data" => [
                    "amount_percent" => $clientTransferPct,
                    "destination" => $destination,
                ],
                "default_payment_method" => $card,
                "default_tax_rates" => [
                        $default_tax_rates
                    ],
                "expand" => ["latest_invoice.payment_intent"],
            ];

            if($client_id) {
                $data["metadata"] = ['client_id' => $client_id];
            }

            $createSubscription = \Stripe\Subscription::create($data);
            
            $response['status'] = true;
            $response['data'] = $createSubscription;
            return $response;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function unSubscripe($stripeConnection, $subscription_id)
    {
        $response = array();

        # Set your Stripe API key
        $stripe = new \Stripe\StripeClient($stripeConnection);

        try {
            $unSubscription = $stripe->subscriptions->update(
                $subscription_id,
                ['cancel_at_period_end' => true]
            );

            $response['status'] = true;
            $response['data'] = $unSubscription;
            return $response;
        } catch (ApiErrorException $e) {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function getAllInvoice($stripeConnection, $subscription, $limit)
    {
        # Set your Stripe API key
        Stripe::setApiKey($stripeConnection);
        try 
        {
            $invoice = \Stripe\Invoice::all([
                'subscription' => $subscription,
                'limit' => $limit
            ]);
            return $invoice;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function createStripeCustomerWithAccount($getStripeConnection, $description, $email, $connectId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            $createCustomer = \Stripe\Customer::create(
                array(
                    "description" => $description, 
                    "email"=> $email
                ),
                array(
                    "stripe_account" => $connectId
                )
            );
            return $createCustomer;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function chargesRetrieve($getStripeConnection, $chargeKey)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # charge retrieve
            $charge = \Stripe\Charge::retrieve($chargeKey);

            return $charge;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function chargesUpdate($getStripeConnection, $destinationPayment, $description, $createStripeCustomerId, $connectId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # charge update
            $chargesUpdate = \Stripe\Charge::update(
                $destinationPayment, 
                [
                    'description' => $description,
                    'customer' =>$createStripeCustomerId
                ],[
                    "stripe_account" => $connectId
                ]
            );

            $response['status'] = true;
            $response['data'] = $chargesUpdate;
            return $response;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function createPaymentIntentWithCard($getStripeConnection, $connectId, $domain, $takeawayName, $totalAmount, $clientTransferAmount, $description, $paymentDetail, $customer, $currency_code = 'gbp')
    {
        $takeawayName = substr($takeawayName,0,22);
        if(!isset($customer->id))
        {
            $response['status'] = false;
            $response['data'] = $customer;
            return $response;
        }

        // if($paymentDetail['sva'] == 1) {
        //     $savecard = "off_session"; 
        // } else {
        //     $savecard = "on_session"; 
        // }

        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            if($paymentDetail['sva'] == 1) {
                # create payment intent
                $paymentIntent = \Stripe\PaymentIntent::create([
                    'payment_method' => $paymentDetail['paymentMethodId'],
                    'metadata' => [
                        'domain' => $domain,
                        'portal' => 'FK'
                    ],
                    'amount' => $totalAmount,
                    'statement_descriptor_suffix' => $takeawayName,
                    'description' => $description,
                    'currency' => $currency_code,
                    'automatic_payment_methods' => [
                        'enabled' => true,
                        'allow_redirects' => 'never'
                    ],
                    'setup_future_usage' => 'off_session',
                    'confirm' => true,
                    'capture_method' => 'automatic',
                    'on_behalf_of' => $connectId,
                    'transfer_data' => [
                        "amount" => $clientTransferAmount,
                        'destination' => $connectId,
                    ],
                ], [
                    "idempotency_key" => uniqid(),
                ]);
            } else {
                # create payment intent
                $paymentIntent = \Stripe\PaymentIntent::create([
                    'payment_method' => $paymentDetail['paymentMethodId'],
                    'metadata' => [
                        'domain' => $domain,
                        'portal' => 'FK'
                    ],
                    'amount' => $totalAmount,
                    'statement_descriptor_suffix' => $takeawayName,
                    'description' => $description,
                    'currency' => $currency_code,
                    'automatic_payment_methods' => [
                        'enabled' => true,
                        'allow_redirects' => 'never'
                    ],
                    'customer' => $customer->id,
                    'confirm' => true,
                    'capture_method' => 'automatic',
                    "on_behalf_of" => $connectId,
                    'transfer_data' => [
                        "amount" => $clientTransferAmount,
                        'destination' => $connectId,
                    ],
                ], [
                    "idempotency_key" => uniqid(),
                ]);
            }
            
            $response['status'] = true;
            $response['data'] = $paymentIntent;
            return $response;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function createPaymentIntentWithSaveCard($getStripeConnection, $connectId, $domain, $takeawayName, $totalAmount, $clientTransferAmount, $description, $customerKey, $currency_code = 'gbp')
    {
        $takeawayName = substr($takeawayName,0,22);
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # create payment intent for save card
            $paymentIntent = \Stripe\PaymentIntent::create([
                'metadata' => [
                    'domain' => $domain,
                    'portal' => 'FK SV'
                ],
                'amount' => $totalAmount,
                'statement_descriptor_suffix' => $takeawayName,
                'description' => $description,
                'currency' => $currency_code,
                'capture_method' => 'automatic',
                'automatic_payment_methods' => [
                    'enabled' => true,
                    'allow_redirects' => 'never'
                ],
                'setup_future_usage' => 'off_session',
                'on_behalf_of' => $connectId,
                'customer' => $customerKey,
                // 'payment_method_types' => ['card'],
                'transfer_data' => [
                    "amount" => $clientTransferAmount,
                    'destination' => $connectId,
                ],
            ], [
                "idempotency_key" => uniqid(),
            ]);

            $response['status'] = true;
            $response['data'] = $paymentIntent;
            return $response;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function createPaymentIntentWithApGp($getStripeConnection, $connectId, $domain, $takeawayName, $totalAmount, $clientTransferAmount, $description, $paymentDetail, $portal, $customer, $currency_code = 'gbp')
    {
        $takeawayName = substr($takeawayName,0,22);
        if(!isset($customer->id))
        {
            $response['status'] = false;
            $response['data'] = $customer;
            return $response;
        }
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            # create payment intent
            $paymentIntent = \Stripe\PaymentIntent::create([
                'payment_method' => $paymentDetail['paymentMethodId'],
                'metadata' => [
                    'domain' => $domain,
                    'integration_check' => 'accept_a_payment',
                    'portal' => 'FK '. $portal
                ],
                'amount' => $totalAmount,
                'statement_descriptor_suffix' => $takeawayName,
                'description' => $description,
                'currency' => $currency_code,
                'automatic_payment_methods' => [
                    'enabled' => true,
                    'allow_redirects' => 'never'
                ],
                'customer' => $customer->id,
                'confirm' => true,
                'capture_method' => 'automatic',
                "on_behalf_of" => $connectId,
                'transfer_data' => [
                    "amount" => $clientTransferAmount,
                    'destination' => $connectId,
                ],
            ], [
                "idempotency_key" => uniqid(),
            ]);

            $response['status'] = true;
            $response['data'] = $paymentIntent;
            return $response;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            $response['status'] = false;
            $response['data'] = $e->getMessage();
            return $response;
        }
    }

    public function retrieveBalance($getStripeConnection)
    {
        try {
            $stripe = new \Stripe\StripeClient($getStripeConnection);
            return $stripe->balance->retrieve([]);
        } catch (ApiErrorException $e) {
            return $e->getMessage();
        }
    }

    public function retrievePaymentIntent($getStripeConnection, $intentId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            $paymentIntent = \Stripe\PaymentIntent::retrieve($intentId);
            return $paymentIntent;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function confirmPaymentIntent($getStripeConnection, $intentId)
    {
        # Set your Stripe API key
        Stripe::setApiKey($getStripeConnection);
        try 
        {
            $paymentIntent = \Stripe\PaymentIntent::retrieve($intentId);
            $confirmedPaymentIntent = $paymentIntent->confirm();
            return $confirmedPaymentIntent;
        } 
        catch (ApiErrorException $e) 
        {
            # Handle any errors from the Stripe API
            return $e->getMessage();
        }
    }

    public function insertInCompletedOrder($data)
    {
       
            $itemsDetail=$data['itemsDetail'];
            $orderDetail=$data['orderDetail'];
            
            $paymentDetail=$data['paymentDetail'];
            $userDetail=$data['userDetail'];
            $chargesDetail=$data['chargesDetail'];
            
            $orderTime =$data['orderTime'];
            $incompletedOrder=$data['incompletedOrder'];
            $customerId=$data['customerId'];
            $logintype =$data['logintype'];
            $totalPrice =$data['totalPrice'];
            $client_id =$data['client_id'];
            $created_at=Date("Y-m-d H:i:s");
          
            // return $this->query("CALL insertInCompletedOrder($client_id,'$itemsDetail',$orderTime,$incompletedOrder,'$customerId',$logintype,$totalPrice,0,'$created_at','$created_at',@new_id)");
            return $this->query("CALL insertInCompletedOrderNew($client_id,'$itemsDetail','$orderDetail','$paymentDetail','$userDetail','$chargesDetail','$orderTime',$incompletedOrder,'$customerId',$logintype,$totalPrice,0,'$created_at','$created_at',@new_id)");
    }


}