<?php

namespace App\Models;

class Subscriber
{
    public ?int $id = null;
    public int $client_id;
    public int $user_id;
    public int $plan_id;
    public ?string $datetime = null;
    public ?bool $status = null;
    public ?bool $is_live = null;
    public object $subscriber;
    public Client $client;

    /**
     * Subscriber constructor.
     * 
     * @param int $client_id Client ID
     * @param int $user_id User ID (subscriber)
     * @param int $plan_id Plan ID
     */
    public function __construct(int $client_id, int $user_id, int $plan_id)
    {
        $this->client_id = $client_id;
        $this->user_id = $user_id;
        $this->plan_id = $plan_id;

        $this->client = new Client($this->client_id);

        if ($this->exists()) {
            $this->id = $this->subscriber->id;
            $this->datetime = $this->subscriber->datetime;
            $this->status = (bool) $this->subscriber->status;
            $this->is_live = (bool) $this->subscriber->is_live;
        }
    }

    /**
     * Check if the subscriber exists in the database.
     * 
     * @return bool
     */
    public function exists(): bool
    {
        $subscriber = $this->client->query(
            "SELECT * FROM `tbl_subscriberslist` WHERE `user_id` = $this->user_id AND `plan_id` = $this->plan_id;"
        );

        if (!empty($subscriber) && isset($subscriber->id)) {
            $this->subscriber = $subscriber;
            return true;
        }

        return false;
    }

    /**
     * Get details of the plan.
     * 
     * @return mixed The plan details or false if not found
     */
    public function getPlanDetails()
    {
        $plan = $this->client->query("SELECT * FROM `tbl_subscribeplan` WHERE `id` = $this->plan_id;");

        return $plan ?: false;
    }

    /**
     * Create a new subscription plan
     * 
     * @return bool The plan details or false if not found
     */
    public function createSubscriber()
    {
        $subscriber = $this->client->query("SELECT * FROM `tbl_subscriberslist` WHERE `user_id`=$this->user_id AND `plan_id`=$this->plan_id;");

        if ($subscriber === false) {
            $this->client->query("INSERT INTO `tbl_subscriberslist` (`user_id`, `plan_id`, `datetime`, `status`, `is_live`) VALUES ($this->user_id, $this->plan_id, '" . date('Y-m-d H:i:s') . "', '1', '1')");

            $subscriber = $this->client->query("SELECT * FROM `tbl_subscriberslist` WHERE `user_id`=$this->user_id AND `plan_id`=$this->plan_id;");
        } else {
            $this->client->query("UPDATE `tbl_subscriberslist` SET `plan_id`=$this->plan_id,`datetime`='" . date('Y-m-d H:i:s') . "',`status`='1',`is_live`='1' WHERE `id`=$subscriber->id AND `user_id`=$this->user_id");
        }

        return true;
    }

    /**
     * create subscriber details in table
     * 
     * @param array $data
     * 
     * @return bool
     */
    public function createSubscriberDetails($data): bool
    {
        $this->client->query("INSERT INTO `tbl_subscriptiondetails`(`user_id`, `amount`, `vat`, `currency`, `duration_type`, `duration_count`, `customer_id`, `plan_id`, `product_id`, `payment_method`, `client_secret`, `payment_intent`, `subscription_id`, `subscription_status`, `is_live`, `created_at`) VALUES ('" . $data['user_id'] . "','" . $data['amount'] . "','" . $data['vat'] . "','" . $data['currency'] . "','" . $data['duration_type'] . "','" . $data['duration_count'] . "','" . $data['customer_id'] . "','" . $data['plan_id'] . "','" . $data['product_id'] . "','" . $data['payment_method'] . "','" . $data['client_secret'] . "','" . $data['payment_intent'] . "','" . $data['subscription_id'] . "','" . $data['subscription_status'] . "','" . $data['is_live'] . "', '" . date('Y-m-d H:i:s') . "')");

        return true;
    }

    /**
     * update subscriber details in table
     * 
     * @param array $data
     * 
     * @return bool
     */
    public function updateSubscriberDetails($data): bool
    {
        $this->client->query("UPDATE `tbl_subscriptiondetails` SET `subscription_status`='" . $data['subscription_status'] . "',`is_live`='" . $data['is_live'] . "' WHERE `user_id`='" . $data['user_id'] . "' AND `customer_id`='" . $data['customer_id'] . "' AND `subscription_id`='" . $data['subscription_id'] . "'");

        return true;
    }

    /**
     * update a plan details
     * 
     * @param array $data
     * 
     * @return bool
     */
    public function updatePlanDetails(array $data): bool
    {
        if (!empty($data) && isset($this->plan_id)) {
            $query = "UPDATE tbl_subscribeplan SET ";
            $fields = [];

            foreach ($data as $key => $value) {
                $fields[] = "`$key` = '$value'";
            }

            $query .= implode(', ', $fields);
            $query .= " WHERE `id` = $this->plan_id";

            return $this->client->query($query);
        }

        return false;
    }

    /**
     * Check the customer is subscribed
     *
     * @return boolean
     */
    public function is_subscribed(): bool
    {
        $data = $this->client->query("SELECT * FROM tbl_subscriptiondetails WHERE `user_id` = $this->user_id AND `subscription_status`='paid' AND `created_at`<='" . date('Y-m-d H:i:s') . "' AND (`canceled_at` IS NULL OR `canceled_at`>='" . date('Y-m-d H:i:s') . "');");

        if (empty($data)) {
            return false;
        }

        return true;
    }

    /**
     * get all subscription Coupon Codes
     * @return array
     */
    public function getCouponCodes(): array
    {
        $delivery_coupons = $this->client->query("SELECT * FROM tbl_customer_coupon WHERE customer_id = " . $this->user_id . " AND `status` = 1 AND `order_type` IN (0, 2) AND `fromDate` <= " . time() . " AND `toDate` >= " . time());
        $collection_coupons = $this->client->query("SELECT * FROM tbl_customer_coupon WHERE customer_id = " . $this->user_id . " AND `status` = 1 AND `order_type` IN  (1, 2) AND `fromDate` <= " . time() . " AND `toDate` >= " . time());

        $wrapper = function ($coupon) {
            return [
                "id" => $coupon['id'],
                "customer_id" => $coupon['customer_id'],
                "name" => $coupon['name'],
                "code" => $coupon['code'],
                "discount" => $coupon['discount'],
                "type" => $coupon['type'],
                "fromDate" => date('Y-m-d H:i', $coupon['fromDate']),
                "toDate" => date('Y-m-d H:i', $coupon['toDate']),
                "status" => $coupon['status'],
                "order_type" => $coupon['order_type']
            ];
        };

        return [
            'delivery_coupons' => $delivery_coupons ? $wrapper((array) $delivery_coupons) : [],
            'collection_coupons' => $collection_coupons ? $wrapper((array) $collection_coupons) : []
        ];
    }

    /**
     * Add amount in subscriber's wallet
     * 
     * @param float $amount
     * @return void
     */
    public function addCredit(float $amount): void
    {
        $payment = new Payment();

        $total_balance = $payment->getWallet($this->user_id);
        $total_balance = ((float) $total_balance) + $amount;

        $payment->updateWalletAmount($total_balance, $this->user_id);
        $payment->addTransaction([
            'client_id' => $this->client_id,
            'customer_id' => $this->user_id,
            'amount' => $payment->encrypt($amount),
            'type' => 1,
            'status' => 1,
            'datatime' => time(),
        ]);
    }

    /**
     * get subscription information from database
     * 
     * @return array|int
     */
    public function getSubscription()
    {
        $data = $this->client->query("SELECT * FROM tbl_subscriptiondetails WHERE `user_id` = $this->user_id AND `subscription_status`='paid' ORDER BY id DESC;");

        if (isset($data->id)) {
            return (array) $data;
        } else {
            return 0;
        }
    }

    /**
     * Unsubscription
     * 
     * @param mixed $subscription_id
     * @param mixed $cancel_at
     * @return bool
     */
    public function unsubscribe($subscription_id, $cancel_at): bool
    {
        $result = $this->client->query("UPDATE tbl_subscriptiondetails SET canceled_at = '" . date('Y-m-d h:i:s', $cancel_at) . "' WHERE subscription_id = '$subscription_id'");

        if (isset($result->id)) {
            return true;
        }

        return false;
    }

    /**
     * Check is unsubscribed
     * 
     * @return bool
     */
    public function is_unsubscribe(): bool
    {
        $data = $this->getSubscription();

        if (isset($data['id'])) {
            return $data['canceled_at'] != null;
        }

        return false;
    }
}
